/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2005  Nicolas AGIUS                                     *
 *                                                                         *
 *   This file is part of ECaMM.                                           *
 *                                                                         *
 *   ECaMM is free software; you can redistribute it and/or modify it      *
 *   under the terms of the GNU General Public License as published by     *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   any later version.                                                    *
 *                                                                         *
 *   ECaMM is distributed in the hope that it will be useful, but          *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston,                *
 *    MA  02111-1307, USA.                                                 *
 *                                                                         *
 *   For more information about these matters, see the file named COPYING. *
 *                                                                         *
 *   Mail : nicolas_agius@yahoo.fr                                         *
 *                                                                         *
 ***************************************************************************/


package ecamm;

/**
 * Titre : AideIHM [ecamm]
 * Description : Fentre qui affiche des pages HTML
 * Copyright : GPL 2003 - 2004
 * @author Nicolas AGIUS
 * @version 1.2
 */

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.net.URL;
import javax.swing.event.*;

public class AideIHM extends JDialog
{
    //constante : page du dbut
    private static final String PAGE="help/index.html";
    private static final String ACTION_EXIT="exit";

    //composant graphiques
    private JPanel jPanel1 = new JPanel();
    private BorderLayout borderLayout1 = new BorderLayout();
    private JPanel jPanel2 = new JPanel();
    private JButton jButtonFermer = new JButton();
    private JScrollPane jScrollPane1 = new JScrollPane();
    private JEditorPane jEditorPane1 = new JEditorPane();

    public AideIHM()
    {
        try
        {
            jbInit();
        }
        catch(Exception e)
        {
            Tools.internalError(e,"",Main.cf.debug(),true);
        }

        //dimension et position de la fentre
        this.setSize(590,400);
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        Dimension frameSize = getSize();
        setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);

        //initialisation du texte
        jEditorPane1.setText("Chargement ...");
        this.setVisible(true);
        setURL(getClass().getResource(PAGE));
        jScrollPane1.requestFocusInWindow();
    }

    private void jbInit() throws Exception
    {
        this.setTitle("Aide de ECaMM");
        this.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        jPanel1.setLayout(borderLayout1);
        jButtonFermer.setText("Fermer");
        jButtonFermer.setMnemonic('f');
        jButtonFermer.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                jButtonFermer_actionPerformed(e);
            }
        });
        jEditorPane1.setEditorKit(null);
        jEditorPane1.setEditable(false);
        jEditorPane1.addHyperlinkListener(new javax.swing.event.HyperlinkListener()
        {
            public void hyperlinkUpdate(HyperlinkEvent e)
            {
                jEditorPane1_hyperlinkUpdate(e);
            }
        });
        this.getContentPane().add(jPanel1, BorderLayout.CENTER);
        this.getRootPane().setDefaultButton(jButtonFermer);
        jPanel1.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW)
            .put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE,0),ACTION_EXIT);
        jPanel1.getActionMap().put(ACTION_EXIT,new AbstractAction()
        {
            public void actionPerformed(ActionEvent e)
            {
                jButtonFermer_actionPerformed(e);
            }
        });

        jPanel1.add(jPanel2, BorderLayout.SOUTH);
        jPanel2.add(jButtonFermer, null);
        jPanel1.add(jScrollPane1, BorderLayout.CENTER);
        jScrollPane1.getViewport().add(jEditorPane1, null);
    }

    void jButtonFermer_actionPerformed(ActionEvent e)
    {
        this.dispose();
    }

    //changement de page si clic sur lien hypertext
    void jEditorPane1_hyperlinkUpdate(HyperlinkEvent e)
    {
        if(e.getEventType()==HyperlinkEvent.EventType.ACTIVATED)
            setURL(e.getURL());
    }

    //chargement de la page
    private void setURL(URL url)
    {
        try
        {
            jEditorPane1.setPage(url);
        }
        catch (Exception e)
        {
            JOptionPane.showMessageDialog(this,Tools.formatString(e.toString(),55),
                    "Lecture du fichier impossible",
                    JOptionPane.ERROR_MESSAGE);
        }
    }
}
