/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2005  Nicolas AGIUS                                     *
 *                                                                         *
 *   This file is part of ECaMM.                                           *
 *                                                                         *
 *   ECaMM is free software; you can redistribute it and/or modify it      *
 *   under the terms of the GNU General Public License as published by     *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   any later version.                                                    *
 *                                                                         *
 *   ECaMM is distributed in the hope that it will be useful, but          *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston,                *
 *    MA  02111-1307, USA.                                                 *
 *                                                                         *
 *   For more information about these matters, see the file named COPYING. *
 *                                                                         *
 *   Mail : nicolas_agius@yahoo.fr                                         *
 *                                                                         *
 ***************************************************************************/
package ecamm;

/**
 * Titre :  AjouterCTL [ECaMM]
 * Description : Classe de controle d'ajout de donnes
 * Copyright : GPL v2.0 - 2005~2006
 * @author : Nicolas AGIUS
 * @version 1.1
 */

/* CONSTANTES DE DONNEES
 *
 * Valeurs pour D_TYPE:
 * 1=Email
 * 2=Tlphone
 * 3=Direct
 * 4=Automatique
 *
 * Valeurs pour D_PRI:
 * 1=Haute
 * 2=Normale
 * 3=Basse
 *
 * Valeurs pour D_ETAT:
 * 1=A traiter
 * 2=En cours
 * 3=En attente
 * 4=Rsolut
 *
 */

import java.sql.SQLException;
import java.util.*;


public class AjouterCTL
{
    private boolean m_bDebug;
    private ConnectionBase m_cDB;

    public AjouterCTL(ConnectionBase maBase, boolean debug)
    {
        m_cDB=maBase;
        m_bDebug=debug;
    }

    public boolean addContact(String nom, String prenom, String mail, String telw, String telp, String orga, CircoDATA cir)
    {
        return addData("insert into CONTACT (CT_CI_ID,CT_NOM,CT_PRENOM,CT_MAIL,CT_TELW,CT_TELP,CT_ORGA) values ("
                       + cir.getID() + ",'" + Tools.protectString(nom) + "','" + Tools.protectString(prenom)
                       + "','"+Tools.protectString(mail) + "','"+Tools.protectString(telw) + "','"
                       + Tools.protectString(telp) + "','" + Tools.protectString(orga)+"')");
    }

    public boolean updateContact(ContactDATA ct,String nom, String prenom, String mail, String telw, String telp, String orga, CircoDATA cir)
    {
	return addData("update CONTACT set CT_CI_ID=" + cir.getID() + ",CT_NOM='" + Tools.protectString(nom)
                       + "',CT_PRENOM='"+Tools.protectString(prenom) + "',CT_MAIL='"+Tools.protectString(mail)
                       + "',CT_TELW='" + Tools.protectString(telw) + "',CT_TELP='" + Tools.protectString(telp)
                       + "',CT_ORGA='" + Tools.protectString(orga) + "' where CT_ID=" + ct.getID());
    }

    public boolean addEtab(String nom, String rne, String slis, CommuneDATA com)
    {
        return addData("insert into ETABLISSEMENT (ET_CO_ID,ET_NOM,ET_RNE,ET_SLIS) values (" + com.getID()
                       + ",'" + Tools.protectString(nom) + "','" + Tools.protectString(rne)
                       + "','" + Tools.protectString(slis)+"')");
    }

    public boolean updateEtab(EtabDATA et,String nom, String rne, String slis, CommuneDATA com)
    {
	return addData("update ETABLISSEMENT set ET_CO_ID=" + com.getID() + ",ET_NOM='"
                       + Tools.protectString(nom) + "',ET_RNE='" + Tools.protectString(rne)
                       + "',ET_SLIS='" + Tools.protectString(slis) + "' where ET_ID=" + et.getID());
    }

    public boolean addDemande(String prob,String rem,int type,int priorite,
                              Vector etabs,Vector contacts,boolean dateAuto,Date date)
    {
        String sqlDate;
        if(dateAuto)
        {
            sqlDate = "NOW()";
        }
        else    //date d'enregistrement en manuel
        {
            GregorianCalendar cal = new GregorianCalendar();
            cal.setTime(date);

            //heure  00:00
            sqlDate = "makedate("+cal.get(Calendar.YEAR)+","+cal.get(Calendar.DAY_OF_YEAR)+")";
        }


        //CAT_ID , FLAG et ETAT par default
        if(!addData("insert into DEMANDE (D_U_IDO,D_PROB,D_SOL,D_TYPE,D_PRI,D_DATEO) values ("
                    + Main.cf.getUserID() + ",'"+Tools.protectString(prob) + "','"
                    + Tools.protectString(rem) + "'," + type + "," + priorite + "," + sqlDate +")"))
        {
            return false;
        }

        for(int i=0;i<etabs.size();i++)
        {
            if(!addData("insert into ETD (ETD_ET_ID,ETD_D_ID) values ("+((EtabDATA)etabs.get(i)).getID()+",LAST_INSERT_ID())"))
                return false;
        }

        for(int i=0;i<contacts.size();i++)
        {
            if(!addData("insert into CTD (CTD_CT_ID,CTD_D_ID) values ("+((ContactDATA)contacts.get(i)).getID()+",LAST_INSERT_ID())"))
                return false;
        }

        return true;
    }

    //enregistrement des donnes
    private boolean addData(String req)
    {
        try
        {
            m_cDB.ExecuteUpdate(req);
        }
        catch(SQLException e)
        {
            Tools.sqlError(e,m_bDebug);
            return false;

        }
        catch(Exception e)
        {
            Tools.internalError(e,"",m_bDebug,false);
            return false;
        }
        return true;
    }
}
