/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2005  Nicolas AGIUS                                     *
 *                                                                         *
 *   This file is part of ECaMM.                                           *
 *                                                                         *
 *   ECaMM is free software; you can redistribute it and/or modify it      *
 *   under the terms of the GNU General Public License as published by     *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   any later version.                                                    *
 *                                                                         *
 *   ECaMM is distributed in the hope that it will be useful, but          *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston,                *
 *    MA  02111-1307, USA.                                                 *
 *                                                                         *
 *   For more information about these matters, see the file named COPYING. *
 *                                                                         *
 *   Mail : nicolas_agius@yahoo.fr                                         *
 *                                                                         *
 ***************************************************************************/
package ecamm;

/**
 * Titre :  AjouterContactIHM [ECaMM]
 * Description : Fentre d'ajout d'un contact (permet aussi l'dition)
 * Copyright : GPL v2.0 - 2005
 * @author : Nicolas AGIUS
 * @version 1.0
 */

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.border.*;

public class AjouterContactIHM extends JFrame
{
    //Constantes pour le keybinding
    private static final String ACTION_SAVE="save";
    private static final String ACTION_CANCEL="cancel";
    private static final String ACTION_OPENBOX = "openpopup";

    //variable de traitement
    private ConfigurationCTL cf=Main.cf;
    private RechercherCTL m_ctlRechercher;
    private AjouterCTL m_ctlAjouter;
    private boolean m_bEditMode;
    private ContactDATA m_cContact;     //contact  diter
    private CreerIHM m_ihmCalledFrom;   //ihm appelante (utilis pour notification ajout contact)

    //composant graphiques
    private JTextField jTextFieldTelP = new JTextField();
    private JButton jButtonSave = new JButton();
    private JTextField jTextFieldPrenom = new JTextField();
    private JTextField jTextFieldOrga = new JTextField();
    private JTextField jTextFieldNom = new JTextField();
    private JLabel jLabel7 = new JLabel();
    private JComboBox jComboBoxCirco = new JComboBox();
    private JTextField jTextFieldTelW = new JTextField();
    private JLabel jLabel6 = new JLabel();
    private JLabel jLabel5 = new JLabel();
    private JTextField jTextFieldMail = new JTextField();
    private JLabel jLabel4 = new JLabel();
    private JLabel jLabel3 = new JLabel();
    private JLabel jLabel2 = new JLabel();
    private JPanel jPanelContact = new JPanel();
    private JLabel jLabel1 = new JLabel();
    private JPanel jPanel1 = new JPanel();
    private JButton jButtonCancel = new JButton();
    private JPanel jPanel2 = new JPanel();
    private TitledBorder titledBorder1;
    private JLabel jLabel8 = new JLabel();
    private JComboBox jComboBoxCommune = new JComboBox();
    private GridBagLayout gridBagLayout1 = new GridBagLayout();
    private GridBagLayout gridBagLayout2 = new GridBagLayout();

    public AjouterContactIHM()
    {
        cf.registerIHM(this);

        //Classe de controle mutualise
        m_ctlRechercher=cf.getRechercherCTL();
        m_ctlAjouter=cf.getAjouterCTL();

        try
        {
            jbInit();
            keyBindingInit();
        }
        catch(Exception e)
        {
            Tools.internalError(e,"",cf.debug(),true);
        }

        //remplissage de la combo box circo
        CircoDATA circos[]= m_ctlRechercher.getCircos();
        jComboBoxCirco.addItem(new CircoDATA());
        for(int i=0;i<circos.length;i++)
            jComboBoxCirco.addItem(circos[i]);

        //remplissage de la combo box communes
        CommuneDATA communes[]= m_ctlRechercher.getCommunes();
        jComboBoxCommune.addItem(new CommuneDATA());
        for(int i=0;i<communes.length;i++)
            jComboBoxCommune.addItem(communes[i]);

        //dimensionnnement
        pack();
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        Dimension frameSize = getSize();
        setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);
    }

    private void jbInit() throws Exception
    {
        titledBorder1 = new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134))," Circonscription : ");
        jPanelContact.setLayout(gridBagLayout1);
        jLabel2.setText("Prenom :");
        jLabel3.setText("E-Mail :");
        jLabel4.setText("Tel. Bureau :");
        jLabel5.setText("Tel. portable :");
        jLabel6.setText("Organisation :");
        jLabel7.setText("Nom :");
        jComboBoxCirco.setPreferredSize(new Dimension(240,19));
        jButtonSave.setText("Enregistrer");
        jButtonSave.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                jButtonSave_actionPerformed(e);
            }
        });
        jLabel1.setText("Nom :");
        jButtonCancel.setText("Annuler");
        jButtonCancel.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                jButtonCancel_actionPerformed(e);
            }
        });
        jPanel2.setBorder(titledBorder1);
        jPanel2.setLayout(gridBagLayout2);
        jLabel8.setText("Rechercher la circo par une commune :");
        jComboBoxCommune.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                jComboBoxCommune_actionPerformed(e);
            }
        });
        this.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
        this.addWindowListener(new java.awt.event.WindowAdapter()
        {
            public void windowClosing(WindowEvent e)
            {
                this_windowClosing(e);
            }
        });
        this.getContentPane().add(jPanelContact, BorderLayout.CENTER);
        jPanelContact.add(jLabel2, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(6, 6, 0, 0), 46, 4));
        jPanelContact.add(jLabel4, new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(6, 6, 0, 0), 27, 4));
        jPanelContact.add(jLabel3, new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(6, 6, 0, 0), 56, 4));
        jPanelContact.add(jLabel5, new GridBagConstraints(0, 4, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(6, 6, 0, 0), 22, 4));
        jPanelContact.add(jLabel6, new GridBagConstraints(0, 5, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(6, 6, 0, 0), 19, 4));
        jPanelContact.add(jTextFieldNom, new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(6, 0, 0, 6), 0, 0));
        jPanelContact.add(jTextFieldMail, new GridBagConstraints(1, 2, 1, 1, 1.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(6, 0, 0, 6), 0, 0));
        jPanelContact.add(jTextFieldTelW, new GridBagConstraints(1, 3, 1, 1, 1.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(6, 0, 0, 6), 0, 0));
        jPanelContact.add(jTextFieldTelP, new GridBagConstraints(1, 4, 1, 1, 1.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(6, 0, 0, 6), 0, 0));
        jPanelContact.add(jTextFieldOrga, new GridBagConstraints(1, 5, 1, 1, 1.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(6, 0, 0, 6), 0, 0));
        jPanelContact.add(jLabel1, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(6, 6, 0, 0), 63, 4));
        jPanelContact.add(jTextFieldPrenom, new GridBagConstraints(1, 1, 1, 1, 1.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(6, 0, 0, 6), 0, 0));
        jPanelContact.add(jPanel2, new GridBagConstraints(0, 6, 2, 1, 1.0, 0.0
            ,GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL, new Insets(6, 6, 0, 6), 0, 0));
        jPanel2.add(jComboBoxCirco, new GridBagConstraints(0, 1, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(6, 32, 0, 6), 0, 0));
        jPanel2.add(jLabel7, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(6, 6, 0, 25), 0, 0));
        jPanel2.add(jLabel8, new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(12, 6, 0, 0), 0, 0));
        jPanel2.add(jComboBoxCommune, new GridBagConstraints(0, 3, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(6, 32, 12, 6), 0, 0));
        jPanelContact.add(jPanel1, new GridBagConstraints(0, 7, 2, 1, 1.0, 1.0
            ,GridBagConstraints.SOUTH, GridBagConstraints.HORIZONTAL, new Insets(6, 0, 0, 0), 0, 0));
        jPanel1.add(jButtonSave, null);
        jPanel1.add(jButtonCancel, null);
	jTextFieldNom.requestFocusInWindow();
    }

    private void keyBindingInit() throws Exception
    {
        //Keys bindings gnraux
        InputMap inputMap=jPanelContact.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW);
        ActionMap actionMap=jPanelContact.getActionMap();
        inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER,0),ACTION_SAVE);
        inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE,0),ACTION_CANCEL);
        actionMap.put(ACTION_SAVE,new AbstractAction()
        {
            public void actionPerformed(ActionEvent e)
            {
                jButtonSave_actionPerformed(e);
            }
        });
        actionMap.put(ACTION_CANCEL,new AbstractAction()
        {
            public void actionPerformed(ActionEvent e)
            {
                jButtonCancel_actionPerformed(e);
            }
        });

	//key bindings comboBox
	jComboBoxCommune.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_SPACE,0),ACTION_OPENBOX);
	jComboBoxCommune.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER,0),ACTION_SAVE);
	jComboBoxCommune.getActionMap().put(ACTION_SAVE,new AbstractAction()
	{
	    public void actionPerformed(ActionEvent e)
	    {
		jButtonSave_actionPerformed(e);
	    }
	});
	jComboBoxCommune.getActionMap().put(ACTION_OPENBOX,new AbstractAction()
	{
	    public void actionPerformed(ActionEvent e)
	    {
		jComboBoxCommune.showPopup();
	    }
	});
	jComboBoxCirco.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_SPACE,0),ACTION_OPENBOX);
	jComboBoxCirco.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER,0),ACTION_SAVE);
	jComboBoxCirco.getActionMap().put(ACTION_SAVE,new AbstractAction()
	{
	    public void actionPerformed(ActionEvent e)
	    {
		jButtonSave_actionPerformed(e);
	    }
	});
	jComboBoxCirco.getActionMap().put(ACTION_OPENBOX,new AbstractAction()
	{
	    public void actionPerformed(ActionEvent e)
	    {
		jComboBoxCirco.showPopup();
	    }
	});
    }

    //pop-up de confirmation
    private boolean confirm()
    {
        return JOptionPane.showConfirmDialog(this,"Etes vous sr de vouloir enregistrer\nces donnes ?",
                        "Confirmation de mise  jour",
                        JOptionPane.YES_NO_OPTION,
                        JOptionPane.WARNING_MESSAGE)==JOptionPane.YES_OPTION;
    }

    //vide les champs de saisie
    private void clear()
    {
        m_ihmCalledFrom=null;
	jTextFieldNom.setText("");
	jTextFieldMail.setText("");
	jTextFieldPrenom.setText("");
	jTextFieldTelW.setText("");
	jTextFieldTelP.setText("");
	jTextFieldOrga.setText("");
	jComboBoxCirco.setSelectedIndex(0);
	jComboBoxCommune.setSelectedIndex(0);
    }

    void jButtonCancel_actionPerformed(ActionEvent e)
    {
        super.setVisible(false);
	clear();
    }

    void this_windowClosing(WindowEvent e)
    {
        jButtonCancel_actionPerformed(null);
    }

    //slection automatique de la circonscription
    public void jComboBoxCommune_actionPerformed(ActionEvent e)
    {
        ComboBoxModel model =jComboBoxCirco.getModel();

        CircoDATA cir=((CommuneDATA)jComboBoxCommune.getSelectedItem()).getCirco();
        if(!cir.isVide())
        {
            int i;
            for(i=0;i<model.getSize() && !cir.equals(model.getElementAt(i));i++);
            jComboBoxCirco.setSelectedIndex(i);
        }
    }

    public void jButtonSave_actionPerformed(ActionEvent e)
    {
        //controle des champs obligatoires
        String nom=jTextFieldNom.getText();
        if(nom.length()<=0)
        {
            JOptionPane.showMessageDialog(this,"Vous devez saisir le nom.",
               "Erreur de saisie",JOptionPane.ERROR_MESSAGE);
            return;
        }

        CircoDATA cir=(CircoDATA)jComboBoxCirco.getSelectedItem();
        if(cir.isVide())
        {
            JOptionPane.showMessageDialog(this,"Vous devez slectionner la circonscription.",
               "Erreur de saisie",JOptionPane.ERROR_MESSAGE);
            return;
        }

        if(confirm())
        {
	    if(m_bEditMode) //edition
	    {
		if(m_ctlAjouter.updateContact(m_cContact,nom,jTextFieldPrenom.getText(),jTextFieldMail.getText(),
					   jTextFieldTelW.getText(),jTextFieldTelP.getText(),
					   jTextFieldOrga.getText(),cir))
		{
		    //vidage et fermeture de la fentre
		    jButtonCancel_actionPerformed(null);
		}
	    }
	    else            //insertion
	    {
		if(m_ctlAjouter.addContact(nom,jTextFieldPrenom.getText(),jTextFieldMail.getText(),
					   jTextFieldTelW.getText(),jTextFieldTelP.getText(),
					   jTextFieldOrga.getText(),cir))
		{
		    //vidage et fermeture de la fentre
                    if(m_ihmCalledFrom!=null) m_ihmCalledFrom.notifyNewContact();
		    jButtonCancel_actionPerformed(null);
		}
	    }
        }
    }

    //ouverture en mode Ajout normal
    public void setVisible(boolean b)
    {
        this.setTitle("Nouveau contact");
	clear();
	super.setVisible(b);
	jTextFieldNom.requestFocusInWindow();
	m_bEditMode=false;
    }

    //ouverture en mode Ajout avec notification d'enregistrement
    public void setVisible(CreerIHM calledFrom)
    {
        setVisible(true);
        m_ihmCalledFrom=calledFrom;
    }

    //ouverture en mode Edition
    public void edit(ContactDATA ct)
    {
	//remplissage des lments
	jTextFieldNom.setText(ct.getNom());
	jTextFieldPrenom.setText(ct.getPrenom());
	jTextFieldMail.setText(ct.getMail());
	jTextFieldTelW.setText(ct.getTelW());
	jTextFieldTelP.setText(ct.getTelP());
	jTextFieldOrga.setText(ct.getOrga());

	//slection de la Circo
	ComboBoxModel model =jComboBoxCirco.getModel();
	CircoDATA cir=ct.getCirco();
	if(!cir.isVide())
	{
	    int i;
	    for(i=0;i<model.getSize() && !cir.equals(model.getElementAt(i));i++);
	    jComboBoxCirco.setSelectedIndex(i);
	}
	jComboBoxCommune.setSelectedIndex(0);

	//Activation de la fentre
	m_bEditMode=true;
	m_cContact=ct;
        this.setTitle("Edition du contact");
	super.setVisible(true);
	jTextFieldNom.requestFocusInWindow();
    }
}
