/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2005  Nicolas AGIUS                                     *
 *                                                                         *
 *   This file is part of ECaMM.                                           *
 *                                                                         *
 *   ECaMM is free software; you can redistribute it and/or modify it      *
 *   under the terms of the GNU General Public License as published by     *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   any later version.                                                    *
 *                                                                         *
 *   ECaMM is distributed in the hope that it will be useful, but          *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston,                *
 *    MA  02111-1307, USA.                                                 *
 *                                                                         *
 *   For more information about these matters, see the file named COPYING. *
 *                                                                         *
 *   Mail : nicolas_agius@yahoo.fr                                         *
 *                                                                         *
 ***************************************************************************/
package ecamm;

/**
 * Titre :  AjouterEtabIHM [ECaMM]
 * Description : Fentre d'ajout d'un tablissement (permet aussi l'dition)
 * Copyright : GPL v2.0 - 2005
 * @author : Nicolas AGIUS
 * @version 1.0
 */

import javax.swing.*;
import java.awt.event.*;
import java.awt.*;

public class AjouterEtabIHM extends JFrame
{
    //Constantes pour le keybinding
    private static final String ACTION_SAVE="save";
    private static final String ACTION_CANCEL="cancel";
    private static final String ACTION_OPENBOX = "openpopup";

    //variable de traitement
    private ConfigurationCTL cf=Main.cf;
    private RechercherCTL m_ctlRechercher;
    private AjouterCTL m_ctlAjouter;
    private boolean m_bEditMode;
    private EtabDATA m_eEtab;     //etab  diter
    private CreerIHM m_ihmCalledFrom;   //ihm appelante (utilis pour notification ajout etablissement)

    //composant graphiques
    private JPanel jPanel1 = new JPanel();
    private JTextField jTextFieldRNE = new JTextField();
    private JButton jButtonCancel = new JButton();
    private JButton jButtonSave = new JButton();
    private JLabel jLabel11 = new JLabel();
    private JTextField jTextFieldSlis = new JTextField();
    private JLabel jLabel10 = new JLabel();
    private JLabel jLabel9 = new JLabel();
    private JLabel jLabel8 = new JLabel();
    private JTextField jTextFieldNom = new JTextField();
    private FlowLayout flowLayout1 = new FlowLayout();
    private JComboBox jComboBoxCommune = new JComboBox();
    private JPanel jPanel2 = new JPanel();
    private GridBagLayout gridBagLayout1 = new GridBagLayout();

    public AjouterEtabIHM()
    {
        cf.registerIHM(this);

        //Classe de controle mutualise
        m_ctlRechercher=cf.getRechercherCTL();
        m_ctlAjouter=cf.getAjouterCTL();

        try
        {
            jbInit();
            keyBindingInit();
        }
        catch(Exception e)
        {
            Tools.internalError(e,"",cf.debug(),true);
        }

        //remplissage de la combo box communes
        CommuneDATA communes[]= m_ctlRechercher.getCommunes();
        jComboBoxCommune.addItem(new CommuneDATA());
        for(int i=0;i<communes.length;i++)
            jComboBoxCommune.addItem(communes[i]);

        //dimensionnement
        pack();
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        Dimension frameSize = getSize();
        setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);
    }

    private void jbInit() throws Exception
    {
        jPanel1.setLayout(flowLayout1);
        jComboBoxCommune.setPreferredSize(new Dimension(210,19));
        jButtonCancel.setText("Annuler");
        jButtonCancel.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                jButtonCancel_actionPerformed(e);
            }
        });
        jButtonSave.setText("Enregistrer");
        jButtonSave.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                jButtonSave_actionPerformed(e);
            }
        });
        jLabel11.setText("Commune :");
        jLabel10.setText("Nom du SLIS :");
        jLabel9.setText("RNE :");
        jLabel8.setText("Nom complet :");
        jPanel2.setLayout(gridBagLayout1);
        this.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
        this.addWindowListener(new java.awt.event.WindowAdapter()
        {
            public void windowClosing(WindowEvent e)
            {
                this_windowClosing(e);
            }
        });
        this.getContentPane().add(jPanel2, BorderLayout.CENTER);
        jPanel2.add(jPanel1, new GridBagConstraints(0, 4, 2, 1, 1.0, 1.0
            ,GridBagConstraints.SOUTH, GridBagConstraints.HORIZONTAL, new Insets(12, 0, 6, 0), 0, 0));
        jPanel1.add(jButtonSave, null);
        jPanel1.add(jButtonCancel, null);
        jPanel2.add(jLabel9, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(6, 6, 0, 0), 0, 4));
        jPanel2.add(jLabel10, new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(6, 6, 0, 0), 0, 4));
        jPanel2.add(jLabel11, new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(6, 6, 0, 0), 0, 4));
        jPanel2.add(jTextFieldNom, new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(6, 6, 0, 6), 0, 0));
        jPanel2.add(jTextFieldRNE, new GridBagConstraints(1, 1, 1, 1, 1.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(6, 6, 0, 6), 0, 0));
        jPanel2.add(jTextFieldSlis, new GridBagConstraints(1, 2, 1, 1, 1.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(6, 6, 0, 6), 0, 0));
        jPanel2.add(jComboBoxCommune, new GridBagConstraints(1, 3, 1, 1, 1.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(6, 6, 0, 6), 0, 0));
        jPanel2.add(jLabel8, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(6, 6, 0, 0), 0, 4));
    }

    private void keyBindingInit() throws Exception
    {
        //Keys bindings gnraux
        InputMap inputMap=jPanel2.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW);
        ActionMap actionMap=jPanel2.getActionMap();
        inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER,0),ACTION_SAVE);
        inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE,0),ACTION_CANCEL);
        actionMap.put(ACTION_SAVE,new AbstractAction()
        {
            public void actionPerformed(ActionEvent e)
            {
                jButtonSave_actionPerformed(e);
            }
        });
        actionMap.put(ACTION_CANCEL,new AbstractAction()
        {
            public void actionPerformed(ActionEvent e)
            {
                jButtonCancel_actionPerformed(e);
            }
        });

	//key bindings comboBox
	jComboBoxCommune.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_SPACE,0),ACTION_OPENBOX);
	jComboBoxCommune.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER,0),ACTION_SAVE);
	jComboBoxCommune.getActionMap().put(ACTION_SAVE,new AbstractAction()
	{
	    public void actionPerformed(ActionEvent e)
	    {
		jButtonSave_actionPerformed(e);
	    }
	});
	jComboBoxCommune.getActionMap().put(ACTION_OPENBOX,new AbstractAction()
	{
	    public void actionPerformed(ActionEvent e)
	    {
		jComboBoxCommune.showPopup();
	    }
	});
    }

    //pop-up de confirmation
    private boolean confirm()
    {
        return JOptionPane.showConfirmDialog(this,"Etes vous sr de vouloir enregistrer\nces donnes ?",
                        "Confirmation de mise  jour",
                        JOptionPane.YES_NO_OPTION,
                        JOptionPane.WARNING_MESSAGE)==JOptionPane.YES_OPTION;
    }

    //vide les champs de saisie
    private void clear()
    {
        m_ihmCalledFrom=null;
	jTextFieldNom.setText("");
	jTextFieldRNE.setText("");
	jTextFieldSlis.setText("");
	jComboBoxCommune.setSelectedIndex(0);
    }

    void jButtonCancel_actionPerformed(ActionEvent e)
    {
        super.setVisible(false);
	clear();
    }

    void this_windowClosing(WindowEvent e)
    {
        jButtonCancel_actionPerformed(null);
    }

    public void jButtonSave_actionPerformed(ActionEvent e)
    {
        //controle des champs obligatoires
        String nom=jTextFieldNom.getText();
        if(nom.length()<=0)
        {
            JOptionPane.showMessageDialog(this,"Vous devez saisir le nom.",
               "Erreur de saisie",JOptionPane.ERROR_MESSAGE);
            return;
        }

        CommuneDATA com=(CommuneDATA)jComboBoxCommune.getSelectedItem();
        if(com.isVide())
        {
            JOptionPane.showMessageDialog(this,"Vous devez slectionner la commune.",
               "Erreur de saisie",JOptionPane.ERROR_MESSAGE);
            return;
        }

        if(confirm())
        {
	    if(m_bEditMode) //edition
	    {
		if(m_ctlAjouter.updateEtab(m_eEtab,nom,jTextFieldRNE.getText(),jTextFieldSlis.getText(),com))
		{
		    //vidage et fermeture de la fentre
		    jButtonCancel_actionPerformed(null);
		}
	    }
	    else                 //insertion
	    {
		if(m_ctlAjouter.addEtab(nom,jTextFieldRNE.getText(),jTextFieldSlis.getText(),com))
		{
		    //vidage et fermeture de la fentre
                    if(m_ihmCalledFrom!=null) m_ihmCalledFrom.notifyNewEtab();
		    jButtonCancel_actionPerformed(null);
		}
	    }
        }
    }

    //ouverture en mode Ajout
    public void setVisible(boolean b)
    {
        this.setTitle("Nouvel tablissement");
	clear();
	super.setVisible(b);
	jTextFieldNom.requestFocusInWindow();
	m_bEditMode=false;
    }

    //ouverture en mode Ajout avec notification d'enregistrement
    public void setVisible(CreerIHM calledFrom)
    {
        setVisible(true);
        m_ihmCalledFrom=calledFrom;
    }

    //ouverture en mode Edition
    public void edit(EtabDATA et)
    {
	//remplissage des lments
	jTextFieldNom.setText(et.getNom());
	jTextFieldRNE.setText(et.getRNE());
	jTextFieldSlis.setText(et.getSlis());

	//slection de la Circo
	ComboBoxModel model =jComboBoxCommune.getModel();
	CommuneDATA com=et.getCommune();
	if(!com.isVide())
	{
	    int i;
	    for(i=0;i<model.getSize() && !com.equals(model.getElementAt(i));i++);
	    jComboBoxCommune.setSelectedIndex(i);
	}

	//Activation de la fentre
	m_bEditMode=true;
	m_eEtab=et;
	this.setTitle("Edition de l'tablissement");
	super.setVisible(true);
	jTextFieldNom.requestFocusInWindow();
    }
}
