/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2005  Nicolas AGIUS                                     *
 *                                                                         *
 *   This file is part of ECaMM.                                           *
 *                                                                         *
 *   ECaMM is free software; you can redistribute it and/or modify it      *
 *   under the terms of the GNU General Public License as published by     *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   any later version.                                                    *
 *                                                                         *
 *   ECaMM is distributed in the hope that it will be useful, but          *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston,                *
 *    MA  02111-1307, USA.                                                 *
 *                                                                         *
 *   For more information about these matters, see the file named COPYING. *
 *                                                                         *
 *   Mail : nicolas_agius@yahoo.fr                                         *
 *                                                                         *
 ***************************************************************************/
package ecamm;

/**
 * Titre :  ArchiveDATA [ECaMM]
 * Description : Classe de donnes Archives
 * Copyright : GPL v2.0 - 2005
 * @author : Nicolas AGIUS
 * @version 1.0
 */

import java.sql.SQLException;
import java.sql.ResultSet;
import java.util.Vector;

public class ArchiveDATA extends Data
{
    private boolean m_bFull; //si Etab et Contacts remplits

    private int m_iType;
    private int m_iNumDossier;
    private CategorieDATA m_dCategorie;
    private String m_sUserO;
    private String m_sUserC;
    private String m_sProb;
    private String m_sSol;
    private String m_sDateO;
    private String m_sDateC;

    private Vector m_vContacts=new Vector();
    private Vector m_vEtabs=new Vector();

    public ArchiveDATA()
    {
	p_iID=NONE;
	m_iType=NONE;
        m_iNumDossier=0;
	m_dCategorie=new CategorieDATA();
	m_sUserO="";
	m_sUserC="";
	m_sProb="";
	m_sSol="";
	m_sDateO="";
	m_sDateC="";
	m_bFull=false;
    }

    public ArchiveDATA(ResultSet rs) throws SQLException
    {
	p_iID=rs.getInt("A_ID");
	m_iType=rs.getInt("A_TYPE");
        m_iNumDossier=rs.getInt("A_NUMDO"); //Si NULL retourne 0
	m_sProb=rs.getString("A_PROB");
	m_dCategorie=new CategorieDATA(rs);
	m_sUserO=rs.getString("LOGIN_UO");
	m_sUserC=rs.getString("LOGIN_UC");
	m_sSol=rs.getString("A_SOL");
	m_sDateO=rs.getString("DATEO");
	m_sDateC=rs.getString("DATEC");
	m_bFull=false;
    }

    //ajoute un contact
    public void addContact(ResultSet rs) throws SQLException
    {
        if(!m_bFull)
	    m_vContacts.add(new ContactDATA(rs));
    }

    //ajoute un tablissement
    public void addEtab(ResultSet rs) throws SQLException
    {
        if(!m_bFull)
            m_vEtabs.add(new EtabDATA(rs));
    }

    public void setFull()
    {
	m_bFull=true;
    }

    public Vector getContacts()
    {
	return m_vContacts;
    }

    public Vector getEtabs()
    {
	return m_vEtabs;
    }

    public int getContactsNumber()
    {
	return m_vContacts.size();
    }

    public int getEtabsNumber()
    {
	return m_vEtabs.size();
    }

    public int getType()
    {
	return m_iType;
    }

    public int getNumDossier()
    {
	return m_iNumDossier;
    }

    public CategorieDATA getCategorie()
    {
	return m_dCategorie;
    }

    public String getUserClose()
    {
	return m_sUserC;
    }

    public String getUserOpen()
    {
	return m_sUserO;
    }

    public String getProb()
    {
	return m_sProb;
    }

    public String getSol()
    {
	return m_sSol;
    }

    public String getDateClose()
    {
	return m_sDateC;
    }

    public String getDateOpen()
    {
	return m_sDateO;
    }

    public boolean isFull()
    {
	return m_bFull;
    }

    public String toString()
    {
	String ret;
	//affiche seulement la premire ligne
	int i=m_sProb.indexOf('\n');
	if(i==-1)
	    ret=m_sProb.trim();
	else
	   ret=m_sProb.substring(0,i).trim();

       return ret;
    }

    public String getFiche()
    {
	String type;

	switch(m_iType)
	{
	    case 1: type="Email"; break;
	    case 2: type="Tlphone"; break;
	    case 3: type="Direct"; break;
	    case 4: type="Automatique"; break;
	    default: type="Inconnu"; break;
	}

	return new String("Dossier n:\t" + m_iNumDossier
			  + "\nOuvert le :\t"+ m_sDateO + " par " + m_sUserO
			  + "\nRsolut le :\t"+ m_sDateC + " par " + m_sUserC
			  + "\nType :    \t" + type
			  + "\nCatgorie :\t" + m_dCategorie.getNom()
			  + "\n\nProblme :\n----------\n" + m_sProb
			  + "\n\nSolution :\n----------\n" + m_sSol);
    }
}
