/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2005  Nicolas AGIUS                                     *
 *                                                                         *
 *   This file is part of ECaMM.                                           *
 *                                                                         *
 *   ECaMM is free software; you can redistribute it and/or modify it      *
 *   under the terms of the GNU General Public License as published by     *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   any later version.                                                    *
 *                                                                         *
 *   ECaMM is distributed in the hope that it will be useful, but          *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston,                *
 *    MA  02111-1307, USA.                                                 *
 *                                                                         *
 *   For more information about these matters, see the file named COPYING. *
 *                                                                         *
 *   Mail : nicolas_agius@yahoo.fr                                         *
 *                                                                         *
 ***************************************************************************/
package ecamm;

/**
 * Titre :  ConfigurationrCTL [ECaMM]
 * Description : Classe de controle de la configuration
 * Copyright : GPL v2.0 - 2005~2006
 * @author : Nicolas AGIUS
 * @version 2.0
 */

import java.io.*;
import javax.swing.*;
import java.awt.*;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.*;


public class ConfigurationCTL
{
    //paramtres constants
    private static final String REGEX_FILENAME="^ECaMM.*\\.ini$"; //pattern du nom du fichier de conf
    private static final String DF_FILENAME="ECaMM.ini"; //nom par defautl du fichier de conf
    private static final String FWP_FILE="fwp.ini";     //Framework params file

    //constantes pour Properties
    private static final String P_LAF="LAF_ClassName";
    private static final String P_ODBC="ODBC_Name";
    private static final String P_VW="VISU_Width";
    private static final String P_VH="VISU_Height";
    private static final String P_VX="VISU_PosX";
    private static final String P_VY="VISU_PosY";
    private static final String P_EW="EDIT_Width";
    private static final String P_EH="EDIT_Height";
    private static final String P_EX="EDIT_PosX";
    private static final String P_EY="EDIT_PosY";
    private static final String P_CW="CREE_Width";
    private static final String P_CH="CREE_Height";
    private static final String P_CX="CREE_PosX";
    private static final String P_CY="CREE_PosY";
    private static final String P_DEBUG="DEBUG";
    private static final String P_SHOWCOMMUNE="Show_Commune";  //affchage du nom de commune dans visuIHM
    private static final String P_LOGIN="LOGIN";
    private static final String P_MDP="PASSWORD";
    private static final String P_SHOWLOGIN="Show_Login";
    private static final String P_SAVEPASS="Save_Password";
    private static final String P_JDBC="JDBC_DriverName";
    private static final String P_RDELAY="REFRESH_DELAY";
    private static final String P_FULLTEXT="FULLTEXT_Mode";

    //valeurs par dfaut
    private static final int DF_VW=400;
    private static final int DF_VH=300;
    private static final int DF_VX=0;
    private static final int DF_VY=0;
    private static final int DF_EW=430;
    private static final int DF_EH=430;
    private static final int DF_EX=0;
    private static final int DF_EY=0;
    private static final int DF_CW=490;
    private static final int DF_CH=510;
    private static final int DF_CX=0;
    private static final int DF_CY=0;
    private static final int DF_RDELAY=90;
    private static final String DF_JDBC="com.mysql.jdbc.Driver";  //MySQL driver

    //Paramtres gnraux
    private Properties m_pData;     //donne de configuration
    private Properties m_pFwp;      //paramtre de framework
    private String m_sFileName;     //Nom du fichier de conf
    private boolean m_bModified;    //si config modifie
    private boolean m_bNewConfig;   //si premier lancemenent
    private boolean m_bAskProfile;  //si demande choix du profil
    private int m_iUserID;          //No utilisateur
    private Splash m_Sp;            //pour afficher/cacher le splashScreen

    //Paramtres de gestion du LAF
    private Vector m_vListeIHM;
    private UIManager.LookAndFeelInfo m_laf[];
    private int m_iCurrentLaf=0;    //laf metal par default

    //Accs unique  la base de donne (evite les connections redondante)
    private ConnectionBase m_cDB;

    //Classe de controle mutualise
    private RechercherCTL m_ctlRechercher;
    private EditerCTL m_ctlEditer;
    private AjouterCTL m_ctlAjouter;


    public ConfigurationCTL(String[] argv, Splash sp)
    {
        m_Sp=sp;

        //Installation du LookAndFeel de Borland JBuilder
        UIManager.installLookAndFeel("Borland","com.borland.plaf.borland.BorlandLookAndFeel");

        m_laf=UIManager.getInstalledLookAndFeels();
        m_vListeIHM=new Vector();
        m_bModified=false;
        m_bNewConfig=false;
        m_bAskProfile=false;
        m_pData=new Properties();
        m_pFwp=new Properties();
        m_iUserID=-1;

        //gestion des paramtres de ligne de commande
        readCommandLineOptions(argv);

        //lecture du fichier (si pas forc en ligne de commande)
        if(m_sFileName==null) m_sFileName=findConfigFileName();
        load();

        //apparence
        setLaf(getLafName(m_pData.getProperty(P_LAF)));
    }


    //sauvegarde de la config dans un fichier
    public void save()
    {
        if(m_bModified)
        {
            //sauvegarde si modifications
            try
            {
                if(debug()) System.out.println("Saving file..");
                OutputStream out=new FileOutputStream(m_sFileName);
                m_pData.store(out,"ECaMM Configuration");
                m_bModified=false;    //remise  zero du flag
            }
            catch(Exception e)
            {
                if(debug()) e.printStackTrace();
                JOptionPane.showMessageDialog(null,Tools.formatString(e.toString(),55),
                                "Erreur lors de l'enregistrement de la configuration",
                                JOptionPane.ERROR_MESSAGE);
            }
        }
    }

    //lecture de la config
    public void load()
    {
        //lecture des paramtres de frameworks
        try
        {
            InputStream in=getClass().getResourceAsStream(FWP_FILE);
            m_pFwp.load(in);
        }
        catch(Exception e)
        {
            e.printStackTrace();
            fatalError(FWP_FILE);
        }

        //lecture du fichier de configuration
        try
        {
            InputStream in=new FileInputStream(m_sFileName);
            m_pData.load(in);
        }
        catch(Exception e)
        {
            //valeur par dfault en cas de problme

            // P_ODBC,P_LOGIN,P_MDP  null -> initialisation dans ConfigurationIHM
            m_pData.setProperty(P_LAF,UIManager.getSystemLookAndFeelClassName());
            m_pData.setProperty(P_VW,String.valueOf(DF_VW));
            m_pData.setProperty(P_VH,String.valueOf(DF_VH));
            m_pData.setProperty(P_VX,String.valueOf(DF_VX));
            m_pData.setProperty(P_VY,String.valueOf(DF_VY));
            m_pData.setProperty(P_EW,String.valueOf(DF_EW));
            m_pData.setProperty(P_EH,String.valueOf(DF_EH));
            m_pData.setProperty(P_EX,String.valueOf(DF_EX));
            m_pData.setProperty(P_EY,String.valueOf(DF_EY));
            m_pData.setProperty(P_CW,String.valueOf(DF_CW));
            m_pData.setProperty(P_CH,String.valueOf(DF_CH));
            m_pData.setProperty(P_CX,String.valueOf(DF_CX));
            m_pData.setProperty(P_CY,String.valueOf(DF_CY));
            m_pData.setProperty(P_RDELAY,String.valueOf(DF_RDELAY));
            m_pData.setProperty(P_DEBUG,"False");
            m_pData.setProperty(P_SHOWCOMMUNE,"True");
            m_pData.setProperty(P_SHOWLOGIN,"True");
            m_pData.setProperty(P_SAVEPASS,"False");
            m_pData.setProperty(P_FULLTEXT,"True");
            m_pData.setProperty(P_JDBC,DF_JDBC);

            m_bModified=true;
            m_bNewConfig=true;
        }
    }

    //Ouverture de la base
    public boolean loadDbDriver()
    {
        try
        {
            //creation de la connection vers la base
            m_cDB = new ConnectionBase(getOdbcName(), getJDBCDriverName(), debug(), true);
        }
        catch(Exception e)
        {
            m_Sp.setVisible(false);     //hide du splash pour popup
            Tools.sqlError(e,debug());  //popup erreur
            return false;
        }

        //Instanciation des classe de controles
        m_ctlRechercher = new RechercherCTL(m_cDB,debug());
        m_ctlEditer = new EditerCTL(m_cDB,debug());
        m_ctlAjouter = new AjouterCTL(m_cDB,debug());

        return true;
    }

    //rcupration du nom de profil (nom du ficher de conf)
    private String findConfigFileName()
    {
        //parcourt du rpertoire courant
        File[] list=new File(".").listFiles(new FileFilter()
        {
            //pour trouver les fichiers de conf probable (-rw- ECaMM*.ini)
            public boolean accept(File f)
            {
                return (f.isFile() &&
                        f.canRead() &&
                        f.canWrite() &&
                        f.getName().matches(REGEX_FILENAME));
            }
        });

        //si pas file dialog demand
        if(!m_bAskProfile)
        {
            //Si aucun fichier trouv : par dfault
            if(list.length <= 0)
            {
                System.out.println("Cfg : No file found. Using default...");
                return DF_FILENAME;
            }

            //Si un seul fichier trouv : on le prend
            if(list.length == 1)
            {
                System.out.println("Cfg : Auto-selecting " + list[0].getName() + "...");
                return list[0].getName();
            }
        }

        //Affichage de la liste des fichiers trouvs
        String[] filenames=new String[list.length+1];
        for(int i=0; i<list.length; filenames[i]=list[i++].getName());
        filenames[list.length]="Autre ...";     //dans la dernire case

        m_Sp.setVisible(false); //Hide du splash
        String rep=(String)JOptionPane.showInputDialog(null,"Selectioner le profil",
                                    "Configuration",JOptionPane.QUESTION_MESSAGE,
                                    null,filenames,filenames[0]);

        //si cancel
        if(rep==null) System.exit(0);

        if(rep.equals(filenames[list.length]))  //choix autre
        {
            FileDialog fd=new FileDialog(new Frame(),"ECaMM - Slection du profil",FileDialog.LOAD);
            fd.setFilenameFilter(new FilenameFilter()   //marche pas sous XP
            {
                //N'accepte que les fichiers .ini
                public boolean accept(File dir, String name)
                {
                    return name.toLowerCase().endsWith(".ini");
                }
            });
            fd.show();

            //si cancel
            if(fd.getFile()==null) System.exit(0);

            rep=fd.getDirectory() + fd.getFile();
            System.out.println("Cfg : External selection : " + rep);
        }
        m_Sp.setVisible(true); //Show du splash
        return rep;
    }

    //fonctions internes
    //==================

    private String getLafName(String lafClassName)
    {
        if(lafClassName==null) //si pas dans fichier de conf
            return null;

        for(int i=0;i<m_laf.length;i++)
        {
            if(lafClassName.equalsIgnoreCase(m_laf[i].getClassName()))
                return m_laf[i].getName();
        }

        return null;    //si pas trouv dans la liste
    }

    private void readCommandLineOptions(String[] argv)
    {
        if(argv.length == 1 && argv[0].equalsIgnoreCase("--ask-profile"))
        {
            m_bAskProfile = true;
        }
        else if(argv.length == 2 && argv[0].equalsIgnoreCase("--force-profile"))
        {
            m_sFileName = argv[1].trim();
            m_bAskProfile = false;
        }
        else if(argv.length != 0)
        {
            System.out.println("Usage : java -jar ECaMM.jar [--ask-profile|--force-profile filename]");
            System.exit(3);
        }
    }

    private void fatalError(String key)
    {
        JOptionPane.showMessageDialog(null,"Erreur dans les paramtres du package :\n" +
                    "-->   " + key + "\nLe programme va tre arrt.",
                    "Erreur interne",
                    JOptionPane.ERROR_MESSAGE);
        System.exit(1);
    }

    //Lecture des paramtres de configurations
    //========================================

    //accs aux paramtres de framework
    public String getFwp(String key)
    {
        String p=m_pFwp.getProperty(key.toUpperCase());
        if(p==null) fatalError(key.toUpperCase());

        return p;
    }

    public boolean isItNewConfig()
    {
        return m_bNewConfig;
    }

    public int getLafCount()
    {
        return m_laf.length;
    }

    public String getConfigFileName()
    {
        return m_sFileName;
    }

    public String getLafName(int lafNumber)
    {
        return m_laf[lafNumber].getName();
    }

    public int getCurrentLafNumber()
    {
        return m_iCurrentLaf;
    }

    public String getOdbcName()
    {
        //pas de valeur pas dfault -> initialis dans ConfigurationIHM
        return m_pData.getProperty(P_ODBC);
    }

    public String getJDBCDriverName()
    {
        String jdbc;
        jdbc = m_pData.getProperty(P_JDBC);
        if(jdbc==null) //par dfault si pas dans fichier
        {
            jdbc=DF_JDBC;
            m_pData.setProperty(P_JDBC,DF_JDBC);
            m_bModified=true;
        }
        return jdbc;
    }

    public Dimension getVisuSize()
    {
        try
        {
            return new Dimension(
                    Integer.parseInt(m_pData.getProperty(P_VW)),
                    Integer.parseInt(m_pData.getProperty(P_VH))
                        );
        }
        catch(Exception e)  //par dfaut en cas d'erreur de syntaxe
        {
            return new Dimension(DF_VW,DF_VH);
        }
    }

    public Point getVisuLocation()
    {
        try
        {
            return new Point(
                    Integer.parseInt(m_pData.getProperty(P_VX)),
                    Integer.parseInt(m_pData.getProperty(P_VY))
                        );
        }
        catch(Exception e)  //par dfaut en cas d'erreur de syntaxe
        {
            return new Point(DF_VX,DF_VY);
        }
    }

    public Dimension getEditSize()
    {
        try
        {
            return new Dimension(
                    Integer.parseInt(m_pData.getProperty(P_EW)),
                    Integer.parseInt(m_pData.getProperty(P_EH))
                        );
        }
        catch(Exception e)  //par dfaut en cas d'erreur de syntaxe
        {
            return new Dimension(DF_EW,DF_EH);
        }
    }

    public Point getEditLocation()
    {
        try
        {
            return new Point(
                    Integer.parseInt(m_pData.getProperty(P_EX)),
                    Integer.parseInt(m_pData.getProperty(P_EY))
                        );
        }
        catch(Exception e)  //par dfaut en cas d'erreur de syntaxe
        {
            return new Point(DF_EX,DF_EY);
        }
    }

    public Dimension getCreerSize()
    {
        try
        {
            return new Dimension(
                    Integer.parseInt(m_pData.getProperty(P_CW)),
                    Integer.parseInt(m_pData.getProperty(P_CH))
                        );
        }
        catch(Exception e)  //par dfaut en cas d'erreur de syntaxe
        {
            return new Dimension(DF_CW,DF_CH);
        }
    }

    public Point getCreerLocation()
    {
        try
        {
            return new Point(
                    Integer.parseInt(m_pData.getProperty(P_CX)),
                    Integer.parseInt(m_pData.getProperty(P_CY))
                        );
        }
        catch(Exception e)  //par dfaut en cas d'erreur de syntaxe
        {
            return new Point(DF_CX,DF_CY);
        }
    }

    public String getLogin()
    {
        //pas de valeur pas dfault -> initialis dans ConfigurationIHM
        return m_pData.getProperty(P_LOGIN);
    }

    public String getPassword()
    {
        //pas de valeur pas dfault -> initialis dans ConfigurationIHM
        return m_pData.getProperty(P_MDP);
    }

    public int getRefreshDelay()
    {
        try
        {
            return Integer.parseInt(m_pData.getProperty(P_RDELAY));
        }
        catch(Exception e)    //par dfault si pas dans fichier
        {
            setRefreshDelay(DF_RDELAY);
            return DF_RDELAY;
        }
    }

    public RechercherCTL getRechercherCTL()
    {
        return m_ctlRechercher;
    }

    public EditerCTL getEditerCTL()
    {
        return m_ctlEditer;
    }

    public AjouterCTL getAjouterCTL()
    {
        return m_ctlAjouter;
    }

    public boolean debug()
    {
        try
        {
            return m_pData.getProperty(P_DEBUG).equalsIgnoreCase("True");
        }
        catch(NullPointerException e)    //par dfault si pas dans fichier
        {
            m_pData.setProperty(P_DEBUG,"False");
            m_bModified=true;
            return false;
        }
    }

    public boolean isShowLogin()
    {
        try
        {
            return m_pData.getProperty(P_SHOWLOGIN).equalsIgnoreCase("True");
        }
        catch(NullPointerException e)    //par dfault si pas dans fichier
        {
            setShowLogin(true);
            return true;
        }
    }

    public boolean isSavePassword()
    {
        try
        {
            return m_pData.getProperty(P_SAVEPASS).equalsIgnoreCase("True");
        }
        catch(NullPointerException e)    //par dfault si pas dans fichier
        {
            setSavePassword(false);
            return false;
        }
    }

    public boolean isShowCommune()
    {
        try //si paramtre pas dans le fichier (migration v1.1 -> v1.2)
        {
            return m_pData.getProperty(P_SHOWCOMMUNE).equalsIgnoreCase("True");
        }
        catch(NullPointerException e)
        {
            setShowCommune(true);
            return true;
        }
    }

    public boolean isFulltextMode()
    {
        try //si paramtre pas dans le fichier (migration v1.2 -> v2.0)
        {
            return m_pData.getProperty(P_FULLTEXT).equalsIgnoreCase("True");
        }
        catch(NullPointerException e)
        {
            setFulltextMode(true);
            return true;
        }
    }


    //Ecriture des paramtres de configurations
    //=========================================

    public void setOdbcName(String odbcName)
    {
        if(odbcName!=null && odbcName.length()>0) //si non vide
        {
            m_pData.setProperty(P_ODBC,odbcName);
            m_bModified=true;
        }
    }

    public void setVisuSize(Dimension dim)
    {
        if(!dim.equals(getVisuSize())) //si modifi
        {
            m_pData.setProperty(P_VW,String.valueOf(dim.width));
            m_pData.setProperty(P_VH,String.valueOf(dim.height));
            m_bModified=true;
        }
    }

    public void setVisuLocation(Point loc)
    {
        if(!loc.equals(getVisuLocation())) //si modifi
        {
            m_pData.setProperty(P_VX,String.valueOf(loc.x));
            m_pData.setProperty(P_VY,String.valueOf(loc.y));
            m_bModified=true;
        }
    }

    public void setEditSize(Dimension dim)
    {
        if(!dim.equals(getEditSize())) //si modifi
        {
            m_pData.setProperty(P_EW,String.valueOf(dim.width));
            m_pData.setProperty(P_EH,String.valueOf(dim.height));
            m_bModified=true;
        }
    }

    public void setEditLocation(Point loc)
    {
        if(!loc.equals(getEditLocation())) //si modifi
        {
            m_pData.setProperty(P_EX,String.valueOf(loc.x));
            m_pData.setProperty(P_EY,String.valueOf(loc.y));
            m_bModified=true;
        }
    }

    public void setCreerSize(Dimension dim)
    {
        if(!dim.equals(getCreerSize())) //si modifi
        {
            m_pData.setProperty(P_CW,String.valueOf(dim.width));
            m_pData.setProperty(P_CH,String.valueOf(dim.height));
            m_bModified=true;
        }
    }

    public void setCreerLocation(Point loc)
    {
        if(!loc.equals(getCreerLocation())) //si modifi
        {
            m_pData.setProperty(P_CX,String.valueOf(loc.x));
            m_pData.setProperty(P_CY,String.valueOf(loc.y));
            m_bModified=true;
        }
    }

    public void setSavePassword(boolean save)
    {
        m_pData.setProperty(P_SAVEPASS,save?"True":"False");
        m_bModified=true;
    }

    public void setLogin(String login)
    {
        m_pData.setProperty(P_LOGIN,login);
        m_bModified=true;
    }

    public void setPassword(String mdp)
    {
        m_pData.setProperty(P_MDP,mdp);
        m_bModified=true;
    }

    public void setRefreshDelay(int delay)
    {
        if(delay>0)
        {
            m_pData.setProperty(P_RDELAY,String.valueOf(delay));
            m_bModified=true;
        }
    }

    public void setShowLogin(boolean show)
    {
        m_pData.setProperty(P_SHOWLOGIN,show?"True":"False");
        m_bModified=true;
    }

    public void setShowCommune(boolean show)
    {
        m_pData.setProperty(P_SHOWCOMMUNE,show?"True":"False");
        m_bModified=true;
    }

    public void setFulltextMode(boolean on)
    {
        String oldValue=(String)m_pData.setProperty(P_FULLTEXT, on ? "True" : "False");

        if(oldValue==null || !oldValue.equalsIgnoreCase(on ? "True" : "False")) //si modifi
        {
            m_bModified = true;
        }
    }

    public void setLaf(String lafName)
    {
        int laf=0;
        if(lafName==null)    //protection si pas dans le fichier (getLafName retourne null)
        {
            //si pas dans le fichier m_iCurrentLaf=0 -> premier laf trouv
            saveCurrentLaf();
        }
        else
        {
            for (int i = 0; i < m_laf.length; i++)
            {
                if (lafName.equalsIgnoreCase(m_laf[i].getName()))
                    laf = i;
            }

            //mise  jour du laf
            if (m_iCurrentLaf != laf)
            {
                try
                {
                    UIManager.setLookAndFeel(m_laf[laf].getClassName());
                }
                catch (Exception e)
                {
                    Tools.internalError(e, "", debug(), false);
                }
                m_iCurrentLaf = laf;

                //Mise a jour des interfaces
                Iterator it = m_vListeIHM.iterator();
                while(it.hasNext())
                {
                    SwingUtilities.updateComponentTreeUI((JFrame)it.next());
                }
            }
        }
    }

    public void saveCurrentLaf()
    {
        m_pData.setProperty(P_LAF,m_laf[m_iCurrentLaf].getClassName());
        m_bModified=true;
    }

    public void registerIHM(JFrame ihm)
    {
        m_vListeIHM.add(ihm);
    }

    public int getUserID()
    {
        return m_iUserID;
    }

    public boolean checkPassword(String password)
    {
        try
        {
            ResultSet rs = m_cDB.ExecuteQuery("select * from USER where U_LOGIN = '" + getLogin() + "'");  //execute la requete

            if(rs.first())
            {
                m_iUserID=rs.getInt("U_ID");
                return password.equals(rs.getString("U_PASSWD"));
            }
        }
        catch(SQLException e)
        {
            Tools.sqlError(e,debug());
        }
        catch(Exception e)
        {
            Tools.internalError(e,"",debug(),false);
        }

        return false;   //en cas d'erreur ou login incorrect
    }
}
