/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2005  Nicolas AGIUS                                     *
 *                                                                         *
 *   This file is part of ECaMM.                                           *
 *                                                                         *
 *   ECaMM is free software; you can redistribute it and/or modify it      *
 *   under the terms of the GNU General Public License as published by     *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   any later version.                                                    *
 *                                                                         *
 *   ECaMM is distributed in the hope that it will be useful, but          *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston,                *
 *    MA  02111-1307, USA.                                                 *
 *                                                                         *
 *   For more information about these matters, see the file named COPYING. *
 *                                                                         *
 *   Mail : nicolas_agius@yahoo.fr                                         *
 *                                                                         *
 ***************************************************************************/
package ecamm;

/**
 * Titre :  ConfigurationIHM [ECaMM]
 * Description : Fentre de configuration
 * Copyright : GPL v2.0 - 2005~2006
 * @author : Nicolas AGIUS
 * @version 1.1
 */

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.border.*;

public class ConfigurationIHM extends JFrame
{
    //Constantes pour le keybinding
    private static final String ACTION_SAVE="save";
    private static final String ACTION_CANCEL="cancel";

    //variable pour la gestion de l'interface
    private ConfigurationCTL cf=Main.cf;
    private JRadioButton m_bApparences[];

    //composant graphiques
    private JPanel jPanel1 = new JPanel();
    private JLabel jLabel1 = new JLabel();
    private JTextField jTextFieldDB = new JTextField();
    private JCheckBox jCheckBoxShowLogin = new JCheckBox();
    private JCheckBox jCheckBoxSavePass = new JCheckBox();
    private JTextField jTextFieldLogin = new JTextField();
    private JPasswordField jPasswordField = new JPasswordField();
    private JLabel jLabel2 = new JLabel();
    private JLabel jLabel3 = new JLabel();
    private JPanel jPanel2 = new JPanel();
    private JButton jButtonOK = new JButton();
    private JButton jButtonCancel = new JButton();
    private JPanel jPanelLF = new JPanel();
    private TitledBorder titledBorder1;
    private ButtonGroup buttonGroup1 = new ButtonGroup();
    private GridLayout gridLayout1 = new GridLayout();
    private JLabel jLabel4 = new JLabel();
    private JLabel jLabel5 = new JLabel();
    private GridBagLayout gridBagLayout1 = new GridBagLayout();
    private JLabel jLabelDelay = new JLabel();
    private JSlider jSlider1 = new JSlider();
    private JLabel jLabel6 = new JLabel();
    private JCheckBox jCheckBoxShowCom = new JCheckBox();

    //boite de dialogue pour l'authentification
    public static String showLogin()
    {
        ConfigurationCTL conf=Main.cf;
        String password;
        final JDialog d=new JDialog(new JFrame(),"ECaMM",true);

        //Cration du dialog
        JPanel jPanelMain=new JPanel();
        GridLayout gridLayout1 = new GridLayout();
        JPanel jPanelBt=new JPanel();
        JCheckBox jCheckBoxSavePass = new JCheckBox("Mmoriser le mot de passe");
        JTextField jTextFieldLogin = new JTextField();
        JPasswordField jPasswordField = new JPasswordField();
        JButton jButtonCancel = new JButton("Annuler");
        jButtonCancel.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                System.exit(0);
            }
        });
        JButton jButtonOK = new JButton("OK");
        jButtonOK.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
               d.dispose();
            }
        });
        gridLayout1.setRows(5);
        gridLayout1.setColumns(1);
        gridLayout1.setVgap(2);
        jPanelMain.setLayout(gridLayout1);
        jPanelMain.add(new JLabel("Login :"));
        jPanelMain.add(jTextFieldLogin);
        jPanelMain.add(new JLabel("Mot de passe :"));
        jPanelMain.add(jPasswordField);
        jPanelMain.add(jCheckBoxSavePass);

        jPanelBt.add(jButtonOK);
        jPanelBt.add(jButtonCancel);

        d.getContentPane().add(jPanelMain, BorderLayout.CENTER);
        d.getContentPane().add(jPanelBt, BorderLayout.SOUTH);
        d.getContentPane().add(new JPanel(), BorderLayout.WEST);
        d.getContentPane().add(new JPanel(), BorderLayout.EAST);

        //dimensionnement
        d.pack();
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        Dimension frameSize = d.getSize();
        d.setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);

        //Keys binding
        InputMap inputMap=d.getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW);
        ActionMap actionMap=d.getRootPane().getActionMap();
        inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER,0),ACTION_SAVE);
        inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE,0),ACTION_CANCEL);
        actionMap.put(ACTION_SAVE,new AbstractAction()
        {
            public void actionPerformed(ActionEvent e)
            {
                d.dispose();
            }
        });
        actionMap.put(ACTION_CANCEL,new AbstractAction()
        {
            public void actionPerformed(ActionEvent e)
            {
                System.exit(0);
            }
        });

        //lecture des donnes enregistes
        jCheckBoxSavePass.setSelected(conf.isSavePassword());
        if(conf.isSavePassword()) jPasswordField.setText(conf.getPassword());
        jTextFieldLogin.setText(conf.getLogin());

        //affichage du dialog modal
        d.show();

        //rcupration des donnees
        password=new String(jPasswordField.getPassword());
        conf.setPassword(jCheckBoxSavePass.isSelected()?password:"");
        conf.setLogin(jTextFieldLogin.getText());
        conf.setSavePassword(jCheckBoxSavePass.isSelected());

        return password;
    }

    public ConfigurationIHM()
    {
        cf.registerIHM(this);

        try
        {
            jbInit();
            lafInit();
            keyBindingInit();
        }
        catch(Exception e)
        {
            Tools.internalError(e,"",cf.debug(),true);
        }

        //lecture de la conf
        jTextFieldDB.setText(cf.getOdbcName());
        jCheckBoxShowLogin.setSelected(cf.isShowLogin());
        jCheckBoxSavePass.setSelected(cf.isSavePassword());
        jCheckBoxShowCom.setSelected(cf.isShowCommune());
        jTextFieldLogin.setText(cf.getLogin());
        if(cf.isSavePassword())
            jPasswordField.setText(cf.getPassword());
        if(!cf.isShowLogin())
            jCheckBoxSavePass.setEnabled(false);
        jSlider1.setValue(cf.getRefreshDelay());

        //dimmensionnement
        pack();
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        Dimension frameSize = getSize();
        setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);
        setVisible(true);
    }

    private void lafInit() throws Exception
    {
        //cration du panel "Apparences"
        m_bApparences=new JRadioButton[cf.getLafCount()];
        gridLayout1.setRows(cf.getLafCount());
        for(int i=0;i<m_bApparences.length;i++)
        {
            m_bApparences[i]=new JRadioButton(cf.getLafName(i));
            m_bApparences[i].setMnemonic(cf.getLafName(i).charAt(0));
            buttonGroup1.add(m_bApparences[i]);
            m_bApparences[i].addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    jRadioButtonLF_actionPerformed(e);
                }
            });
            jPanelLF.add(m_bApparences[i]);
        }

        m_bApparences[cf.getCurrentLafNumber()].setSelected(true);
    }

    private void jbInit() throws Exception
    {
        titledBorder1 = new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134))," Apparence : ");
        jLabel1.setText("Nom du lien ODBC (ou driver JDBC) :");
        jPanel1.setLayout(gridBagLayout1);
        this.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        this.setTitle("Configuration    [" + cf.getConfigFileName() + "]");
        jCheckBoxShowLogin.setText("Afficher la fentre d\'authentification au dmarrage");
        jCheckBoxShowLogin.addItemListener(new java.awt.event.ItemListener()
        {
            public void itemStateChanged(ItemEvent e)
            {
                jCheckBoxShowLogin_itemStateChanged(e);
            }
        });
        jCheckBoxSavePass.setText("Mmoriser le mot de passe");
        jLabel2.setText("Login :");
        jLabel3.setText("Mot de passe : ");
        jButtonOK.setText("OK");
        jButtonOK.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                jButtonOK_actionPerformed(e);
            }
        });
        jButtonCancel.setText("Annuler");
        jButtonCancel.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                jButtonCancel_actionPerformed(e);
            }
        });
        jPanelLF.setBorder(titledBorder1);
        jPanelLF.setLayout(gridLayout1);
        gridLayout1.setColumns(1);
        gridLayout1.setHgap(10);
        jLabel4.setText("Dlai de rafraichissement :");
        jLabel5.setText("secondes");
        jLabel6.setText("Fentre principale :");
        jCheckBoxShowCom.setText("Afficher la commune de l'tablissement");
        jSlider1.setMinorTickSpacing(10);
        jSlider1.setMajorTickSpacing(60);
        jSlider1.setMinimum(1);
        jSlider1.setValue(0);
        jSlider1.setPaintTicks(true);
        jSlider1.setMaximum(900);
        jSlider1.setPreferredSize(new Dimension(300,32));
        jSlider1.addChangeListener(new javax.swing.event.ChangeListener()
        {
            public void stateChanged(ChangeEvent e)
            {
                jSlider1_stateChanged(e);
            }
        });
        this.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        this.getContentPane().add(jPanel1, BorderLayout.CENTER);
        jPanel1.add(jPanelLF, new GridBagConstraints(0, 6, 5, 1, 1.0, 0.0
            ,GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL, new Insets(6, 6, 0, 6), 0, 0));
        jPanel1.add(jLabel1, new GridBagConstraints(0, 0, 4, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(6, 6, 0, 6), 0, 0));
        jPanel1.add(jTextFieldDB, new GridBagConstraints(0, 1, 4, 1, 1.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(6, 16, 0, 16), 0, 0));
        jPanel1.add(jCheckBoxShowLogin, new GridBagConstraints(0, 2, 5, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(12, 6, 0, 0), 0, 0));
        jPanel1.add(jCheckBoxSavePass, new GridBagConstraints(0, 3, 5, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(3, 6, 0, 6), 0, 0));
        jPanel1.add(jLabel2, new GridBagConstraints(0, 4, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(3, 16, 0, 0), 61, 4));
        jPanel1.add(jTextFieldLogin, new GridBagConstraints(1, 4, 4, 1, 1.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(3, 0, 0, 16), 0, 0));
        jPanel1.add(jPasswordField, new GridBagConstraints(1, 5, 4, 1, 1.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(3, 0, 0, 16), 0, 0));
        jPanel1.add(jLabel3, new GridBagConstraints(0, 5, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(3, 16, 0, 0), 17, 4));
        jPanel1.add(jLabel4, new GridBagConstraints(0, 9, 2, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(3, 20, 0, 0), 0, 0));
        jPanel1.add(jPanel2, new GridBagConstraints(0, 11, 5, 1, 1.0, 1.0
            ,GridBagConstraints.SOUTH, GridBagConstraints.HORIZONTAL, new Insets(0, 6, 6, 6), 0, 0));
        jPanel2.add(jButtonOK, null);
        jPanel2.add(jButtonCancel, null);
        jPanel1.add(jLabel5, new GridBagConstraints(3, 9, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(3, 3, 0, 16), 0, 0));
        jPanel1.add(jLabelDelay, new GridBagConstraints(2, 9, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(3, 6, 0, 0), 0, 0));
        jPanel1.add(jSlider1, new GridBagConstraints(0, 10, 4, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(6, 6, 6, 6), 0, 0));
        jPanel1.add(jLabel6, new GridBagConstraints(0, 7, 4, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(12, 6, 0, 6), 0, 0));
        jPanel1.add(jCheckBoxShowCom, new GridBagConstraints(0, 8, 4, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(3, 16, 0, 16), 0, 0));

    }

    private void keyBindingInit() throws Exception
    {
        //Keys binding
        InputMap inputMap=jPanel1.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW);
        ActionMap actionMap=jPanel1.getActionMap();
        inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER,0),ACTION_SAVE);
        inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE,0),ACTION_CANCEL);
        actionMap.put(ACTION_SAVE,new AbstractAction()
        {
            public void actionPerformed(ActionEvent e)
            {
                jButtonOK_actionPerformed(e);
            }
        });
        actionMap.put(ACTION_CANCEL,new AbstractAction()
        {
            public void actionPerformed(ActionEvent e)
            {
                jButtonCancel_actionPerformed(e);
            }
        });
    }

    void jRadioButtonLF_actionPerformed(ActionEvent e)
    {
        //rcupration du laf slectionn
        cf.setLaf(e.getActionCommand());
    }

    void jButtonCancel_actionPerformed(ActionEvent e)
    {
        dispose();
    }

    void jCheckBoxShowLogin_itemStateChanged(ItemEvent e)
    {
        boolean showLogin=jCheckBoxShowLogin.isSelected();

        jPasswordField.setEnabled(!showLogin);
        jTextFieldLogin.setEnabled(!showLogin);
        jCheckBoxSavePass.setEnabled(showLogin);
        if(!showLogin) jCheckBoxSavePass.setSelected(true);
        jTextFieldLogin.setBackground(showLogin?Color.lightGray:Color.white);
        jPasswordField.setBackground(showLogin?Color.lightGray:Color.white);
    }

    void jButtonOK_actionPerformed(ActionEvent e)
    {
        cf.setOdbcName(jTextFieldDB.getText());
        cf.setLogin(jTextFieldLogin.getText());
        cf.setRefreshDelay(jSlider1.getValue());
        cf.setPassword(jCheckBoxSavePass.isSelected()?new String(jPasswordField.getPassword()):"");
        cf.setSavePassword(jCheckBoxSavePass.isSelected());
        cf.setShowLogin(jCheckBoxShowLogin.isSelected());
        cf.setShowCommune(jCheckBoxShowCom.isSelected());
        cf.saveCurrentLaf();
        cf.save();

        dispose();
    }

    void jSlider1_stateChanged(ChangeEvent e)
    {
        jLabelDelay.setText(String.valueOf(jSlider1.getValue()));
    }
}
