/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2005  Nicolas AGIUS                                     *
 *                                                                         *
 *   This file is part of ECaMM.                                           *
 *                                                                         *
 *   ECaMM is free software; you can redistribute it and/or modify it      *
 *   under the terms of the GNU General Public License as published by     *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   any later version.                                                    *
 *                                                                         *
 *   ECaMM is distributed in the hope that it will be useful, but          *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston,                *
 *    MA  02111-1307, USA.                                                 *
 *                                                                         *
 *   For more information about these matters, see the file named COPYING. *
 *                                                                         *
 *   Mail : nicolas_agius@yahoo.fr                                         *
 *                                                                         *
 ***************************************************************************/

package ecamm;

/**
 * Titre :  ConnectionBase
 * Description : Permet la connection avec une base de donne via un lien ODBC
 * Copyright : GPL 2003 - 2005
 * @author : Nicolas AGIUS
 * @version 3.2
 */

import java.sql.*;
import java.lang.*;

public class ConnectionBase
{
        private String m_sNomBase;          //nom du lien ODBC
        private boolean m_bPrintQuery;      //affichage des requte SQL
        private boolean m_bKeepAlive=true;  //si garde ouverte la connection
        private Connection m_cCon;

        private void init(String nomBase,String jdbcDriverName) throws Exception
        {
            if(nomBase.startsWith("jdbc:"))
            {   //driver JDBC
                Class.forName(jdbcDriverName).newInstance();
            }
            else
            {   //driver ODBC
                Class.forName("sun.jdbc.odbc.JdbcOdbcDriver").newInstance();
            }

            if(m_bKeepAlive)
                m_cCon=getConnection();
            else
                getConnection();    //juste pour le test de validit
        }

        public ConnectionBase(String nomBase) throws Exception
        {
            m_sNomBase=new String(nomBase);   //sauvegarde du nom du lien ODBC
            m_bPrintQuery=false;

            init(nomBase,null);
        }

        public ConnectionBase(String nomBase, String jdbcDriverName) throws Exception
        {
            m_sNomBase=new String(nomBase);   //sauvegarde du nom du lien ODBC
            m_bPrintQuery=false;

            init(nomBase,jdbcDriverName);
        }

        public ConnectionBase(String nomBase, String jdbcDriverName, boolean printQuery) throws Exception
        {
            m_sNomBase=new String(nomBase);   //sauvegarde du nom du lien ODBC
            m_bPrintQuery=printQuery;

            init(nomBase,jdbcDriverName);
        }

        public ConnectionBase(String nomBase, String jdbcDriverName, boolean printQuery, boolean keepAlive) throws Exception
        {
            m_sNomBase=new String(nomBase);   //sauvegarde du nom du lien ODBC
            m_bPrintQuery=printQuery;
            m_bKeepAlive=keepAlive;

            init(nomBase,jdbcDriverName);
        }


        /**
          * Etablit la connection avec la base
          */
	private Connection getConnection() throws SQLException
	{
            if(m_sNomBase.startsWith("jdbc:"))
                return DriverManager.getConnection(m_sNomBase);
            else
		return DriverManager.getConnection("jdbc:odbc:" + m_sNomBase);

        //gestion retour null
	}

        /**
          * Execute la requete avec gnration d'un ResulSet
          */
	public ResultSet ExecuteQuery(String query) throws SQLException
	{
            if(m_bPrintQuery) System.out.println(query);

            Connection con ;

            if(m_bKeepAlive)
                con = m_cCon;
            else
                con = getConnection();

            Statement st;

            st = con.createStatement(
                                ResultSet.TYPE_SCROLL_INSENSITIVE,
                                ResultSet.CONCUR_READ_ONLY);
            return st.executeQuery(query);
	}


        /**
          * Execute la requete sans gnration de ResulSet
          */
	public void ExecuteUpdate(String query) throws SQLException
	{
            if(m_bPrintQuery) System.out.println(query);

            Connection con ;

            if(m_bKeepAlive)
                con = m_cCon;
            else
                con = getConnection();

            Statement st;

            st = con.createStatement();
            st.executeUpdate(query);
            st.close();
            if(!m_bKeepAlive) con.close();
	}
}
