/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2005  Nicolas AGIUS                                     *
 *                                                                         *
 *   This file is part of ECaMM.                                           *
 *                                                                         *
 *   ECaMM is free software; you can redistribute it and/or modify it      *
 *   under the terms of the GNU General Public License as published by     *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   any later version.                                                    *
 *                                                                         *
 *   ECaMM is distributed in the hope that it will be useful, but          *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston,                *
 *    MA  02111-1307, USA.                                                 *
 *                                                                         *
 *   For more information about these matters, see the file named COPYING. *
 *                                                                         *
 *   Mail : nicolas_agius@yahoo.fr                                         *
 *                                                                         *
 ***************************************************************************/
package ecamm;

/**
 * Titre :  DemandeDATA [ECaMM]
 * Description : Classe de donnes Demandes
 * Copyright : GPL v2.0 - 2005
 * @author : Nicolas AGIUS
 * @version 1.0
 */

import java.sql.SQLException;
import java.sql.ResultSet;
import java.util.Vector;
import java.util.StringTokenizer;

public class DemandeDATA extends Data
{
    private int m_iType;
    private int m_iPriorite;
    private int m_iEtat;
    private CategorieDATA m_dCategorie;
    private String m_sUser;
    private String m_sProb;
    private String m_sSol;
    private String m_sDate;
    private String m_sVisuPrefix;

    private Vector m_vContacts=new Vector();
    private Vector m_vEtabs=new Vector();

    public DemandeDATA()
    {
        p_iID=NONE;
        m_iType=NONE;
        m_iPriorite=NONE;
        m_iEtat=NONE;
        m_dCategorie=new CategorieDATA();
        m_sUser="";
        m_sProb="";
        m_sSol="";
        m_sDate="";
	m_sVisuPrefix=null;  //affichage du nom et de la commune de l'etablissement si 1 seul etab
    }

    public DemandeDATA(ResultSet rs) throws SQLException
    {
        p_iID=rs.getInt("D_ID");    //rempli uniquement D_ID pour recherche No Dossier

        try     //remplissage semi-complet optionnel (pour recherche visu)
        {
            m_iPriorite = rs.getInt("D_PRI");
            m_sProb = rs.getString("D_PROB");
            m_iEtat = rs.getInt("D_ETAT");
        }
        catch (SQLException e){}

        try  //remplissage pour recherche visu -> pas utilis en mode normal
        {
            String etName = rs.getString("ET_NOM"); //peut etre null
            String coName = rs.getString("CO_NOM");
            if(etName != null && coName != null)
            {
                StringTokenizer st = new StringTokenizer(coName);
                m_sVisuPrefix = "[" + etName;
                if(Main.cf.isShowCommune())
                {
                    //ne garde que les 2 premiers mots de la commune
                    if(st.hasMoreTokens()) m_sVisuPrefix += " ," + st.nextToken();
                    if(st.hasMoreTokens()) m_sVisuPrefix += " " + st.nextToken();
                }
                m_sVisuPrefix += "] ";
            }
        }
        catch (SQLException e){}

        try     //remplissage complet optionnel (pour edition)
        {
            m_iType = rs.getInt("D_TYPE");
            m_dCategorie=new CategorieDATA(rs);
            m_sUser=rs.getString("U_LOGIN");
            m_sSol=rs.getString("D_SOL");
            m_sDate=rs.getString("DATE");
        }
        catch(SQLException e){}
    }

    //ajoute un contact
    public void addContact(ResultSet rs) throws SQLException
    {
        m_vContacts.add(new ContactDATA(rs));
    }

    //ajoute un tablissement
    public void addEtab(ResultSet rs) throws SQLException
    {
        m_vEtabs.add(new EtabDATA(rs));
    }

    public Vector getContacts()
    {
        return m_vContacts;
    }

    public Vector getEtabs()
    {
        return m_vEtabs;
    }

    public int getContactsNumber()
    {
        return m_vContacts.size();
    }

    public int getEtabsNumber()
    {
        return m_vEtabs.size();
    }

    public int getType()
    {
        return m_iType;
    }

    public int getPriorite()
    {
        return m_iPriorite;
    }

    public int getEtat()
    {
        return m_iEtat;
    }

    public CategorieDATA getCategorie()
    {
        return m_dCategorie;
    }

    public String getUser()
    {
        return m_sUser;
    }

    public String getProb()
    {
        return m_sProb;
    }

    public String getSol()
    {
        return m_sSol;
    }

    public String getDate()
    {
        return m_sDate;
    }

    public String toString()
    {
	String ret = "";

        if(m_sProb!=null)  //si pas remplissage partiel "recherche dossier"
        {
            //affiche seulement la premire ligne
            ret=Tools.firstLine(m_sProb);
        }

       //rappel du nom de l'etablissement
       if(m_sVisuPrefix!=null)
	   ret = m_sVisuPrefix + ret;

       return ret;
    }
}

