/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2005  Nicolas AGIUS                                     *
 *                                                                         *
 *   This file is part of ECaMM.                                           *
 *                                                                         *
 *   ECaMM is free software; you can redistribute it and/or modify it      *
 *   under the terms of the GNU General Public License as published by     *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   any later version.                                                    *
 *                                                                         *
 *   ECaMM is distributed in the hope that it will be useful, but          *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston,                *
 *    MA  02111-1307, USA.                                                 *
 *                                                                         *
 *   For more information about these matters, see the file named COPYING. *
 *                                                                         *
 *   Mail : nicolas_agius@yahoo.fr                                         *
 *                                                                         *
 ***************************************************************************/
package ecamm;

/**
 * Titre :  AjouterCTL [ECaMM]
 * Description : Classe de controle d'dition des donnes
 * Copyright : GPL v2.0 - 2005
 * @author : Nicolas AGIUS
 * @version 1.0
 */

import java.sql.ResultSet;
import java.sql.SQLException;

public class EditerCTL
{
    private boolean m_bDebug;
    private ConnectionBase m_cDB;

    public EditerCTL(ConnectionBase maBase, boolean debug)
    {
        m_cDB=maBase;
        m_bDebug=debug;
    }

    public String getLockUser(int id)
    {
        try
        {
            ResultSet rs = m_cDB.ExecuteQuery("SELECT U_LOGIN FROM DEMANDE,USER where D_FLAG=U_ID and D_ID="+ id);  //execute la requete
            if(rs.first())
                return rs.getString("U_LOGIN");
            else
                return null;
        }
        catch(SQLException e)
        {
            Tools.sqlError(e,m_bDebug);
        }
        catch(Exception e)
        {
            Tools.internalError(e,"",m_bDebug,false);
        }
        return null;
    }

    public boolean updateDemande(DemandeDATA dem, String sol,String prob,CategorieDATA cat,int pri,int etat)
    {
        if(!updateData("update DEMANDE set D_CA_ID="+cat.getID()+", D_PRI="+pri+", D_ETAT="+etat+",D_SOL='"+Tools.protectString(sol)+"', D_PROB='"+ Tools.protectString(prob)+"'where D_ID="+dem.getID()))
            return false;

        if(etat==4)//si etat=resolut : dplacement dans la table ARCHIVE
        {
            //unlock ici pour changement table
            unlock(dem);

            //Proprit de Mysql : Les valeurs en auto_increment ne sont jamais rutiliss
            // -> D_ID reste unique mme aprs le transfert de table

            //transfert de table
            if(!updateData("insert into ARCHIVE (A_U_IDC, A_DATEC, A_U_IDO, A_CA_ID, A_PROB, A_SOL, A_TYPE, A_NUMDO, A_DATEO)"+
                      "select "+Main.cf.getUserID()+", NOW(), D_U_IDO, D_CA_ID, D_PROB, D_SOL, D_TYPE, D_ID, D_DATEO "+
                      "from DEMANDE where D_ID="+dem.getID()))
                return false;

            if(!updateData("insert into CTA (CTA_A_ID,CTA_CT_ID) select LAST_INSERT_ID(),CTD_CT_ID from CTD where CTD_D_ID="+dem.getID()))
                return false;

            if(!updateData("insert into ETA (ETA_A_ID,ETA_ET_ID) select LAST_INSERT_ID(),ETD_ET_ID from ETD where ETD_D_ID="+dem.getID()))
                return false;

            //suppression dans DEMANDE
            updateData("delete from ETD where ETD_D_ID="+dem.getID());
            updateData("delete from CTD where CTD_D_ID="+dem.getID());
            updateData("delete from DEMANDE where D_ID="+dem.getID());
        }

        return true;
    }

    public void lock(DemandeDATA demande)
    {
        updateData("update DEMANDE set D_FLAG="+Main.cf.getUserID()+" where D_ID="+demande.getID());
    }

    public void unlock(DemandeDATA dem)
    {
        if(dem!=null)
            updateData("update DEMANDE set D_FLAG=0 where D_ID="+dem.getID());
    }

    public DemandeDATA getDemande(int id)
    {
        try
        {
            ResultSet rs1 = m_cDB.ExecuteQuery("select *,DATE_FORMAT(D_DATEO,'%d/%m/%Y %H:%i') as DATE from DEMANDE,USER,CATEGORIE where CA_ID=D_CA_ID and U_ID=D_U_IDO and D_ID="+id);
            rs1.first();
            DemandeDATA dem=new DemandeDATA(rs1);

            ResultSet rs2 = m_cDB.ExecuteQuery("select * from CONTACT,CIRCO,CTD where CI_ID=CT_CI_ID and CT_ID=CTD_CT_ID and CTD_D_ID="+id);
            rs2.beforeFirst();
            while(rs2.next()) dem.addContact(rs2);


            ResultSet rs3 = m_cDB.ExecuteQuery("select * from ETABLISSEMENT,COMMUNE,CIRCO,ETD where CO_ID=ET_CO_ID and ET_ID=ETD_ET_ID and CO_CI_ID=CI_ID and ETD_D_ID="+id);
            rs3.beforeFirst();
            while(rs3.next()) dem.addEtab(rs3);

            return dem;

        }
        catch(SQLException e)
        {
            Tools.sqlError(e,m_bDebug);
        }
        catch(Exception e)
        {
            Tools.internalError(e,"",m_bDebug,false);
        }
        return null;
    }

    private boolean updateData(String req)
    {
        try
        {
            m_cDB.ExecuteUpdate(req);
        }
        catch(SQLException e)
        {
            Tools.sqlError(e,m_bDebug);
            return false;

        }
        catch(Exception e)
        {
            Tools.internalError(e,"",m_bDebug,false);
            return false;
        }
        return true;
    }
}
