/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2005  Nicolas AGIUS                                     *
 *                                                                         *
 *   This file is part of ECaMM.                                           *
 *                                                                         *
 *   ECaMM is free software; you can redistribute it and/or modify it      *
 *   under the terms of the GNU General Public License as published by     *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   any later version.                                                    *
 *                                                                         *
 *   ECaMM is distributed in the hope that it will be useful, but          *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston,                *
 *    MA  02111-1307, USA.                                                 *
 *                                                                         *
 *   For more information about these matters, see the file named COPYING. *
 *                                                                         *
 *   Mail : nicolas_agius@yahoo.fr                                         *
 *                                                                         *
 ***************************************************************************/
package ecamm;

/**
 * Titre :  EditerIHM [ECaMM]
 * Description : Fentre d'dition d'une demande
 * Copyright : GPL v2.0 - 2005~2006
 * @author : Nicolas AGIUS
 * @version 2.0
 */

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Vector;
import javax.swing.undo.UndoManager;
import javax.swing.event.UndoableEditEvent;
import javax.swing.event.UndoableEditListener;
import javax.swing.text.DefaultEditorKit;
import java.util.HashMap;
import java.awt.datatransfer.DataFlavor;
import javax.swing.undo.CannotUndoException;
import javax.swing.undo.CannotRedoException;

public class EditerIHM extends JFrame
{
    //Constantes pour le keybinding
    private static final String ACTION_SAVE="save";
    private static final String ACTION_CANCEL="cancel";
    private static final String ACTION_VIEWET="etablissement";
    private static final String ACTION_VIEWCT="contact";
    private static final String ACTION_FOCUSCAT="categorie";
    private static final String ACTION_FOCUSSOL="solution";
    private static final String ACTION_FOCUSPROB="probleme";
    private static final String ACTION_UNDOPROB="undo-prob";
    private static final String ACTION_REDOPROB="redo-prob";
    private static final String ACTION_UNDOSOL="undo-sol";
    private static final String ACTION_REDOSOL="redo-sol";


    //numration pour boite de dialogue
    public static final String OPTIONS[]={"Ok","Lecture seule"};

    //variable de traitement
    private ConfigurationCTL cf=Main.cf;
    private EditerCTL m_ctlEditer;
    private RechercherCTL m_ctlRechercher;
    private RechercherEtabIHM m_ctlRechEtab;
    private RechercherContactIHM m_ctlRechContact;
    private DemandeDATA m_dDemande;
    private boolean m_bReadOnly;

    //composant graphiques
    private JPanel jPanel1 = new JPanel();
    private JLabel jLabelEtabs = new JLabel();
    private JLabel jLabelContacts = new JLabel();
    private JLabel jLabelDate = new JLabel();
    private JLabel jLabel4 = new JLabel();
    private JLabel jLabel5 = new JLabel();
    private JScrollPane jScrollPaneProb = new JScrollPane();
    private JScrollPane jScrollPaneSol = new JScrollPane();
    private JTextArea jTextAreaProb = new JTextArea();
    private JTextArea jTextAreaSol = new JTextArea();
    private JLabel jLabel6 = new JLabel();
    private JLabel jLabel7 = new JLabel();
    private JScrollPane jScrollPaneCat = new JScrollPane();
    private JList jListCategorie = new JList();
    private JPanel jPanel2 = new JPanel();
    private GridLayout gridLayout1 = new GridLayout();
    private JRadioButton jRadioButtonEtat1 = new JRadioButton();
    private JRadioButton jRadioButtonEtat2 = new JRadioButton();
    private JRadioButton jRadioButtonEtat3 = new JRadioButton();
    private JRadioButton jRadioButtonEtat4 = new JRadioButton();
    private JPanel jPanel3 = new JPanel();
    private JButton jButtonSave = new JButton();
    private JButton jButtonCancel = new JButton();
    private ButtonGroup buttonGroupEtat = new ButtonGroup();
    private JLabel jLabel8 = new JLabel();
    private JPanel jPanel4 = new JPanel();
    private GridLayout gridLayout2 = new GridLayout();
    private ButtonGroup buttonGroupPri = new ButtonGroup();
    private JRadioButton jRadioButtonPri1 = new JRadioButton();
    private JRadioButton jRadioButtonPri2 = new JRadioButton();
    private JRadioButton jRadioButtonPri3 = new JRadioButton();
    private JButton jButtonViewEtabs = new JButton();
    private JButton jButtonViewContacts = new JButton();
    private GridBagLayout gridBagLayout1 = new GridBagLayout();
    private UndoManager undoManagerProb = new UndoManager();
    private JPopupMenu jPopupMenuProb = new JPopupMenu();
    private JMenuItem jMenuItemUndoProb = new JMenuItem();
    private JMenuItem jMenuItemRedoProb = new JMenuItem();
    private JMenuItem jMenuItemPasteProb;
    private JMenuItem jMenuItemCutProb;
    private JMenuItem jMenuItemCopyProb;
    private UndoManager undoManagerSol = new UndoManager();
    private JPopupMenu jPopupMenuSol = new JPopupMenu();
    private JMenuItem jMenuItemUndoSol = new JMenuItem();
    private JMenuItem jMenuItemRedoSol = new JMenuItem();
    private JMenuItem jMenuItemPasteSol;
    private JMenuItem jMenuItemCutSol;
    private JMenuItem jMenuItemCopySol;

/*
    public EditerIHM()//pour le concepteur DANGEREUX
    {
	try {
	    jbInit();
	} catch (Exception ex) {
	    ex.printStackTrace();
	}

	pack();
    }
*/

    public EditerIHM(RechercherEtabIHM ctlRechEtab,RechercherContactIHM ctlRechContact)
    {
        cf.registerIHM(this);
        setSize(cf.getEditSize());
        setLocation(cf.getEditLocation());
	m_ctlRechEtab=ctlRechEtab;
	m_ctlRechContact=ctlRechContact;

        //Classe de controle mutualise
        m_ctlEditer=cf.getEditerCTL();
        m_ctlRechercher=cf.getRechercherCTL();

        try
        {
            popupMenuInit();
            jbInit();
            keyBindingInit();
        }
        catch(Exception e)
        {
            Tools.internalError(e,"",cf.debug(),true);
        }

        //remplissage des catgories
        jListCategorie.setListData(m_ctlRechercher.getCategories());
    }

    private void jbInit() throws Exception
    {
        jPanel1.setLayout(gridBagLayout1);
        jLabel4.setText("Rappel du problme :");
        jLabel5.setText("Cause  / Solution :");
        jLabel6.setText("Catgorie :");
        jLabel7.setText("Etat :");
        jPanel2.setLayout(gridLayout1);
        gridLayout1.setRows(4);
        gridLayout1.setColumns(1);
        jRadioButtonEtat1.setText("A traiter");
        jRadioButtonEtat1.setActionCommand("1");
        jRadioButtonEtat1.setMnemonic('t');
        jRadioButtonEtat2.setSelected(true);
        jRadioButtonEtat2.setText("En cours");
        jRadioButtonEtat2.setMnemonic('2');
        jRadioButtonEtat2.setMnemonic('c');
        jRadioButtonEtat2.setActionCommand("2");
        jRadioButtonEtat3.setText("En attente");
        jRadioButtonEtat3.setMnemonic('a');
        jRadioButtonEtat3.setActionCommand("3");
        jRadioButtonEtat4.setText("Rsolut");
        jRadioButtonEtat4.setMnemonic('r');
        jRadioButtonEtat4.setActionCommand("4");
        jButtonSave.setText("Enregistrer");
        jButtonSave.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                jButtonSave_actionPerformed(e);
            }
        });
        jButtonCancel.setText("Annuler");
        jButtonCancel.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                jButtonCancel_actionPerformed(e);
            }
        });
        this.setTitle("Edition d\'une demande");
        this.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
        this.addWindowListener(new java.awt.event.WindowAdapter()
        {
            public void windowClosing(WindowEvent e)
            {
                this_windowClosing(e);
            }
        });
        jListCategorie.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        jTextAreaProb.setWrapStyleWord(true);
        jTextAreaProb.setLineWrap(true);
        jTextAreaProb.getDocument().addUndoableEditListener(new UndoableEditListener()
        {
            public void undoableEditHappened(UndoableEditEvent e)
            {
                undoManagerProb.addEdit(e.getEdit());
            }
        });
        jTextAreaSol.setWrapStyleWord(true);
        jTextAreaSol.setLineWrap(true);
        jTextAreaSol.getDocument().addUndoableEditListener(new UndoableEditListener()
        {
            public void undoableEditHappened(UndoableEditEvent e)
            {
                undoManagerSol.addEdit(e.getEdit());
            }
        });
        jLabel8.setText("Priorit :");
        jPanel4.setLayout(gridLayout2);
        gridLayout2.setRows(3);
        gridLayout2.setColumns(1);
        jRadioButtonPri1.setText("Haute");
        jRadioButtonPri1.setMnemonic('h');
        jRadioButtonPri1.setActionCommand("1");
        jRadioButtonPri2.setText("Normale");
        jRadioButtonPri2.setMnemonic('n');
        jRadioButtonPri2.setActionCommand("2");
        jRadioButtonPri3.setText("Basse");
        jRadioButtonPri3.setMnemonic('b');
        jRadioButtonPri3.setActionCommand("3");
        jButtonViewEtabs.setMinimumSize(new Dimension(17, 20));
        jButtonViewEtabs.setPreferredSize(new Dimension(17, 20));
        jButtonViewEtabs.setMargin(new Insets(2, 2, 2, 2));
        jButtonViewEtabs.setText("?");
        jButtonViewEtabs.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                jButtonViewEtab_actionPerformed(e);
            }
        });
        jButtonViewContacts.setMinimumSize(new Dimension(17, 20));
        jButtonViewContacts.setPreferredSize(new Dimension(17, 20));
        jButtonViewContacts.setMargin(new Insets(2, 2, 2, 2));
        jButtonViewContacts.setText("?");
        jButtonViewContacts.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                jButtonViewContact_actionPerformed(e);
            }
        });
        jScrollPaneCat.setMinimumSize(new Dimension(24, 90));
        jScrollPaneCat.setPreferredSize(new Dimension(260, 90));
        this.getContentPane().add(jPanel1, BorderLayout.CENTER);
        jPanel1.add(jLabel4, new GridBagConstraints(0, 3, 4, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(12, 6, 0, 0), 0, 0));
        jPanel1.add(jScrollPaneSol, new GridBagConstraints(0, 6, 4, 1, 1.0, 0.7
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(6, 16, 0, 16), 0, 0));
        jPanel1.add(jScrollPaneProb, new GridBagConstraints(0, 4, 4, 1, 1.0, 0.3
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(6, 16, 0, 16), 0, 0));
        jPanel1.add(jLabel5, new GridBagConstraints(0, 5, 4, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(6, 6, 0, 0), 0, 0));
        jPanel1.add(jPanel4, new GridBagConstraints(3, 8, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTH, GridBagConstraints.NONE, new Insets(3, 20, 0, 20), 0, -17));
        jPanel4.add(jRadioButtonPri1, null);
        jPanel4.add(jRadioButtonPri2, null);
        jPanel4.add(jRadioButtonPri3, null);
        jPanel1.add(jScrollPaneCat, new GridBagConstraints(0, 8, 2, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(3, 16, 0, 0), 0, 0));
        jPanel1.add(jButtonViewEtabs, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(6, 9, 0, 0), 0, 0));
        jPanel1.add(jButtonViewContacts, new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(3, 9, 0, 0), 0, 0));
        jPanel1.add(jLabelContacts, new GridBagConstraints(1, 2, 3, 1, 1.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(3, 10, 0, 6), 0, 0));
        jPanel1.add(jLabelEtabs, new GridBagConstraints(1, 1, 3, 1, 1.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(6, 10, 0, 6), 0, 0));
        jPanel1.add(jLabelDate, new GridBagConstraints(0, 0, 4, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(6, 6, 0, 6), 0, 0));
        jPanel1.add(jLabel6, new GridBagConstraints(0, 7, 2, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(6, 6, 0, 0), 0, 0));
        jPanel1.add(jLabel7, new GridBagConstraints(2, 7, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(6, 6, 0, 0), 0, 0));
        jPanel1.add(jLabel8, new GridBagConstraints(3, 7, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(6, 6, 0, 0), 0, 0));
        jPanel1.add(jPanel2, new GridBagConstraints(2, 8, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTH, GridBagConstraints.NONE, new Insets(3, 20, 12, 0), 0, -20));
        jPanel2.add(jRadioButtonEtat1, null);
        jPanel2.add(jRadioButtonEtat2, null);
        jPanel2.add(jRadioButtonEtat3, null);
        jPanel2.add(jRadioButtonEtat4, null);
        jPanel1.add(jPanel3, new GridBagConstraints(0, 9, 4, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(6, 0, 6, 0), 0, 0));
        jPanel3.add(jButtonSave, null);
        jPanel3.add(jButtonCancel, null);
        jScrollPaneCat.getViewport().add(jListCategorie, null);
        jScrollPaneProb.getViewport().add(jTextAreaProb, null);
        jScrollPaneSol.getViewport().add(jTextAreaSol, null);
        buttonGroupEtat.add(jRadioButtonEtat1);
        buttonGroupEtat.add(jRadioButtonEtat2);
        buttonGroupEtat.add(jRadioButtonEtat3);
        buttonGroupEtat.add(jRadioButtonEtat4);
        buttonGroupPri.add(jRadioButtonPri1);
        buttonGroupPri.add(jRadioButtonPri2);
        buttonGroupPri.add(jRadioButtonPri3);
    }

    private void keyBindingInit() throws Exception
    {
        //Keys binding gnraux
        InputMap inputMap=jPanel1.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW);
        ActionMap actionMap=jPanel1.getActionMap();
        inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER,0),ACTION_SAVE);
        inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_F10,0),ACTION_SAVE);
        inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE,0),ACTION_CANCEL);
        inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_F7,0),ACTION_VIEWCT);
        inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_F6,0),ACTION_VIEWET);
        inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_Q,ActionEvent.CTRL_MASK),ACTION_FOCUSCAT);
        inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_S,ActionEvent.CTRL_MASK),ACTION_FOCUSSOL);
	inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_P,ActionEvent.CTRL_MASK),ACTION_FOCUSPROB);
        actionMap.put(ACTION_SAVE,new AbstractAction()
        {
            public void actionPerformed(ActionEvent e)
            {
		if(jButtonSave.isEnabled())
                    jButtonSave_actionPerformed(e);
            }
        });
        actionMap.put(ACTION_CANCEL,new AbstractAction()
        {
            public void actionPerformed(ActionEvent e)
            {
		if(confirmCancel())
		    jButtonCancel_actionPerformed(e);
            }
        });
        actionMap.put(ACTION_VIEWCT,new AbstractAction()
        {
            public void actionPerformed(ActionEvent e)
            {
                jButtonViewContact_actionPerformed(e);
            }
        });
        actionMap.put(ACTION_VIEWET,new AbstractAction()
        {
            public void actionPerformed(ActionEvent e)
            {
                jButtonViewEtab_actionPerformed(e);
            }
        });
        actionMap.put(ACTION_FOCUSCAT,new AbstractAction()
        {
            public void actionPerformed(ActionEvent e)
            {
                jListCategorie.requestFocusInWindow();
            }
        });
        actionMap.put(ACTION_FOCUSSOL,new AbstractAction()
        {
            public void actionPerformed(ActionEvent e)
            {
                jTextAreaSol.requestFocusInWindow();
            }
        });
	actionMap.put(ACTION_FOCUSPROB,new AbstractAction()
	{
	    public void actionPerformed(ActionEvent e)
	    {
		jTextAreaProb.requestFocusInWindow();
	    }
	});

        //Keybinding textArea undo/redo
        jTextAreaProb.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_Z,ActionEvent.CTRL_MASK),ACTION_UNDOPROB);
        jTextAreaProb.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_Y,ActionEvent.CTRL_MASK),ACTION_REDOPROB);
        jTextAreaProb.getActionMap().put(ACTION_UNDOPROB,new AbstractAction()
        {
            public void actionPerformed(ActionEvent e)
            {
               jMenuItemUndoProb_actionPerformed(e);
            }
        });
        jTextAreaProb.getActionMap().put(ACTION_REDOPROB,new AbstractAction()
        {
            public void actionPerformed(ActionEvent e)
            {
                jMenuItemRedoProb_actionPerformed(e);
            }
        });
        jTextAreaSol.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_Z,ActionEvent.CTRL_MASK),ACTION_UNDOSOL);
        jTextAreaSol.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_Y,ActionEvent.CTRL_MASK),ACTION_REDOSOL);
        jTextAreaSol.getActionMap().put(ACTION_UNDOSOL,new AbstractAction()
        {
            public void actionPerformed(ActionEvent e)
            {
               jMenuItemUndoSol_actionPerformed(e);
            }
        });
        jTextAreaSol.getActionMap().put(ACTION_REDOSOL,new AbstractAction()
        {
            public void actionPerformed(ActionEvent e)
            {
                jMenuItemRedoSol_actionPerformed(e);
            }
        });
    }

    private void popupMenuInit() throws Exception
    {
        //Cration de la hashmap des actions de jTextAreaProb
        HashMap hmProb = new HashMap();
        Action[] a = jTextAreaProb.getActions();
        for(int i=0; i<a.length; hmProb.put(a[i].getValue(Action.NAME), a[i++]));

        //Cration de la hashmap des actions de jTextAreaSol
        HashMap hmSol = new HashMap();
        Action[] b = jTextAreaSol.getActions();
        for(int i=0; i<b.length; hmSol.put(b[i].getValue(Action.NAME), b[i++]));

        //cration du menu popup prob
        jMenuItemUndoProb.setText("Annuler");
        jMenuItemUndoProb.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                jMenuItemUndoProb_actionPerformed(e);
            }
        });

        jMenuItemRedoProb.setText("Rtablir");
        jMenuItemRedoProb.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                jMenuItemRedoProb_actionPerformed(e);
            }
        });

        jPopupMenuProb.add(jMenuItemUndoProb);
        jPopupMenuProb.add(jMenuItemRedoProb);
        jPopupMenuProb.addSeparator();

        //ajout des actions prdfinies dans le menu prob
        jMenuItemCutProb=jPopupMenuProb.add((Action)hmProb.get(DefaultEditorKit.cutAction));
        jMenuItemCutProb.setText("Couper");
        jMenuItemCopyProb=jPopupMenuProb.add((Action)hmProb.get(DefaultEditorKit.copyAction));
        jMenuItemCopyProb.setText("Copier");
        jMenuItemPasteProb=jPopupMenuProb.add((Action)hmProb.get(DefaultEditorKit.pasteAction));
        jMenuItemPasteProb.setText("Coller");
        jPopupMenuProb.addSeparator();
        jPopupMenuProb.add((Action)hmProb.get(DefaultEditorKit.selectAllAction)).setText("Slectionner tout");

        //cration du menu popup sol
        jMenuItemUndoSol.setText("Annuler");
        jMenuItemUndoSol.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                jMenuItemUndoSol_actionPerformed(e);
            }
        });

        jMenuItemRedoSol.setText("Rtablir");
        jMenuItemRedoSol.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                jMenuItemRedoSol_actionPerformed(e);
            }
        });

        jPopupMenuSol.add(jMenuItemUndoSol);
        jPopupMenuSol.add(jMenuItemRedoSol);
        jPopupMenuSol.addSeparator();

        //ajout des actions prdfinies dans le menu sol
        jMenuItemCutSol=jPopupMenuSol.add((Action)hmSol.get(DefaultEditorKit.cutAction));
        jMenuItemCutSol.setText("Couper");
        jMenuItemCopySol=jPopupMenuSol.add((Action)hmSol.get(DefaultEditorKit.copyAction));
        jMenuItemCopySol.setText("Copier");
        jMenuItemPasteSol=jPopupMenuSol.add((Action)hmSol.get(DefaultEditorKit.pasteAction));
        jMenuItemPasteSol.setText("Coller");
        jPopupMenuSol.addSeparator();
        jPopupMenuSol.add((Action)hmSol.get(DefaultEditorKit.selectAllAction)).setText("Slectionner tout");

        //ouverture du popup prob
        jTextAreaProb.addMouseListener(new MouseAdapter()
        {
            public void mousePressed(MouseEvent e)
            {
                maybeShowPopup(e);
            }

            public void mouseReleased(MouseEvent e)
            {
                maybeShowPopup(e);
            }

            private void maybeShowPopup(MouseEvent e)
            {
                if(e.isPopupTrigger())
                {
                    //activation de "coller" si texte prsent dans le presse papier
                    try
                    {
                        String st = (String)Toolkit.getDefaultToolkit().getSystemClipboard().getContents(null).getTransferData(DataFlavor.stringFlavor);
                        jMenuItemPasteProb.setEnabled(st.length()>0 && !m_bReadOnly);
                    }
                    catch(Exception ex)
                    {
                        jMenuItemPasteProb.setEnabled(false);
                    }

                    //activation de "couper" "copier" si texte slectionn
                    boolean textSelected = jTextAreaProb.getSelectedText()!=null;
                    jMenuItemCopyProb.setEnabled(textSelected);
                    jMenuItemCutProb.setEnabled(textSelected && !m_bReadOnly);

                    //activation de "annuler" "refaire" si possible
                    jMenuItemUndoProb.setEnabled(undoManagerProb.canUndo() && !m_bReadOnly);
                    jMenuItemRedoProb.setEnabled(undoManagerProb.canRedo() && !m_bReadOnly);

                    jTextAreaProb.requestFocus();
                    jPopupMenuProb.show(e.getComponent(),e.getX(), e.getY());
                }
            }
        });

        //ouverture du popup sol
        jTextAreaSol.addMouseListener(new MouseAdapter()
        {
            public void mousePressed(MouseEvent e)
            {
                maybeShowPopup(e);
            }

            public void mouseReleased(MouseEvent e)
            {
                maybeShowPopup(e);
            }

            private void maybeShowPopup(MouseEvent e)
            {
                if(e.isPopupTrigger())
                {
                    //activation de "coller" si texte prsent dans le presse papier
                    try
                    {
                        String st = (String)Toolkit.getDefaultToolkit().getSystemClipboard().getContents(null).getTransferData(DataFlavor.stringFlavor);
                        jMenuItemPasteSol.setEnabled(st.length()>0 && !m_bReadOnly);
                    }
                    catch(Exception ex)
                    {
                        jMenuItemPasteSol.setEnabled(false);
                    }

                    //activation de "couper" "copier" si texte slectionn
                    boolean textSelected = jTextAreaSol.getSelectedText()!=null;
                    jMenuItemCopySol.setEnabled(textSelected);
                    jMenuItemCutSol.setEnabled(textSelected && !m_bReadOnly);

                    //activation de "annuler" "refaire" si possible
                    jMenuItemUndoSol.setEnabled(undoManagerSol.canUndo() && !m_bReadOnly);
                    jMenuItemRedoSol.setEnabled(undoManagerSol.canRedo() && !m_bReadOnly);

                    jTextAreaSol.requestFocus();
                    jPopupMenuSol.show(e.getComponent(),e.getX(), e.getY());
                }
            }
        });
    }

    //pop-up de confirmation d'enregistrement
    private boolean confirm()
    {
       return JOptionPane.showConfirmDialog(this,"Etes vous sr de vouloir enregistrer\nces donnes ?",
                       "Confirmation de mise  jour",JOptionPane.YES_NO_OPTION,
                       JOptionPane.WARNING_MESSAGE)==JOptionPane.YES_OPTION;
    }

    //pop-up de confirmation d'annulation
    private boolean confirmCancel()
    {
       return JOptionPane.showConfirmDialog(this,"Etes vous sr de vouloir annuler sans enregistrer ?",
		       "Confirmation d'annulation",JOptionPane.YES_NO_OPTION,
		       JOptionPane.WARNING_MESSAGE)==JOptionPane.YES_OPTION;
    }

    private void afficherDemande(int demandeID)
    {
        int i;
        DemandeDATA dem=m_ctlEditer.getDemande(demandeID);
	m_dDemande=dem;

        //Le texte
        jTextAreaProb.setText(dem.getProb());
        jTextAreaSol.setText(dem.getSol());

        //flush des tampons undo
        undoManagerProb.discardAllEdits();
        undoManagerSol.discardAllEdits();

        //positionnement du ScrollPane en haut
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                jScrollPaneProb.getViewport().setViewPosition(new Point(0,0));
            }
        });

        //l'etat
        switch(dem.getEtat())
        {
            case 1: jRadioButtonEtat1.setSelected(true);    break;  //A traiter
            case 2: jRadioButtonEtat2.setSelected(true);    break;  //En cours
            case 3: jRadioButtonEtat3.setSelected(true);    break;  //En attente
            default:                                    //Rsolut impossible -> dans ARCHIVE
        }

        //la priorit
        switch(dem.getPriorite())
        {
            case 1: jRadioButtonPri1.setSelected(true);    break;  //Haute
            case 2: jRadioButtonPri2.setSelected(true);    break;  //Normale
            case 3: jRadioButtonPri3.setSelected(true);    break;  //Basse
            default:
        }

        //La catgorie
        ListModel model=jListCategorie.getModel();
        for(i=model.getSize()-1;i>=0 && !dem.getCategorie().equals(model.getElementAt(i));i--);  //parcour en sens inverse pour slectionner 'aucune' en cas de problme
        jListCategorie.setSelectedIndex(i);

        //les labels
        String labelEt=new String("Etablissements");
        Vector etabs=dem.getEtabs();
        for(i=0;i<etabs.size();labelEt+=" : "+(EtabDATA)etabs.get(i++));
        jLabelEtabs.setText(labelEt);

        String labelCt=new String("Contacts");
        Vector contacts=dem.getContacts();
        for(i=0;i<contacts.size();labelCt+=" : "+(ContactDATA)contacts.get(i++));
        jLabelContacts.setText(labelCt);

        jLabelDate.setText("Ouvert le "+dem.getDate()
                           + " par "+dem.getUser());

        this.setTitle("Edition de la demande n " + dem.getID());
    }

    private void prepareAfficherDemande(DemandeDATA demande,boolean readOnly)
    {
	if(m_dDemande!=null)    //si une demande en cours d'dition
	{
	    jButtonSave_actionPerformed(null);
	    jButtonCancel_actionPerformed(null);    //cancel si confirm=non
	}

	//verrouillage des lments
	m_bReadOnly=readOnly;
	jRadioButtonEtat1.setEnabled(!readOnly);
	jRadioButtonEtat2.setEnabled(!readOnly);
	jRadioButtonEtat3.setEnabled(!readOnly);
	jRadioButtonEtat4.setEnabled(!readOnly);
	jRadioButtonPri1.setEnabled(!readOnly);
	jRadioButtonPri2.setEnabled(!readOnly);
	jRadioButtonPri3.setEnabled(!readOnly);
	jTextAreaProb.setEditable(!readOnly);
	jTextAreaSol.setEditable(!readOnly);
	jButtonSave.setEnabled(!readOnly);
	jListCategorie.setEnabled(!readOnly);

	//Edition de la demande
	afficherDemande(demande.getID());
	if(!readOnly) m_ctlEditer.lock(demande);
	setVisible(true);
	jTextAreaSol.requestFocusInWindow();
    }

    public void editer(DemandeDATA demande)
    {
        String lockUser=m_ctlEditer.getLockUser(demande.getID());
        if(lockUser==null)
        {
	    prepareAfficherDemande(demande,false);  //dition complte
        }
        else
        {
	    if(lockUser.equalsIgnoreCase(cf.getLogin()))
	    {
                if(m_dDemande!=null && m_dDemande.equals(demande))
                {
                    setVisible(true);   //demande dj en cours d'dition par l'utilisateur
                }
                else //problme d'intgrit de la base (flag non remis  0)
                {
                    prepareAfficherDemande(demande,false);  //dition complte
                }
	    }
	    else
	    {
		if(JOptionPane.showOptionDialog(null,"Demande en cours d'dition par "+lockUser+".",
			   "Edition en cours",0,JOptionPane.INFORMATION_MESSAGE,
			   null,OPTIONS,OPTIONS[0])==1)
		{
		    prepareAfficherDemande(demande,true);  //dition en lecture seule
		}
	    }
        }
    }

    void jButtonSave_actionPerformed(ActionEvent e)
    {
	if(!m_bReadOnly)
	{
	    int etat=Integer.parseInt(buttonGroupEtat.getSelection().getActionCommand());
	    int pri=Integer.parseInt(buttonGroupPri.getSelection().getActionCommand());
	    if(confirm())
	    {
		if(m_ctlEditer.updateDemande(m_dDemande,jTextAreaSol.getText(),jTextAreaProb.getText(),
					     (CategorieDATA)jListCategorie.getSelectedValue(),pri,etat))
		{
		    if(etat==4) m_dDemande=null; //si etat=rsolut : unlock dans updateDemande (changement table)
		    jButtonCancel_actionPerformed(null);    //nettoyage de la fentre
		}
	    }
	}
    }

    void jButtonCancel_actionPerformed(ActionEvent e)
    {
        setVisible(false);
        if(!m_bReadOnly) m_ctlEditer.unlock(m_dDemande);
        m_dDemande=null;
    }

    void this_windowClosing(WindowEvent e)
    {
        jButtonCancel_actionPerformed(null);
    }

    void jButtonViewEtab_actionPerformed(ActionEvent e)
    {
	if(m_dDemande!=null)
            m_ctlRechEtab.show(m_dDemande.getEtabs());
    }

    void jButtonViewContact_actionPerformed(ActionEvent e)
    {
	if(m_dDemande!=null)
	    m_ctlRechContact.show(m_dDemande.getContacts());
    }

    void jMenuItemUndoProb_actionPerformed(ActionEvent e)
    {
        try
        {
            undoManagerProb.undo();
        }
        catch (CannotUndoException ex) {}
    }

    void jMenuItemRedoProb_actionPerformed(ActionEvent e)
    {
        try
        {
            undoManagerProb.redo();
        }
        catch(CannotRedoException ex) {}
    }

    void jMenuItemUndoSol_actionPerformed(ActionEvent e)
    {
        try
        {
            undoManagerSol.undo();
        }
        catch (CannotUndoException ex) {}
    }

    void jMenuItemRedoSol_actionPerformed(ActionEvent e)
    {
        try
        {
            undoManagerSol.redo();
        }
        catch(CannotRedoException ex) {}
    }

}
