/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2006  Nicolas AGIUS                                     *
 *                                                                         *
 *   This file is part of ECaMM.                                           *
 *                                                                         *
 *   ECaMM is free software; you can redistribute it and/or modify it      *
 *   under the terms of the GNU General Public License as published by     *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   any later version.                                                    *
 *                                                                         *
 *   ECaMM is distributed in the hope that it will be useful, but          *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston,                *
 *    MA  02111-1307, USA.                                                 *
 *                                                                         *
 *   For more information about these matters, see the file named COPYING. *
 *                                                                         *
 *   Mail : nicolas_agius@yahoo.fr                                         *
 *                                                                         *
 ***************************************************************************/
package ecamm;

/**
 * Titre :  AjouterCTL [ECaMM]
 * Description : Classe de controle de recherche de donnes
 * Copyright : GPL v2.0 - 2005~2006
 * @author : Nicolas AGIUS
 * @version 1.1
 */

import java.util.Date;
import java.util.GregorianCalendar;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Calendar;

public class RechercherCTL
{
    private boolean m_bDebug;
    private ConnectionBase m_DB;

    //constante pour le type de Donne
    private static final int COMMUNE=0;
    private static final int CIRCO=1;
    private static final int ETAB=2;
    private static final int CONTACT=3;
    private static final int DEMANDE=4;
    private static final int CATEGORIE=5;
    private static final int ARCHIVE=6;

    public RechercherCTL(ConnectionBase maBase, boolean debug)
    {
        m_DB=maBase;
        m_bDebug=debug;
    }

    //rcupre l'heure du serveur
    public Date getServerDate()
    {
        try
        {
            ResultSet rs = m_DB.ExecuteQuery("select now() as date");
            rs.first();
            return rs.getDate("date", new GregorianCalendar());
        }
        catch(SQLException e)
        {
            Tools.sqlError(e,m_bDebug);
        }
        catch(Exception e)
        {
            Tools.internalError(e,"",m_bDebug,false);
        }

        return new Date(); //retoune la date du client en cas d'erreur
    }

    public CommuneDATA[] getCommunes()
    {
        return (CommuneDATA[])findData("select * from COMMUNE,CIRCO where CI_ID=CO_CI_ID order by CO_NOM",COMMUNE);
    }

    public CircoDATA[] getCircos()
    {
        return (CircoDATA[])findData("select * from CIRCO order by CI_NOM",CIRCO);
    }

    public EtabDATA[] getEtabs(String motif)
    {
        return (EtabDATA[])findData("select * from ETABLISSEMENT,COMMUNE,CIRCO where ET_CO_ID=CO_ID and CO_CI_ID=CI_ID and ET_NOM like '%"+motif+"%' order by ET_NOM",ETAB);
    }

    public EtabDATA[] getEtabs(String motif, boolean parNom, CommuneDATA com)
    {
	String req="select * from ETABLISSEMENT,COMMUNE,CIRCO where ET_CO_ID=CO_ID and CO_CI_ID=CI_ID";
	if(parNom)
	   req += " and ET_NOM like '%"+motif+"%'";
	else
	   req += " and ET_RNE like '%"+motif+"%'";

       if(!com.isVide())
	   req += " and CO_ID = " + com.getID();

	return (EtabDATA[])findData(req + " order by ET_NOM",ETAB);
    }

    public EtabDATA getLastNewEtab()
    {
        EtabDATA[] ets = (EtabDATA[])findData("select * from ETABLISSEMENT,COMMUNE,CIRCO where ET_CO_ID=CO_ID and CO_CI_ID=CI_ID and ET_ID=LAST_INSERT_ID()",ETAB);
        return ets[0];    //un seul etablissement
    }

    public ContactDATA[] getContacts(String motif)
    {
        return (ContactDATA[])findData("select * from CONTACT,CIRCO where CT_CI_ID=CI_ID and ( CT_NOM like '%"+motif+"%' or CT_PRENOM like '%"+motif+"%') order by CT_NOM",CONTACT);
    }

    public ContactDATA[] getContacts(String motif, boolean parNom, CircoDATA cir)
    {
	String req="select * from CONTACT,CIRCO where CT_CI_ID=CI_ID";
	if(parNom)
	   req += " and ( CT_NOM like '%"+motif+"%' or CT_PRENOM like '%"+motif+"%')";
	else
	   req += " and CT_ORGA like '%"+motif+"%'";

       if(!cir.isVide())
	   req += " and CI_ID = " + cir.getID();

	return (ContactDATA[])findData(req + " order by CT_NOM",CONTACT);
    }

    public ContactDATA getLastNewContact()
    {
        ContactDATA[] cts = (ContactDATA[])findData("select * from CONTACT,CIRCO where CT_CI_ID=CI_ID and CT_ID=LAST_INSERT_ID()",CONTACT);
        return cts[0];    //un seul contact
    }

    public CategorieDATA[] getCategories()
    {
        return (CategorieDATA[])findData("select * from CATEGORIE",CATEGORIE);
    }

    public DemandeDATA[] getDemandes()
    {
	//ncssite Mysql 4.1.11 minimum (sous-requtes)
	return (DemandeDATA[])findData("select D_ID,D_PRI,D_TYPE,D_PROB,D_ETAT,ET_NOM,CO_NOM from DEMANDE left join (select * from ETD group by ETD_D_ID having count(*) = 1) as unik on D_ID=unik.ETD_D_ID left join ETABLISSEMENT on ETD_ET_ID=ET_ID left join COMMUNE on CO_ID=ET_CO_ID order by D_DATEO",DEMANDE);

        //return (DemandeDATA[])findData("select D_ID,D_PRI,D_TYPE,D_PROB,D_ETAT,ET_NOM from DEMANDE left join (select * from ETD group by ETD_D_ID having count(*) = 1) as unik on D_ID=unik.ETD_D_ID left join ETABLISSEMENT on ETD_ET_ID=ET_ID order by D_DATEO",DEMANDE);
	//return (DemandeDATA[])findData("select D_ID,D_PRI,D_TYPE,D_PROB,D_ETAT from DEMANDE",DEMANDE);
    }

    public DemandeDATA getDemande(int numDossier)
    {
        DemandeDATA[] dem = (DemandeDATA[])findData("select D_ID from DEMANDE where D_ID = " + numDossier,DEMANDE);

        if(dem.length!=0)
            return dem[0];
        else
            return null;
    }

    public ArchiveDATA[] getArchives(int numDossier)
    {
        //ne retourne qu'une Archive dans le tableau :
        //normalement pas de doublon car D_ID unique pour mysql
        //mais peut-tre pas si manipulation directe sur la base

	return (ArchiveDATA[])findData("select A_ID,A_PROB,A_SOL,A_TYPE,A_NUMDO,CA_NOM,CA_ID,"
	    +"DATE_FORMAT(A_DATEO,'%d/%m/%Y') as DATEO,DATE_FORMAT(A_DATEC,'%d/%m/%Y') as DATEC,"
	    +"UO.U_LOGIN as LOGIN_UO, UC.U_LOGIN as LOGIN_UC "
	    +"from ARCHIVE,CATEGORIE, USER as UO, USER as UC where UO.U_ID=A_U_IDO and UC.U_ID=A_U_IDC "
	    +"and CA_ID=A_CA_ID and A_NUMDO=" + numDossier,ARCHIVE);
    }

    public ArchiveDATA[] getArchives(ContactDATA ct)
    {
	return (ArchiveDATA[])findData("select A_ID,A_PROB,A_SOL,A_TYPE,A_NUMDO,CA_NOM,CA_ID,"
	    +"DATE_FORMAT(A_DATEO,'%d/%m/%Y') as DATEO,DATE_FORMAT(A_DATEC,'%d/%m/%Y') as DATEC,"
	    +"UO.U_LOGIN as LOGIN_UO, UC.U_LOGIN as LOGIN_UC "
	    +"from ARCHIVE,CATEGORIE,CTA, USER as UO, USER as UC where UO.U_ID=A_U_IDO and UC.U_ID=A_U_IDC "
	    +"and CA_ID=A_CA_ID and CTA_A_ID=A_ID and CTA_CT_ID=" + ct.getID(),ARCHIVE);
    }

    public ArchiveDATA[] getArchives(EtabDATA et)
    {
	return (ArchiveDATA[])findData("select A_ID,A_PROB,A_SOL,A_TYPE,A_NUMDO,CA_NOM,CA_ID,"
	    +"DATE_FORMAT(A_DATEO,'%d/%m/%Y') as DATEO,DATE_FORMAT(A_DATEC,'%d/%m/%Y') as DATEC,"
	    +"UO.U_LOGIN as LOGIN_UO, UC.U_LOGIN as LOGIN_UC "
	    +"from ARCHIVE,CATEGORIE,ETA, USER as UO, USER as UC where UO.U_ID=A_U_IDO and UC.U_ID=A_U_IDC "
	    +"and CA_ID=A_CA_ID and ETA_A_ID=A_ID and ETA_ET_ID=" + et.getID(),ARCHIVE);
    }

    public ArchiveDATA[] getArchives(boolean byText, String text, boolean fullTextOn,
				     boolean byCat, CategorieDATA cat,
				     boolean byDateU, int type, Date date,
				     boolean byDateI, Date date1, Date date2)
    {
	GregorianCalendar cal = new GregorianCalendar();
	String req="select A_ID,A_PROB,A_SOL,A_TYPE,A_NUMDO,CA_NOM,CA_ID,"
	    +"DATE_FORMAT(A_DATEO,'%d/%m/%Y') as DATEO,DATE_FORMAT(A_DATEC,'%d/%m/%Y') as DATEC,"
	    +"UO.U_LOGIN as LOGIN_UO, UC.U_LOGIN as LOGIN_UC "
	    +"from ARCHIVE,CATEGORIE,USER as UO, USER as UC where UO.U_ID=A_U_IDO and UC.U_ID=A_U_IDC "
	    +"and CA_ID=A_CA_ID";

	//recherche par motif fulltext
	if(byText && text.length()>1)
	{
            if(fullTextOn)    //mode FULLTEXT
            {
                req += " and match(A_PROB,A_SOL) against('" + text + "' in boolean mode)";
            }
            else             //mode normal (plus long, mais plus de positifs)
            {
                req += " and ( A_PROB like '%" + text + "%' or A_SOL like '%" + text + "%' )";
            }
	}

	//recherche par catgorie
	if(byCat && !cat.isVide())
	{
	    req += " and A_CA_ID="+cat.getID();
	}

	//recherche par date unique
	if(byDateU)
	{
	    cal.setTime(date);
	    req += " and datediff(A_DATEO,makedate("+cal.get(Calendar.YEAR)+","+cal.get(Calendar.DAY_OF_YEAR)+"))";
	    switch(type)
	    {
		case 0: req += "<0"; break; //avant
		case 1: req += "=0"; break; //pendant
		case 2: req += ">0"; break; //aprs
		default: req += "=0"; break;
	    }
	}

	//recherche par intervalle de dates
	if(byDateI)
	{
	    cal.setTime(date1);
	    req += " and datediff(A_DATEO,makedate("+cal.get(Calendar.YEAR)+","+cal.get(Calendar.DAY_OF_YEAR)+"))>=0";
	    cal.setTime(date2);
	    req += " and datediff(A_DATEO,makedate("+cal.get(Calendar.YEAR)+","+cal.get(Calendar.DAY_OF_YEAR)+"))<=0";
	}

	return (ArchiveDATA[])findData(req,ARCHIVE);
    }

    //passage par rfrence : pas besoin de retour
    public void fillContactEtabForArchive(ArchiveDATA arc)
    {
        if(!arc.isFull())
        {
            try
            {
                ResultSet rs2 = m_DB.ExecuteQuery(
                    "select * from CONTACT,CIRCO,CTA where CI_ID=CT_CI_ID and CT_ID=CTA_CT_ID and CTA_A_ID=" +
                    arc.getID());
                rs2.beforeFirst();
                while(rs2.next())arc.addContact(rs2);

                ResultSet rs3 = m_DB.ExecuteQuery("select * from ETABLISSEMENT,COMMUNE,CIRCO,ETA where CO_ID=ET_CO_ID and ET_ID=ETA_ET_ID and CO_CI_ID=CI_ID and ETA_A_ID=" +
                                                  arc.getID());
                rs3.beforeFirst();
                while(rs3.next())arc.addEtab(rs3);

                arc.setFull();
            }
            catch(SQLException e)
            {
                Tools.sqlError(e, m_bDebug);
            }
            catch(Exception e)
            {
                Tools.internalError(e, "", m_bDebug, false);
            }
        }
    }


    //lecture des donnes
    private Data[] findData(String req,int type)
    {
        int l;
        Data ret[];

        try
        {
            ResultSet rs = m_DB.ExecuteQuery(req);  //execute la requete

            //initialise le tableau  la bonne taille
            rs.last();
            l=rs.getRow();
            ret=initTab(type,l); //initialisation type pour cast

            //cration de chaque lment et remplissage des donnes
            rs.beforeFirst();
            for(int i=0;rs.next();i++)
            {
                switch(type)
                {
                    case COMMUNE:   ret[i]=new CommuneDATA(rs);  break;
                    case CIRCO:     ret[i]=new CircoDATA(rs);   break;
                    case ETAB:      ret[i]=new EtabDATA(rs);   break;
                    case CONTACT:   ret[i]=new ContactDATA(rs);   break;
                    case DEMANDE:   ret[i]=new DemandeDATA(rs);   break;
                    case CATEGORIE: ret[i]=new CategorieDATA(rs);   break;
		    case ARCHIVE:   ret[i]=new ArchiveDATA(rs);   break;
                }
            }

            return ret;
        }
        catch(SQLException e)
        {
            Tools.sqlError(e,m_bDebug);
        }
        catch(Exception e)
        {
            Tools.internalError(e,"",m_bDebug,false);
        }

        return initTab(type,0); //retour vide en cas d'erreur
    }

    //initialisation du tableau au type corespondant
    private Data[] initTab(int type, int l)
    {
        Data tab[];
        switch(type)
        {
            case COMMUNE:   tab=new CommuneDATA[l];  break;
            case CIRCO:     tab=new CircoDATA[l];   break;
            case ETAB:      tab=new EtabDATA[l];   break;
            case CONTACT:   tab=new ContactDATA[l];   break;
            case DEMANDE:   tab=new DemandeDATA[l];   break;
            case CATEGORIE: tab=new CategorieDATA[l];   break;
	    case ARCHIVE:   tab=new ArchiveDATA[l];   break;
            default:        tab=null;             break;
        }
        return tab;
    }
}
