/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2005  Nicolas AGIUS                                     *
 *                                                                         *
 *   This file is part of ECaMM.                                           *
 *                                                                         *
 *   ECaMM is free software; you can redistribute it and/or modify it      *
 *   under the terms of the GNU General Public License as published by     *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   any later version.                                                    *
 *                                                                         *
 *   ECaMM is distributed in the hope that it will be useful, but          *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston,                *
 *    MA  02111-1307, USA.                                                 *
 *                                                                         *
 *   For more information about these matters, see the file named COPYING. *
 *                                                                         *
 *   Mail : nicolas_agius@yahoo.fr                                         *
 *                                                                         *
 ***************************************************************************/
package ecamm;

/**
 * Titre :  RechercherContactIHM [ECaMM]
 * Description : Fentre de recherche d'un contact
 * Copyright : GPL v2.0 - 2005
 * @author : Nicolas AGIUS
 * @version 1.0
 */

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Vector;
import javax.swing.border.TitledBorder;
import javax.swing.event.ListSelectionEvent;

public class RechercherContactIHM extends JFrame
{
    //Constantes pour le keybinding
    private static final String ACTION_FIND = "find";
    private static final String ACTION_CLOSE = "close";
    private static final String ACTION_FOCUSLIST = "focusliste";
    private static final String ACTION_FOCUSFICHE = "focusfiche";
    private static final String ACTION_FOCUSFIND = "focusfind";
    private static final String ACTION_FOCUSCOMBO = "focuscombo";
    private static final String ACTION_OPENBOX = "openpopup";

    //variable de traitement
    private ConfigurationCTL cf=Main.cf;
    private RechercherCTL m_ctlRechercher;
    private RechercherDemandeIHM m_ihmRechercherDemande;
    private AjouterContactIHM m_ihmAddContact;
    private boolean m_bModeRecherche;   //dsactive la recherche lorsque affiche info depuis editerIHM

    //composant graphiques
    private ButtonGroup buttonGroup1 = new ButtonGroup();
    private GridBagLayout gridBagLayout1 = new GridBagLayout();
    private JPanel jPanel2 = new JPanel();
    private JPanel jPanel1 = new JPanel();
    private JList jListResultat = new JList();
    private JTextArea jTextAreaFiche = new JTextArea();
    private JScrollPane jScrollPaneFiche = new JScrollPane();
    private JLabel jLabelResultat = new JLabel();
    private JLabel jLabel1 = new JLabel();
    private JScrollPane jScrollPaneList = new JScrollPane();
    private JButton jButtonClose = new JButton();
    private JPanel jPanel3 = new JPanel();
    private JButton jButtonFind = new JButton();
    private GridLayout gridLayout1 = new GridLayout();
    private JRadioButton jRadioButtonNom = new JRadioButton();
    private JLabel jLabelCirco = new JLabel();
    private JRadioButton jRadioButtonOrga = new JRadioButton();
    private JComboBox jComboBoxCirco = new JComboBox();
    private JPanel jPanel4 = new JPanel();
    private JTextField jTextFieldFind = new JTextField();
    private GridBagLayout gridBagLayout2 = new GridBagLayout();
    private JPanel jPanel5 = new JPanel();
    private TitledBorder titledBorder1;
    private JButton jButtonViewDemandes = new JButton();
    private JButton jButtonEdit = new JButton();

    public RechercherContactIHM(AjouterContactIHM ihmAddContact)
    {
	cf.registerIHM(this);
	m_ihmAddContact=ihmAddContact;

        //Classe de controle mutualise
	m_ctlRechercher=cf.getRechercherCTL();

	try
	{
	    jbInit();
	}
	catch(Exception e)
	{
	    Tools.internalError(e,"",cf.debug(),true);
	}

	//remplissage de la combo box circos
	CircoDATA circos[]= m_ctlRechercher.getCircos();
	jComboBoxCirco.addItem(new CircoDATA());
	for(int i=0;i<circos.length;i++)
	    jComboBoxCirco.addItem(circos[i]);

	//dimensionnement
	pack();
	Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	Dimension frameSize = getSize();
	setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);
    }

    private void jbInit() throws Exception
    {
	titledBorder1 = new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(165, 163, 151)),"Contacts : ");
	jPanel4.setLayout(gridLayout1);
	jRadioButtonOrga.setText("Organisation");
	jRadioButtonOrga.setMnemonic('o');
	jLabelCirco.setText("Circonsciption :");
	jRadioButtonNom.setSelected(true);
	jRadioButtonNom.setText("Nom ou prnom");
	jRadioButtonNom.setMnemonic('n');
	jButtonViewDemandes.setMargin(new Insets(2, 8, 2, 8));
	jButtonViewDemandes.setText("Demandes...");
	jButtonViewDemandes.setMnemonic('d');
	jButtonViewDemandes.addActionListener(new java.awt.event.ActionListener()
	{
	    public void actionPerformed(ActionEvent e)
	    {
		jButtonViewDemandes_actionPerformed(e);
	    }
	});
	jButtonFind.setMargin(new Insets(2, 10, 2, 10));
        jButtonEdit.setMargin(new Insets(2, 8, 2, 8));
        jButtonEdit.setMnemonic('E');
        jButtonEdit.setText("Editer...");
        jButtonEdit.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                jButtonEdit_actionPerformed(e);
            }
        });
        buttonGroup1.add(jRadioButtonNom);
	buttonGroup1.add(jRadioButtonOrga);
	gridLayout1.setRows(2);
	gridLayout1.setColumns(1);
	jButtonFind.setText("Rechercher");
	jButtonFind.setMnemonic('r');
	jButtonFind.addActionListener(new java.awt.event.ActionListener()
	{
	    public void actionPerformed(ActionEvent e)
	    {
		jButtonFind_actionPerformed(e);
	    }
	});
	jPanel3.setLayout(gridBagLayout2);
	jPanel1.setLayout(gridBagLayout1);
	jTextAreaFiche.setEditable(false);
	jScrollPaneFiche.setPreferredSize(new Dimension(350,100));
	jScrollPaneFiche.setMinimumSize(new Dimension(350,100));
	jLabel1.setText("Fiche du contact :");
	jScrollPaneList.setPreferredSize(new Dimension(350,90));
	jButtonClose.setText("Fermer");
	jButtonClose.setMnemonic('f');
	jButtonClose.addActionListener(new java.awt.event.ActionListener()
	{
	    public void actionPerformed(ActionEvent e)
	    {
		jButtonClose_actionPerformed(e);
	    }
	});
	this.setTitle("Recherche de contacts");
	this.setDefaultCloseOperation(WindowConstants.HIDE_ON_CLOSE);
	jPanel3.setBorder(titledBorder1);
	jListResultat.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	jListResultat.addListSelectionListener(new javax.swing.event.ListSelectionListener()
	{
	    public void valueChanged(ListSelectionEvent e)
	    {
		jListResultat_valueChanged(e);
	    }
	});
        jListResultat.addMouseListener(new MouseAdapter()
        {
            public void mouseClicked(MouseEvent e)
            {
                jListResultat_mouseClicked(e);
            }
        });
	this.getContentPane().add(jPanel1, BorderLayout.CENTER);
        jPanel2.add(jButtonClose);
	jPanel4.add(jRadioButtonNom, null);
	jPanel4.add(jRadioButtonOrga, null);
        jPanel3.add(jTextFieldFind, new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0
	    ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(-6, 16, 0, 12), 0, 0));
	jPanel3.add(jPanel4, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0
	    ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(-10, 6, 0, 6), 0, 0));
	jPanel3.add(jComboBoxCirco, new GridBagConstraints(0, 2, 1, 1, 1.0, 0.0
	    ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 16, 0, 12), 0, 0));
	jPanel3.add(jButtonFind, new GridBagConstraints(1, 2, 1, 1, 0.0, 0.0
	    ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 3), 0, 0));
	jPanel3.add(jLabelCirco, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
	    ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(-6, 6, 6, 0), 0, 0));
	jPanel3.add(jPanel5, new GridBagConstraints(0, 3, 2, 1, 0.0, 0.0
	    ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
        jScrollPaneFiche.getViewport().add(jTextAreaFiche);
	jScrollPaneList.getViewport().add(jListResultat);
        jPanel1.add(jScrollPaneList, new GridBagConstraints(0, 3, 4, 1, 1.0, 0.8
            , GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(6, 16, 0, 16), 0, 0));
        jPanel1.add(jLabel1, new GridBagConstraints(0, 4, 2, 1, 0.0, 0.0
            , GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE, new Insets(6, 6, 6, 0), 0, 0));
        jPanel1.add(jScrollPaneFiche, new GridBagConstraints(0, 5, 4, 1, 1.0, 0.2
            , GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 16, 0, 16), 0, 0));
        jPanel1.add(jPanel2, new GridBagConstraints(0, 6, 4, 1, 0.0, 0.0
            , GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(3, 6, 0, 6), 0, 0));
        jPanel1.add(jLabelResultat, new GridBagConstraints(0, 2, 4, 1, 0.0, 0.0
            , GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(10, 6, 0, 6), 0, 0));
        jPanel1.add(jPanel3, new GridBagConstraints(0, 1, 4, 1, 1.0, 0.0
            , GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(3, 6, 6, 6), 0, 0));
        jPanel1.add(jButtonEdit, new GridBagConstraints(3, 4, 1, 1, 0.0, 0.0
            , GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(6, 0, 6, 20), 0, 0));
        jPanel1.add(jButtonViewDemandes, new GridBagConstraints(2, 4, 1, 1, 1.0, 0.0
            , GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(6, 0, 6, 6), 0, 0));

        //Keys binding gnraux
        InputMap inputMap=jPanel1.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW);
	ActionMap actionMap=jPanel1.getActionMap();
	inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER,0),ACTION_FIND);
	inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE,0),ACTION_CLOSE);
	inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_R,ActionEvent.CTRL_MASK),ACTION_FOCUSLIST);
	inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_F,ActionEvent.CTRL_MASK),ACTION_FOCUSFICHE);
	inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_E,ActionEvent.CTRL_MASK),ACTION_FOCUSFIND);
	inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_I,ActionEvent.CTRL_MASK),ACTION_FOCUSCOMBO);
	actionMap.put(ACTION_FIND,new AbstractAction()
	{
	    public void actionPerformed(ActionEvent e)
	    {
		jButtonFind_actionPerformed(e);
	    }
	});
	actionMap.put(ACTION_CLOSE,new AbstractAction()
	{
	    public void actionPerformed(ActionEvent e)
	    {
		jButtonClose_actionPerformed(e);
	    }
	});
	actionMap.put(ACTION_FOCUSLIST,new AbstractAction()
	{
	    public void actionPerformed(ActionEvent e)
	    {
		jListResultat.requestFocusInWindow();
	    }
	});
	actionMap.put(ACTION_FOCUSFICHE,new AbstractAction()
	{
	    public void actionPerformed(ActionEvent e)
	    {
		jTextAreaFiche.requestFocusInWindow();
	    }
	});
	actionMap.put(ACTION_FOCUSFIND,new AbstractAction()
	{
	    public void actionPerformed(ActionEvent e)
	    {
		jTextFieldFind.requestFocusInWindow();
	    }
	});
	actionMap.put(ACTION_FOCUSCOMBO,new AbstractAction()
	{
	    public void actionPerformed(ActionEvent e)
	    {
		jComboBoxCirco.requestFocusInWindow();
	    }
	});

	//key bindings comboBox
	jComboBoxCirco.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_SPACE,0),ACTION_OPENBOX);
	jComboBoxCirco.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER,0),ACTION_FIND);
	jComboBoxCirco.getActionMap().put(ACTION_FIND,new AbstractAction()
	{
	    public void actionPerformed(ActionEvent e)
	    {
		jButtonFind_actionPerformed(e);
	    }
	});
	jComboBoxCirco.getActionMap().put(ACTION_OPENBOX,new AbstractAction()
	{
	    public void actionPerformed(ActionEvent e)
	    {
		jComboBoxCirco.showPopup();
	    }
	});

	//Gestion du focus traversal
	this.setFocusTraversalPolicy(new FocusTraversalPolicy()
	{
	    public Component getComponentAfter(Container focusCycleRoot,Component a)
	    {
		if(m_bModeRecherche)
		{
		    if      (a.equals(jTextFieldFind))     return jRadioButtonNom;
		    else if (a.equals(jRadioButtonNom))    return jRadioButtonOrga;
		    else if (a.equals(jRadioButtonOrga))   return jComboBoxCirco;
		    else if (a.equals(jComboBoxCirco))     return jButtonFind;
		    else if (a.equals(jButtonFind))        return jListResultat;
		    else if (a.equals(jListResultat))      return jButtonViewDemandes;
		    else if (a.equals(jButtonViewDemandes))return jButtonEdit;
		    else if (a.equals(jButtonEdit))        return jTextAreaFiche;
		    else if (a.equals(jTextAreaFiche))     return jButtonClose;
		    else if (a.equals(jButtonClose))       return jTextFieldFind;
		    return jTextFieldFind;
		}
		else
		{
		    if      (a.equals(jListResultat))      return jButtonViewDemandes;
		    else if (a.equals(jButtonViewDemandes))return jButtonEdit;
		    else if (a.equals(jButtonEdit))        return jTextAreaFiche;
		    else if (a.equals(jTextAreaFiche))     return jButtonClose;
		    else if (a.equals(jButtonClose))       return jListResultat;
		    return jListResultat;
		}
	    }

	    public Component getComponentBefore(Container focusCycleRoot,Component a)
	    {
		if(m_bModeRecherche)
		{
		    if      (a.equals(jButtonClose))       return jTextAreaFiche;
		    else if (a.equals(jTextAreaFiche))     return jButtonEdit;
		    else if (a.equals(jButtonEdit))        return jButtonViewDemandes;
		    else if (a.equals(jButtonViewDemandes))return jListResultat;
		    else if (a.equals(jListResultat))      return jButtonFind;
		    else if (a.equals(jButtonFind))        return jComboBoxCirco;
		    else if (a.equals(jComboBoxCirco))     return jRadioButtonOrga;
		    else if (a.equals(jRadioButtonOrga))   return jRadioButtonNom;
		    else if (a.equals(jRadioButtonNom))    return jTextFieldFind;
		    else if (a.equals(jTextFieldFind))     return jButtonClose;
		    return jTextFieldFind;
		}
		else
		{
		    if      (a.equals(jButtonClose))       return jTextAreaFiche;
		    else if (a.equals(jTextAreaFiche))     return jButtonEdit;
		    else if (a.equals(jButtonEdit))        return jButtonViewDemandes;
		    else if (a.equals(jButtonViewDemandes))return jListResultat;
		    else if (a.equals(jListResultat))      return jButtonClose;
		    return jListResultat;
		}
	    }

	    public Component getDefaultComponent(Container focusCycleRoot) {
		if(m_bModeRecherche) return jTextFieldFind;
		else return jListResultat;
	    }

	    public Component getLastComponent(Container focusCycleRoot) {
		return jButtonClose;
	    }

	    public Component getFirstComponent(Container focusCycleRoot) {
		if(m_bModeRecherche) return jTextFieldFind;
		else return jListResultat;
	    }
	});
    }

    private void setFindEnabled(boolean b)
    {
	m_bModeRecherche=b;
	jButtonFind.setEnabled(b);
	jComboBoxCirco.setEnabled(b);
	jTextFieldFind.setEnabled(b);
	jRadioButtonNom.setEnabled(b);
	jRadioButtonOrga.setEnabled(b);
	jLabelCirco.setEnabled(b);
    }

    private void flushFind()
    {
	jRadioButtonNom.setSelected(true);
	jTextFieldFind.setText("");
	jComboBoxCirco.setSelectedIndex(0);
	jTextAreaFiche.setText("");
	jListResultat.setListData(new ContactDATA[0]);    //vide la liste
    }

    //ouverture en mode recherche
    public void setVisible(boolean b)
    {
	//initialisation des lments
	setFindEnabled(true);
	flushFind();
	jLabelResultat.setText("Rsultat :");
	jTextFieldFind.requestFocusInWindow();
	super.setVisible(b);
    }

    //ouverture en mode affichage
    public void show(Vector contact)
    {
	//initialisation des lments
	setFindEnabled(false);
	flushFind();
	jLabelResultat.setText("Liste des contacts de la demande :");

	//remplissage de la liste
	jListResultat.setListData(contact);

	jListResultat.requestFocusInWindow();
	super.setVisible(true);
    }

    //pour lien crois entre rechercherDemande et rechercherContact
    public void registerIHM(RechercherDemandeIHM ihm)
    {
	m_ihmRechercherDemande=ihm;
    }

    void jButtonFind_actionPerformed(ActionEvent e)
    {
	if(m_bModeRecherche)
	{
	    jListResultat.setListData(m_ctlRechercher.getContacts(jTextFieldFind.getText(),
			    jRadioButtonNom.isSelected(),
			    (CircoDATA)jComboBoxCirco.getSelectedItem()));
	    jTextAreaFiche.setText("");
	    jLabelResultat.setText("Rsultat : "+jListResultat.getModel().getSize()+" contacts");
	    jListResultat.requestFocusInWindow();
	}
    }

    void jButtonClose_actionPerformed(ActionEvent e)
    {
	super.setVisible(false);
    }

    void jListResultat_valueChanged(ListSelectionEvent e)
    {
	if(!jListResultat.isSelectionEmpty())
	    jTextAreaFiche.setText(((ContactDATA)jListResultat.getSelectedValue()).getFiche());
    }

    void jButtonViewDemandes_actionPerformed(ActionEvent e)
    {
	if(!jListResultat.isSelectionEmpty())
	{
	    try
	    {
		m_ihmRechercherDemande.findByContact((ContactDATA)jListResultat.getSelectedValue());
	    }
	    catch(Exception ex) //si ihm n'a pas t register
	    {
		if(cf.debug()) ex.printStackTrace();
		JOptionPane.showMessageDialog(null,Tools.formatString(e.toString(),55),
				    "Erreur interne",
				    JOptionPane.ERROR_MESSAGE);
	    }
	}
	else
	{
	    JOptionPane.showMessageDialog(this,"Vous devez slectionner un contact.",
	       "Action indtermine",JOptionPane.INFORMATION_MESSAGE);
	}
    }

    void jButtonEdit_actionPerformed(ActionEvent e)
    {
	if(!jListResultat.isSelectionEmpty())
	{
	    m_ihmAddContact.edit((ContactDATA)jListResultat.getSelectedValue());
	}
	else
	{
	    JOptionPane.showMessageDialog(this,"Vous devez slectionner un contact.",
	       "Action indtermine",JOptionPane.INFORMATION_MESSAGE);
	}
    }

    void jListResultat_mouseClicked(MouseEvent e)
    {
        if(SwingUtilities.isLeftMouseButton(e) && e.getClickCount()==2)
            jButtonEdit_actionPerformed(null);
    }
}
