/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2005  Nicolas AGIUS                                     *
 *                                                                         *
 *   This file is part of ECaMM.                                           *
 *                                                                         *
 *   ECaMM is free software; you can redistribute it and/or modify it      *
 *   under the terms of the GNU General Public License as published by     *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   any later version.                                                    *
 *                                                                         *
 *   ECaMM is distributed in the hope that it will be useful, but          *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston,                *
 *    MA  02111-1307, USA.                                                 *
 *                                                                         *
 *   For more information about these matters, see the file named COPYING. *
 *                                                                         *
 *   Mail : nicolas_agius@yahoo.fr                                         *
 *                                                                         *
 ***************************************************************************/
package ecamm;

/**
 * Titre :  RechercherDemandeIHM [ECaMM]
 * Description : Fentre de recherche d'une demande
 * Copyright : GPL v2.0 - 2005~2006
 * @author : Nicolas AGIUS
 * @version 1.1
 */

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.border.TitledBorder;
import javax.swing.border.Border;
import java.util.Date;
import javax.swing.event.ListSelectionEvent;
import java.util.Vector;
import java.util.HashMap;
import javax.swing.text.DefaultEditorKit;

public class RechercherDemandeIHM extends JFrame
{
    //Constantes pour l'IHM
    private static final String CARD_UNIQUE="panel date unique";
    private static final String CARD_INTERVAL="panel date interval";
    private static final String CARD_DUMMY="panel date vide";
    private static final String DATE_TYPE[] ={"Avant","Pendant","Aprs"};

    //Constantes pour le keybinding
    private static final String ACTION_FIND = "find";
    private static final String ACTION_CLOSE = "close";
    private static final String ACTION_FOCUSLIST = "focus-liste";
    private static final String ACTION_FOCUSFICHE = "focus-fiche";
    private static final String ACTION_FOCUSFIND = "focus-find";
    private static final String ACTION_OPENBOX = "open-popup";
    private static final String ACTION_VIEWET="etablissement";
    private static final String ACTION_VIEWCT="contact";

    //variable de traitement
    private ConfigurationCTL cf=Main.cf;
    private RechercherCTL m_ctlRechercher;
    private RechercherContactIHM m_ihmRechercherContact;
    private RechercherEtabIHM m_ihmRechercherEtab;
    private boolean m_bModeRecherche;  //dsactive la recherche lorsque affiche info depuis RechercherEtab et RechercherContact

    //composant graphiques
    private JPanel jPanel1 = new JPanel();
    private JTextField jTextField = new JTextField();
    private JLabel jLabelEtabs = new JLabel();
    private JLabel jLabelContacts = new JLabel();
    private JButton jButtonViewContacts = new JButton();
    private JButton jButtonViewEtabs = new JButton();
    private JButton jButtonFind = new JButton();
    private JCheckBox jCheckBoxTexte = new JCheckBox();
    private JCheckBox jCheckBoxCategorie = new JCheckBox();
    private JComboBox jComboBoxCategorie = new JComboBox();
    private JCheckBox jCheckBoxDate = new JCheckBox();
    private JToggleButton jToggleButtonMode = new JToggleButton();
    private JLabel jLabel1 = new JLabel();
    private JLabel jLabel4 = new JLabel();
    private JRadioButton jRadioButtonDateU = new JRadioButton();
    private JRadioButton jRadioButtonDateI = new JRadioButton();
    private JLabel jLabel5 = new JLabel();
    private JComboBox jComboBoxDate = new JComboBox(DATE_TYPE);
    private JSpinner jSpinnerDateU = new JSpinner();
    private JSpinner jSpinnerDateI1 = new JSpinner();
    private JPanel jPanelDateI = new JPanel();
    private JSpinner jSpinnerDateI2 = new JSpinner();
    private JPanel jPanelDateU = new JPanel();
    private JPanel jPanelDate = new JPanel();
    private GridBagLayout gridBagLayout1 = new GridBagLayout();
    private GridBagLayout gridBagLayout2 = new GridBagLayout();
    private JPanel jPanel5 = new JPanel();
    private JButton jButtonClose = new JButton();
    private JScrollPane jScrollPaneFiche = new JScrollPane();
    private JTextArea jTextAreaFiche = new JTextArea();
    private ButtonGroup buttonGroupDate = new ButtonGroup();
    private GridBagLayout gridBagLayout3 = new GridBagLayout();
    private JPanel jPanel2 = new JPanel();
    private GridBagLayout gridBagLayout4 = new GridBagLayout();
    private CardLayout cardLayoutDate = new CardLayout();
    private JPanel jPanel4 = new JPanel();
    private GridBagLayout gridBagLayout6 = new GridBagLayout();
    private Border border7 = BorderFactory.createEtchedBorder(Color.white, new Color(148, 145, 140));
    private Border border8 = new TitledBorder(border7, " Critres de recherche : ");
    private JPanel jPanel3 = new JPanel();
    private Border border1 = BorderFactory.createEtchedBorder(Color.white, new Color(148, 145, 140));
    private Border border2 = new TitledBorder(border1, " Fiche de la demande : ");
    private GridBagLayout gridBagLayout5 = new GridBagLayout();
    private JPanel jPanel6 = new JPanel();
    private JScrollPane jScrollPaneResultat = new JScrollPane();
    private JList jListResultat = new JList();
    private GridBagLayout gridBagLayout7 = new GridBagLayout();
    private Border border3 = BorderFactory.createEtchedBorder(Color.white, new Color(148, 145, 140));
    private Border border4 = new TitledBorder(border3, " Rsultats : ");
    private JLabel jLabelResultat = new JLabel("    ");
    private JPopupMenu jPopupMenuFiche = new JPopupMenu();
    private JMenuItem jMenuItemCopyFiche;

    public RechercherDemandeIHM(RechercherEtabIHM ihm_et,
				RechercherContactIHM ihm_ct)
    {
	cf.registerIHM(this);
	m_ihmRechercherContact=ihm_ct;
	m_ihmRechercherContact.registerIHM(this);
	m_ihmRechercherEtab=ihm_et;
	m_ihmRechercherEtab.registerIHM(this);

        //Classe de controle mutualise
	m_ctlRechercher=cf.getRechercherCTL();

        try
	{
            popupMenuInit();
            jbInit();
            keyBindingInit();
        }
        catch (Exception e)
	{
            Tools.internalError(e,"",cf.debug(),true);
        }

	//remplissage de la combo box categorie
	CategorieDATA categories[]= m_ctlRechercher.getCategories();
	jComboBoxCategorie.addItem(new CategorieDATA());
	for(int i=0;i<categories.length;i++)
	    jComboBoxCategorie.addItem(categories[i]);

	//dimensionnement
	pack();     //calcule une 1ere fois les preferredSize
	SwingUtilities.invokeLater(new Runnable() {
	    public void run() {
		int x=jPanel6.getSize().width-16;  //insets du scrollPane
		jScrollPaneFiche.setMinimumSize(new Dimension(x, 100));
		jScrollPaneFiche.setPreferredSize(new Dimension(x, 100));
		pack();     //reclacule avec les bonne dimensions
	    }
	});

	//positionnement
	Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	Dimension frameSize = getSize();
	setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);

	//initialisation de l'etat des lments de saisie
	jCheckBoxTexte_itemStateChanged(null);
	jCheckBoxDate_itemStateChanged(null);
        jCheckBoxCategorie_itemStateChanged(null);
    }

    private void jbInit() throws Exception
    {
	jSpinnerDateU.setModel(new SpinnerDateModel());
	jSpinnerDateU.setEditor(new JSpinner.DateEditor(jSpinnerDateU, "dd/MM/yyyy"));
	jSpinnerDateI1.setModel(new SpinnerDateModel());
	jSpinnerDateI1.setEditor(new JSpinner.DateEditor(jSpinnerDateI1, "dd/MM/yyyy"));
	jSpinnerDateI2.setModel(new SpinnerDateModel());
	jSpinnerDateI2.setEditor(new JSpinner.DateEditor(jSpinnerDateI2, "dd/MM/yyyy"));
        jButtonViewContacts.setMargin(new Insets(2, 2, 2, 2));
	jButtonViewContacts.setMinimumSize(new Dimension(17, 20));
	jButtonViewContacts.setPreferredSize(new Dimension(17, 20));
        jButtonViewContacts.setText("?");
        jButtonViewContacts.setEnabled(false);
        jButtonViewContacts.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                jButtonViewContacts_actionPerformed(e);
            }
        });
        jButtonViewEtabs.setMargin(new Insets(2, 2, 2, 2));
	jButtonViewEtabs.setMinimumSize(new Dimension(17, 20));
	jButtonViewEtabs.setPreferredSize(new Dimension(17, 20));
        jButtonViewEtabs.setText("?");
        jButtonViewEtabs.setEnabled(false);
        jButtonViewEtabs.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                jButtonViewEtabs_actionPerformed(e);
            }
        });
        jButtonFind.setMargin(new Insets(2, 10, 2, 10));
        jButtonFind.setText("Rechercher");
	jButtonFind.setMnemonic('r');
        jButtonFind.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                jButtonFind_actionPerformed(e);
            }
        });
        jCheckBoxTexte.setMnemonic('T');
        jCheckBoxTexte.setSelected(true);
        jCheckBoxTexte.setText("Par texte :");
        jCheckBoxTexte.addItemListener(new ItemListener()
        {
            public void itemStateChanged(ItemEvent e)
            {
                jCheckBoxTexte_itemStateChanged(e);
            }
        });
        jToggleButtonMode.setText("FullText");
        jToggleButtonMode.setMnemonic('u');
        jToggleButtonMode.setSelected(cf.isFulltextMode());
        jToggleButtonMode.setMargin(new Insets(0, 2, 0, 2));
        jCheckBoxCategorie.setMnemonic('C');
        jCheckBoxCategorie.setText("Par catgorie :");
        jCheckBoxCategorie.addItemListener(new ItemListener()
        {
            public void itemStateChanged(ItemEvent e)
            {
                jCheckBoxCategorie_itemStateChanged(e);
            }
        });
        jCheckBoxDate.setMnemonic('a');
        jCheckBoxDate.setText("Par date :");
        jCheckBoxDate.addItemListener(new ItemListener()
        {
            public void itemStateChanged(ItemEvent e)
            {
                jCheckBoxDate_itemStateChanged(e);
            }
        });
        jLabel1.setText("Entre le");
        jLabel4.setText("et le");
        jRadioButtonDateU.setMnemonic('U');
        jRadioButtonDateU.setSelected(true);
        jRadioButtonDateU.setText("Unique");
        jRadioButtonDateU.addItemListener(new ItemListener()
        {
            public void itemStateChanged(ItemEvent e)
            {
                jRadioButtonDate_itemStateChanged(e);
            }
        });
        jRadioButtonDateI.setMnemonic('I');
        jRadioButtonDateI.setText("Interval");
        jLabel5.setText("le");
        jPanelDateI.setLayout(gridBagLayout1);
        jPanelDateU.setLayout(gridBagLayout2);
        jPanelDate.setLayout(cardLayoutDate);
        jButtonClose.setText("Fermer");
	jButtonClose.setMnemonic('f');
        jButtonClose.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                jButtonClose_actionPerformed(e);
            }
        });
        jTextAreaFiche.setEditable(false);
        jTextAreaFiche.setLineWrap(true);
        jTextAreaFiche.setWrapStyleWord(true);
        jPanel2.setLayout(gridBagLayout4);
        jRadioButtonDateI.addItemListener(new ItemListener()
        {
            public void itemStateChanged(ItemEvent e)
            {
                jRadioButtonDate_itemStateChanged(e);
            }
        });
        jPanel4.setLayout(gridBagLayout6);
        jPanel4.setBorder(border8);
        jPanel3.setLayout(gridBagLayout5);
        jPanel3.setBorder(border2);
        jPanel6.setLayout(gridBagLayout7);
        jListResultat.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	jListResultat.addListSelectionListener(new javax.swing.event.ListSelectionListener()
	{
	    public void valueChanged(ListSelectionEvent e)
	    {
		jListResultat_valueChanged(e);
	    }
	});
        jPanel6.setBorder(border4);
        jPanel5.add(jButtonClose);
	this.setDefaultCloseOperation(WindowConstants.HIDE_ON_CLOSE);
	this.setTitle("Rechercher des demandes");
        this.getContentPane().add(jPanel1, java.awt.BorderLayout.CENTER);
        jPanelDateI.add(jLabel4, new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0
            , GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 6), 0, 0));
        jPanelDateI.add(jLabel1, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            , GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 3, 0, 6), 0, 0));
        jPanelDateI.add(jSpinnerDateI1, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0
            , GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 6), 0, 0));
        jPanelDateI.add(jSpinnerDateI2, new GridBagConstraints(3, 0, 1, 1, 1.0, 0.0
            , GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 6), 0, 0));
        jPanel1.setLayout(gridBagLayout3);
        buttonGroupDate.add(jRadioButtonDateU);
        buttonGroupDate.add(jRadioButtonDateI);
	jPanelDate.add(new JPanel(), CARD_DUMMY);
	jPanelDate.add(jPanelDateU, CARD_UNIQUE);
	jPanelDate.add(jPanelDateI, CARD_INTERVAL);
        jPanel2.add(jRadioButtonDateU, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            , GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
        jPanel2.add(jRadioButtonDateI, new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0
            , GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 2), 0, 0));
        jPanelDateU.add(jLabel5, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0
            , GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 6), 0, 0));
        jPanelDateU.add(jComboBoxDate, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            , GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 3, 0, 6), 0, 0));
        jPanelDateU.add(jSpinnerDateU, new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0
            , GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 6), 0, 0));
        jPanel4.add(jCheckBoxTexte, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
            , GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 3, 0, 0), 0, 0));
        jPanel4.add(jCheckBoxCategorie, new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0
            , GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 3, 0, 0), 0, 0));
        jPanel4.add(jCheckBoxDate, new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0
            , GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 3, 0, 0), 0, 0));
        jScrollPaneFiche.getViewport().add(jTextAreaFiche);
        jPanel4.add(jPanelDate, new GridBagConstraints(0, 4, 2, 1, 0.0, 0.0
            , GridBagConstraints.NORTH, GridBagConstraints.NONE, new Insets(3, 0, 6, 3), 0, 0));
        jScrollPaneResultat.getViewport().add(jListResultat);
        jPanel1.add(jPanel4, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            , GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(3, 6, 6, 0), 0, 0));
        jPanel1.add(jPanel6, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
	    , GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 6, 0, 0), 0, 0));
        jPanel1.add(jPanel3, new GridBagConstraints(1, 0, 1, 2, 1.0, 1.0
            , GridBagConstraints.WEST, GridBagConstraints.BOTH, new Insets(3, 3, 0, 6), 0, 0));
        jPanel1.add(jPanel5, new GridBagConstraints(0, 4, 2, 1, 0.0, 0.0
	    , GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
        jPanel6.add(jScrollPaneResultat, new GridBagConstraints(0, 1, 1, 1, 1.0, 1.0
            , GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 3, 3, 3), 0, 0));
        jPanel3.add(jScrollPaneFiche, new GridBagConstraints(0, 3, 3, 1, 1.0, 1.0
            , GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(6, 3, 3, 3), 0, 0));
        jPanel6.add(jLabelResultat, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            , GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(0, 12, 6, 0), 0, 0));
        jPanel3.add(jButtonViewEtabs, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            , GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 9, 0, 0), 0, 0));
        jPanel3.add(jButtonViewContacts, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
            , GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(3, 9, 3, 0), 0, 0));
        jPanel3.add(jLabelEtabs, new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0
            , GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0, 6, 0, 0), 0, 0));
        jPanel3.add(jLabelContacts, new GridBagConstraints(1, 1, 1, 1, 1.0, 0.0
            , GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0, 6, 0, 0), 0, 0));
        jPanel4.add(jToggleButtonMode, new GridBagConstraints(3, 1, 1, 1, 0.0, 0.0
            , GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(3, 0, 3, 12), 0, 0));
        jPanel4.add(jTextField, new GridBagConstraints(1, 1, 2, 1, 1.0, 0.0
            , GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(3, 12, 3, 3), 0, 0));
        jPanel4.add(jComboBoxCategorie, new GridBagConstraints(1, 2, 3, 1, 0.0, 0.0
            , GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(3, 12, 3, 12), 0, 0));
        jPanel4.add(jPanel2, new GridBagConstraints(1, 3, 2, 1, 0.0, 0.0
            , GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(3, -6, 3, 0), 0, 0));
        jPanel4.add(jButtonFind, new GridBagConstraints(2, 4, 2, 1, 1.0, 0.0
            , GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 3, 6), 0, 0));
    }

    private void keyBindingInit() throws Exception
    {
	//Keys binding gnraux
	InputMap inputMap=jPanel1.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW);
	ActionMap actionMap=jPanel1.getActionMap();
	inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER,0),ACTION_FIND);
	inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE,0),ACTION_CLOSE);
	inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_R,ActionEvent.CTRL_MASK),ACTION_FOCUSLIST);
	inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_F,ActionEvent.CTRL_MASK),ACTION_FOCUSFICHE);
	inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_T,ActionEvent.CTRL_MASK),ACTION_FOCUSFIND);
	inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_F7,0),ACTION_VIEWCT);
	inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_F6,0),ACTION_VIEWET);
	actionMap.put(ACTION_FIND,new AbstractAction()
	{
	    public void actionPerformed(ActionEvent e)
	    {
		jButtonFind_actionPerformed(e);
	    }
	});
	actionMap.put(ACTION_CLOSE,new AbstractAction()
	{
	    public void actionPerformed(ActionEvent e)
	    {
		jButtonClose_actionPerformed(e);
	    }
	});
	actionMap.put(ACTION_FOCUSLIST,new AbstractAction()
	{
	    public void actionPerformed(ActionEvent e)
	    {
		jListResultat.requestFocusInWindow();
	    }
	});
	actionMap.put(ACTION_FOCUSFICHE,new AbstractAction()
	{
	    public void actionPerformed(ActionEvent e)
	    {
		jTextAreaFiche.requestFocusInWindow();
	    }
	});
	actionMap.put(ACTION_FOCUSFIND,new AbstractAction()
	{
	    public void actionPerformed(ActionEvent e)
	    {
		jTextField.requestFocusInWindow();
	    }
	});
	actionMap.put(ACTION_VIEWCT,new AbstractAction()
	{
	    public void actionPerformed(ActionEvent e)
	    {
		jButtonViewContacts_actionPerformed(e);
	    }
	});
	actionMap.put(ACTION_VIEWET,new AbstractAction()
	{
	    public void actionPerformed(ActionEvent e)
	    {
		jButtonViewEtabs_actionPerformed(e);
	    }
	});

	//key bindings comboBox CAtgorie
	jComboBoxCategorie.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_SPACE,0),ACTION_OPENBOX);
	jComboBoxCategorie.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER,0),ACTION_FIND);
	jComboBoxCategorie.getActionMap().put(ACTION_FIND,new AbstractAction()
	{
	    public void actionPerformed(ActionEvent e)
	    {
		jButtonFind_actionPerformed(e);
	    }
	});
	jComboBoxCategorie.getActionMap().put(ACTION_OPENBOX,new AbstractAction()
	{
	    public void actionPerformed(ActionEvent e)
	    {
		jComboBoxCategorie.showPopup();
	    }
	});


	//key bindings comboBoxDate
	jComboBoxDate.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_SPACE,0),ACTION_OPENBOX);
	jComboBoxDate.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER,0),ACTION_FIND);
	jComboBoxDate.getActionMap().put(ACTION_FIND,new AbstractAction()
	{
	    public void actionPerformed(ActionEvent e)
	    {
		jButtonFind_actionPerformed(e);
	    }
	});
	jComboBoxDate.getActionMap().put(ACTION_OPENBOX,new AbstractAction()
	{
	    public void actionPerformed(ActionEvent e)
	    {
		jComboBoxDate.showPopup();
	    }
	});
    }

    private void popupMenuInit() throws Exception
    {
        //Cration de la hashmap des actions de jTextAreaProb
        HashMap hmFiche = new HashMap();
        Action[] a = jTextAreaFiche.getActions();
        for(int i = 0; i < a.length; hmFiche.put(a[i].getValue(Action.NAME), a[i++]));

        jMenuItemCopyFiche = jPopupMenuFiche.add((Action)hmFiche.get(DefaultEditorKit.copyAction));
        jMenuItemCopyFiche.setText("Copier");
        jPopupMenuFiche.addSeparator();
        jPopupMenuFiche.add((Action)hmFiche.get(DefaultEditorKit.selectAllAction)).setText("Slectionner tout");

        //ouverture du popup prob
        jTextAreaFiche.addMouseListener(new MouseAdapter()
        {
            public void mousePressed(MouseEvent e)
            {
                maybeShowPopup(e);
            }

            public void mouseReleased(MouseEvent e)
            {
                maybeShowPopup(e);
            }

            private void maybeShowPopup(MouseEvent e)
            {
                if(e.isPopupTrigger())
                {
                    //activation de "copier" si texte slectionn
                    jMenuItemCopyFiche.setEnabled(jTextAreaFiche.getSelectedText()!=null);

                    jTextAreaFiche.requestFocus();
                    jPopupMenuFiche.show(e.getComponent(),e.getX(), e.getY());
                }
            }
        });
    }


    private void clearFiche()
    {
	jTextAreaFiche.setText("");
	jLabelEtabs.setText("");
	jLabelContacts.setText("");
        jButtonViewEtabs.setEnabled(false);
        jButtonViewContacts.setEnabled(false);
    }

    private void flushFind()
    {
	clearFiche();
	jTextField.setText("");
	jComboBoxCategorie.setSelectedIndex(0);
	jListResultat.setListData(new ArchiveDATA[0]);  //vide la liste
	jCheckBoxDate.setSelected(false);
    }


    private void setFindEnabled(boolean b)
    {
	m_bModeRecherche=b;
	jButtonFind.setEnabled(b);
	jCheckBoxTexte.setEnabled(b);
	jCheckBoxDate.setEnabled(b);
	jCheckBoxCategorie.setEnabled(b);
	jComboBoxCategorie.setEnabled(b);
	jRadioButtonDateI.setEnabled(b);
	jRadioButtonDateU.setEnabled(b);
	jTextField.setEnabled(b);
	jRadioButtonDate_itemStateChanged(null);

	if(b)   //r-initialisation des champs de saisie
	{
	    jCheckBoxTexte_itemStateChanged(null);
	    jCheckBoxDate_itemStateChanged(null);
	    jCheckBoxCategorie_itemStateChanged(null);
	}
    }

    private void showResult(ArchiveDATA arcs[])
    {
	//remplissage de la liste
	jListResultat.setListData(arcs);

	//nettoyage et remplissage des autres lments
	clearFiche();
	jLabelResultat.setText(jListResultat.getModel().getSize()+" demande(s) trouve(s)");
	jListResultat.requestFocusInWindow();
    }

    //remplissage des labels Etabs et Contacts
    private void showContactEtab(ArchiveDATA arc)
    {
        //remplissage du label etablissement
        Vector etabs = arc.getEtabs();
        String labelEt = new String("Etablissements");
        for(int i = 0; i < etabs.size(); labelEt += " : " + (EtabDATA)etabs.get(i++));
        jLabelEtabs.setText(etabs.isEmpty()?"Pas d'tablissement":labelEt);

        //remplissage du label contact
        Vector contacts = arc.getContacts();
        String labelCt = new String("Contacts");
        for(int i = 0; i < contacts.size(); labelCt += " : " + (ContactDATA)contacts.get(i++));
        jLabelContacts.setText(contacts.isEmpty()?"Pas de contact":labelCt);

        //activation des boutons et des label
        jButtonViewEtabs.setEnabled(!etabs.isEmpty());
        jButtonViewContacts.setEnabled(!contacts.isEmpty());
        jLabelEtabs.setEnabled(!etabs.isEmpty());
        jLabelContacts.setEnabled(!contacts.isEmpty());
    }

    public void findByContact(ContactDATA ct)
    {
	//initialisation des lments
	setFindEnabled(false);
	flushFind();

	//recherche et affichage du resultat
	showResult(m_ctlRechercher.getArchives(ct));

	super.setVisible(true);
    }

    public void findByEtab(EtabDATA et)
    {
	//initialisation des lments
	setFindEnabled(false);
	flushFind();

	//recherche et affichage du resultat
	showResult(m_ctlRechercher.getArchives(et));

	super.setVisible(true);
    }


    public void findByNumDossier(int numDossier)
    {
	//initialisation des lments
	setFindEnabled(false);
	flushFind();

	//recherche et affichage du resultat
	showResult(m_ctlRechercher.getArchives(numDossier));

	super.setVisible(true);
    }


    public void setVisible(boolean b)
    {
	//initialisation des lments
	setFindEnabled(true);
	flushFind();
	jLabelResultat.setText("  ");
	jTextField.requestFocusInWindow();
	super.setVisible(b);
    }

    //pour la sauvegarde de la conf depuis VisuIHM on exit
    public boolean isFulltextMode()
    {
        return jToggleButtonMode.isSelected();
    }

    void jButtonFind_actionPerformed(ActionEvent e)
    {
	if(m_bModeRecherche)
	{
	    //recherche et affichage du resultat
	    showResult(m_ctlRechercher.getArchives(jCheckBoxTexte.isSelected(),jTextField.getText(),jToggleButtonMode.isSelected(),
		jCheckBoxCategorie.isSelected(),(CategorieDATA)jComboBoxCategorie.getSelectedItem(),
		jCheckBoxDate.isSelected() && jRadioButtonDateU.isSelected(),jComboBoxDate.getSelectedIndex(),(Date)jSpinnerDateU.getValue(),
		jCheckBoxDate.isSelected() && jRadioButtonDateI.isSelected(),(Date)jSpinnerDateI1.getValue(),(Date)jSpinnerDateI2.getValue()));
	}
    }

    void jButtonViewEtabs_actionPerformed(ActionEvent e)
    {
	if(!jListResultat.isSelectionEmpty())
	{
	    ArchiveDATA arc=(ArchiveDATA)jListResultat.getSelectedValue();
//	    if(!arc.isFull())
//		jButtonDetails_actionPerformed(null);
	    m_ihmRechercherEtab.show(arc.getEtabs());
	}
    }

    void jButtonViewContacts_actionPerformed(ActionEvent e)
    {
	if(!jListResultat.isSelectionEmpty())
	{
	    ArchiveDATA arc=(ArchiveDATA)jListResultat.getSelectedValue();
//	    if(!arc.isFull())
//		jButtonDetails_actionPerformed(null);
	    m_ihmRechercherContact.show(arc.getContacts());
	}
    }

    void jButtonClose_actionPerformed(ActionEvent e)
    {
	super.setVisible(false);
    }

    //changement de carte pour la date
    void jRadioButtonDate_itemStateChanged(ItemEvent e)
    {
	if(jCheckBoxDate.isSelected() && m_bModeRecherche)
	    cardLayoutDate.show(jPanelDate,jRadioButtonDateU.isSelected()?CARD_UNIQUE:CARD_INTERVAL);
	else
	    cardLayoutDate.show(jPanelDate,CARD_DUMMY);
    }

    //Activation de la recherche par texte
    void jCheckBoxTexte_itemStateChanged(ItemEvent e)
    {
	jTextField.setEnabled(jCheckBoxTexte.isSelected());
        jToggleButtonMode.setEnabled(jCheckBoxTexte.isSelected());
    }

    //Activation de la recherche par catgorie
    public void jCheckBoxCategorie_itemStateChanged(ItemEvent e)
    {
	jComboBoxCategorie.setEnabled(jCheckBoxCategorie.isSelected());
    }

    //Activation de la recherche par date
    void jCheckBoxDate_itemStateChanged(ItemEvent e)
    {
	boolean selected=jCheckBoxDate.isSelected();

	jRadioButtonDateU.setEnabled(selected);
	jRadioButtonDateI.setEnabled(selected);
        jRadioButtonDate_itemStateChanged(null);
    }

    //affichage de la demande
    void jListResultat_valueChanged(ListSelectionEvent e)
    {
	if(!jListResultat.isSelectionEmpty())
	{
	    ArchiveDATA arc=(ArchiveDATA)jListResultat.getSelectedValue();

	    //remplissage de la fiche
	    jTextAreaFiche.setText(arc.getFiche());

	    //positionnement du ScrollPane en haut
	    SwingUtilities.invokeLater(new Runnable() {
		public void run() {
		    jScrollPaneFiche.getViewport().setViewPosition(new Point(0,0));
		}
	    });

            if(arc.isFull())
            {
               showContactEtab(arc);
            }
            else    //remplissage automatique des contacts et tablissements
            {
                //affichage vide en attendant le remplissage
                jLabelEtabs.setText("Recherche en cours...");
                jLabelContacts.setText("Recherche en cours...");
                jButtonViewContacts.setEnabled(false);
                jButtonViewEtabs.setEnabled(false);
                jLabelEtabs.setEnabled(false);
                jLabelContacts.setEnabled(false);

                //rcupre et mmorise les contacts et les tablissements
                SwingUtilities.invokeLater(new Runnable()
                {
                    public void run()
                    {
                        ArchiveDATA arc = (ArchiveDATA)jListResultat.getSelectedValue();

                        //passage de l'archive par rfrence -> pas besoin de retour
                        m_ctlRechercher.fillContactEtabForArchive(arc);
                        showContactEtab(arc);
                    }
                });
            }
	}
    }
}
