/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2005  Nicolas AGIUS                                     *
 *                                                                         *
 *   This file is part of ECaMM.                                           *
 *                                                                         *
 *   ECaMM is free software; you can redistribute it and/or modify         *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   any later version.                                                    *
 *                                                                         *
 *   ECaMM is distributed in the hope that it will be useful, but          *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston,                *
 *    MA  02111-1307, USA.                                                 *
 *                                                                         *
 *   For more information about these matters, see the file named COPYING. *
 *                                                                         *
 *   Mail : nicolas_agius@yahoo.fr                                         *
 *                                                                         *
 ***************************************************************************/
package ecamm;

/**
 * Titre : Splash [ECaMM]
 * Description : Affiche un splash screen
 * Copyright : GPL 2003 - 2005
 * @author Nicolas AGIUS
 * @version 2.1
 */

import java.awt.Window;
import java.awt.Image;
import java.awt.Graphics;
import java.awt.Toolkit;
import java.awt.MediaTracker;
import java.awt.Dimension;
import javax.swing.JOptionPane;
//import java.awt.event.*;

public class Splash extends Window
{
    private final static String DEFAULT_IMG="splash.png";
    private Image m_Img;
    private int m_iImgWidth, m_iImgHeight;

    public Splash(Window f)
    {
        super(f);
        splashInit(DEFAULT_IMG);
    }

    public Splash(Window f, String imgName)
    {
        super(f);
        splashInit(imgName);
    }

    private void splashInit(String imgName)
    {
        Toolkit tk=Toolkit.getDefaultToolkit();

        m_Img = tk.createImage(getClass().getResource(imgName));

        MediaTracker tracker = new MediaTracker(this);
        tracker.addImage(m_Img, 0);
        try
        {
            tracker.waitForAll();
        }
        catch (Exception e)
        {
            //afficher impossible de charger le splashsreen
            e.printStackTrace();
            JOptionPane.showMessageDialog(null,"Impossible de charger le SplashScreen :\n" +
                                          Tools.formatString(e.toString(),55),
                                          "Erreur interne",JOptionPane.ERROR_MESSAGE);
        }

        m_iImgWidth = m_Img.getWidth(this);
        m_iImgHeight = m_Img.getHeight(this);

        Dimension screenSize = tk.getScreenSize();
        setBounds((screenSize.width - m_iImgWidth) / 2,
                (screenSize.height - m_iImgHeight) / 2,
                m_iImgWidth,
                m_iImgHeight);

        setVisible(true);

        // pour fermeture automatique du splash  l'apparition de la fentre de rfrence
        /*
        f.addWindowListener(new java.awt.event.WindowAdapter()
        {
            public void windowOpened(WindowEvent we)
            {
                setVisible(false);
                dispose();
            }
        });
        */
    }

    public void paint(Graphics g)
    {
        g.drawImage(m_Img, 0, 0,m_iImgWidth, m_iImgHeight, this);
    }
}
