/***************************************************************************
*                                                                         *
*   Copyright (C) 2006  Nicolas AGIUS                                     *
*                                                                         *
*   This file is part of ECaMM.                                           *
*                                                                         *
*   ECaMM is free software; you can redistribute it and/or modify it      *
*   under the terms of the GNU General Public License as published by     *
*   the Free Software Foundation; either version 2 of the License, or     *
*   any later version.                                                    *
*                                                                         *
*   ECaMM is distributed in the hope that it will be useful, but          *
*   WITHOUT ANY WARRANTY; without even the implied warranty of            *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the Free Software           *
*   Foundation, Inc., 59 Temple Place - Suite 330, Boston,                *
*    MA  02111-1307, USA.                                                 *
*                                                                         *
*   For more information about these matters, see the file named COPYING. *
*                                                                         *
*   Mail : nicolas_agius@yahoo.fr                                         *
*                                                                         *
***************************************************************************/
package ecamm;

import javax.swing.JOptionPane;
import java.util.Calendar;

/**
* Titre :  Tools [ECaMM]
* Description : Boite  outils static<br>
* Copyright GPL v2.0 - 2006
* @author Nicolas AGIUS
* @version 1.2
*/

/*
* ChangeLog
* ---------
*
* 19/01/2006 - v1.0 : Premires fonctions formatString et protectString
* 23/01/2006 - v1.1 : Fonction firstLine et *Error
* 27/02/2006 - v1.2 : Fonction DayOfWeek
*/

public class Tools
{


    /**
     * Insre des retour-chariots dans la chaine pour formatter
     * la largeur  l'affichage.
     *
     * @param input String
     *      Chaine  modifier
     * @param len int
     *      Longueur maximale d'une ligne
     * @return String
     *      La chaine modifie
     */
    public static String formatString(String input,int len)
    {
        char data[];
        data=input.toCharArray();
        String output=new String();
        boolean cut;

        int i,a=0;
        while(a<data.length)
        {
            cut=true;
            if(a+len < data.length)
            {
                for(i=len;data[a+i]!=' ' && i>0;i--);
                if(i==0)
                {
                    i=len;  //pas d'espace pour couper
                    cut=false;
                }
            }
            else
                i=data.length-a;

            output+= String.copyValueOf(data,a,i) + "\n";
            a+=i;
            if(cut) a++; //suppression de l'espace de coupe
        }

        return output;
    }

    /**
     * Fonction de protection de chaine pour requete SQL
     * Ajoute des \ devant les ' et les \ .
     *
     * @param chaine String
     *      La chaine  protger
     * @return String
     *      La chaine modifie
     */
    public static String protectString(String chaine)
    {
        //Le Java c'est bien mais le C c'est mieux !

        int l=0;
        char src[]=chaine.toCharArray();

        //compte le nombre de ' et de /
        for(int i=0;i<src.length;i++)
        {
            if(src[i]=='\'' || src[i]=='\\') l++;
        }

        //cre le tableau destination avec la bonne taille
        char dst[]=new char[src.length+l];

        //copie dans le nouveau tableau en protgeant les caractres
        for(int i=0,j=0;i<src.length;i++)
        {
            if(src[i]=='\'' || src[i]=='\\') dst[j++]='\\';
            dst[j++]=src[i];
        }
        return new String(dst);
    }

    /**
     *  Retourne une chaine contenant seulement la premire ligne de la chaine
     * passe en paramtre.
     *
     * @param str String
     *  La chaine d'origine
     * @return String
     *  La premire ligne de str
     */
    public static String firstLine(String str)
    {
        int i = str.indexOf('\n');
        if (i == -1)
            return str.trim();
        else
            return str.substring(0, i).trim();
    }

    /**
     * Affiche un message d'erreur correcpondant  l'exception.
     * Usage : Tools.internalError(e,"",cf.debug(),false);
     *
     * @param e Exception
     *  L'exception origine de l'erreur
     * @param msg String
     *  Message affich
     * @param debug boolean
     *  Si affichage de la pile sur la console
     * @param fatal boolean
     *  Si fermeture du programme
     */
    public static void internalError(Exception e,String msg, boolean debug, boolean fatal)
    {
        if(debug) e.printStackTrace();

        JOptionPane.showMessageDialog(null,msg + formatString(e.toString(),60),
                                      "Erreur interne",
                                      JOptionPane.ERROR_MESSAGE);
        if(fatal) System.exit(1);
    }

     public static void sqlError(Exception e, boolean debug)
    {
        if(debug) e.printStackTrace();

        JOptionPane.showMessageDialog(null,formatString(firstLine(e.toString()),60),
                                      "Erreur d'accs  la base de donnes",
                                      JOptionPane.ERROR_MESSAGE);
    }

    /**
     *
     *  Retourne le nom du jour correspondant au numro pass en paramtre.
     * @param day int
     *  Le numro du jour dans la semaine, tel que retourn par la fonction Calendar.get(Calendar.DAY_OF_WEEK).<br>
     * Doit donc tre un membre de la classe Calendar.
     * @return String
     * Le nom du jour correspondant, en franais, avec une majuscule.
     * @see java.util.Calendar
     */
    public static String dayOfWeek(int day)
    {
        switch(day)
        {
            case Calendar.MONDAY:
                return "Lundi";
            case Calendar.TUESDAY:
                return "Mardi";
            case Calendar.WEDNESDAY:
                return "Mercredi";
            case Calendar.THURSDAY:
                return "Jeudi";
            case Calendar.FRIDAY:
                return "Vendredi";
            case Calendar.SATURDAY:
                return "Samedi";
            case Calendar.SUNDAY:
                return "Dimanche";
            default:
                return "";
        }
    }

}
