/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2006  Nicolas AGIUS                                     *
 *                                                                         *
 *   This file is part of ECaMM.                                           *
 *                                                                         *
 *   ECaMM is free software; you can redistribute it and/or modify it      *
 *   under the terms of the GNU General Public License as published by     *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   any later version.                                                    *
 *                                                                         *
 *   ECaMM is distributed in the hope that it will be useful, but          *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the Free Software           *
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston,                *
 *    MA  02111-1307, USA.                                                 *
 *                                                                         *
 *   For more information about these matters, see the file named COPYING. *
 *                                                                         *
 *   Mail : nicolas_agius@yahoo.fr                                         *
 *                                                                         *
 ***************************************************************************/
package ecamm;

/**
 * Titre :  VisuIHM [ECaMM]
 * Description : Fentre principale
 * Copyright : GPL v2.0 - 2005~2006
 * @author : Nicolas AGIUS
 * @version 1.2
 */

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class VisuIHM extends JFrame
{
    //Constantes pour le keybinding
    private static final String ACTION_EDIT="edit";

    //variable de traitement
    private ConfigurationCTL cf=Main.cf;
    private java.util.Timer timer=new java.util.Timer();
    private EditerIHM m_ihmEditer;
    private CreerIHM m_ihmCreer1;
    private CreerIHM m_ihmCreer2;
    private AjouterContactIHM m_ihmAddContact;
    private AjouterEtabIHM m_ihmAddEtab;
    private RechercherEtabIHM m_ihmRechEtab;
    private RechercherContactIHM m_ihmRechContact;
    private RechercherDemandeIHM m_ihmRechDemande;

     private RechercherCTL m_ctlRechercher; //repass en local pour lisibilit

    //composant graphiques
    private JMenuBar jMenuBar1 = new JMenuBar();
    private JMenu jMenu1 = new JMenu();
    private JMenuItem jMenuItemRefresh = new JMenuItem();
    private JMenuItem jMenuItemQuitter = new JMenuItem();
    private JMenu jMenu2 = new JMenu();
    private JMenu jMenu3 = new JMenu();
    private JMenuItem jMenuItemHelp = new JMenuItem();
    private JMenuItem jMenuItemAPropos = new JMenuItem();
    private JPanel jPanel1 = new JPanel();
    private JScrollPane jScrollPane1 = new JScrollPane();
    private JLabel jLabel1 = new JLabel();
    private JPanel jPanel2 = new JPanel();
    private JButton jButtonNouveau = new JButton();
    private JButton jButtonEditer = new JButton();
    private JButton jButtonRechercher = new JButton();
    private JList jListDemande = new JList();
    private JMenu jMenu4 = new JMenu();
    private JMenuItem jMenuItemNewContact = new JMenuItem();
    private JMenuItem jMenuItemNewEtab = new JMenuItem();
    private JMenuItem jMenuItemNewDem = new JMenuItem();
    private JMenuItem jMenuItemConfig = new JMenuItem();
    private JMenu jMenu5 = new JMenu();
    private JMenuItem jMenuItemFindContact = new JMenuItem();
    private JMenuItem jMenuItemFindEtab = new JMenuItem();
    private JMenuItem jMenuItemFindDossier = new JMenuItem();
    private JMenuItem jMenuItemFind = new JMenuItem();
    private GridBagLayout gridBagLayout1 = new GridBagLayout();

    public VisuIHM()
    {
        cf.registerIHM(this);
        setSize(cf.getVisuSize());
        setLocation(cf.getVisuLocation());

	//pr-chargement de toutes les fentres
        m_ihmAddEtab=new AjouterEtabIHM();
        m_ihmAddContact=new AjouterContactIHM();
        m_ihmCreer1=new CreerIHM(m_ihmAddContact,m_ihmAddEtab);
        m_ihmCreer2=new CreerIHM(m_ihmAddContact,m_ihmAddEtab);
	m_ihmRechEtab=new RechercherEtabIHM(m_ihmAddEtab);
	m_ihmRechContact=new RechercherContactIHM(m_ihmAddContact);
	m_ihmEditer=new EditerIHM(m_ihmRechEtab,m_ihmRechContact);
	m_ihmRechDemande=new RechercherDemandeIHM(m_ihmRechEtab,m_ihmRechContact);

        //Classe de controle mutualise
        m_ctlRechercher=cf.getRechercherCTL();

        try
        {
            jbInit();
        }
        catch(Exception e)
        {
            Tools.internalError(e,"",cf.debug(),false);
        }

        //lancement du timer de raffraichissement de la liste
        timer.schedule(new java.util.TimerTask()
        {
            public void run()
            {
                refreshDemandes();
            }

        },0,cf.getRefreshDelay()*1000);
    }

    private void jbInit() throws Exception
    {
        jMenu1.setText("Fichier");
        jMenu1.setMnemonic('f');
        jMenuItemRefresh.setText("Raffraichir   ");
        jMenuItemRefresh.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F5,0));
        jMenuItemRefresh.setMnemonic('r');
        jMenuItemRefresh.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                jMenuItemRefresh_actionPerformed(e);
            }
        });
        jMenuItemQuitter.setText("Quitter");
        jMenuItemQuitter.setMnemonic('q');
        jMenuItemQuitter.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X,ActionEvent.ALT_MASK));
        jMenuItemQuitter.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                jMenuItemQuitter_actionPerformed(e);
            }
        });
        jMenu2.setText("Edition");
        jMenu2.setMnemonic('e');
        jMenu3.setText("Aide");
        jMenu3.setMnemonic('a');
        jMenuItemHelp.setText("Index...");
        jMenuItemHelp.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F1,0));
        jMenuItemHelp.setMnemonic('i');
        jMenuItemHelp.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                jMenuItemHelp_actionPerformed(e);
            }
        });
        jMenuItemAPropos.setText("A propos");
        jMenuItemAPropos.setMnemonic('p');
        jMenuItemAPropos.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                jMenuItemAPropos_actionPerformed(e);
            }
        });
        jPanel1.setLayout(gridBagLayout1);
        jLabel1.setText("Demandes en cours :");
        jButtonNouveau.setText("Nouveau");
        jButtonNouveau.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                jButtonNouveau_actionPerformed(e);
            }
        });
        jButtonEditer.setText("Editer");
        jButtonEditer.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                jButtonEditer_actionPerformed(e);
            }
        });
        jButtonRechercher.setText("Rechercher");
        jButtonRechercher.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                jButtonRechercher_actionPerformed(e);
            }
        });
        this.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        this.setJMenuBar(jMenuBar1);
        this.setTitle("ECaMM");
        this.addWindowListener(new java.awt.event.WindowAdapter()
        {
            public void windowClosed(WindowEvent e)
            {
                this_windowClosed(e);
            }
        });
        jMenu4.setText("Nouveau");
        jMenu4.setMnemonic('n');
        jMenuItemNewContact.setText("Contact...");
        jMenuItemNewContact.setMnemonic('c');
        jMenuItemNewContact.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F9,0));
        jMenuItemNewContact.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                jMenuItemNewContact_actionPerformed(e);
            }
        });
        jMenuItemNewEtab.setText("Etablissement...");
        jMenuItemNewEtab.setMnemonic('e');
        jMenuItemNewEtab.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F8,0));
        jMenuItemNewEtab.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                jMenuItemNewEtab_actionPerformed(e);
            }
        });
        jMenuItemNewDem.setText("Demande...");
        jMenuItemNewDem.setMnemonic('d');
        jMenuItemNewDem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F4,0));
        jMenuItemNewDem.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                jMenuItemNewDem_actionPerformed(e);
            }
        });
        jMenuItemConfig.setText("Configuration ...");
        jMenuItemConfig.setMnemonic('c');
        jMenuItemConfig.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                jMenuItemConfig_actionPerformed(e);
            }
        });
        jMenu5.setText("Rechercher");
        jMenu5.setMnemonic('r');
        jMenuItemFindContact.setText("Contact...");
        jMenuItemFindContact.setMnemonic('c');
        jMenuItemFindContact.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F7,0));
        jMenuItemFindContact.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                jMenuItemFindContact_actionPerformed(e);
            }
        });
        jMenuItemFindEtab.setText("Etablissement...");
        jMenuItemFindEtab.setMnemonic('e');
        jMenuItemFindEtab.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F6,0));
        jMenuItemFindEtab.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                jMenuItemFindEtab_actionPerformed(e);
            }
        });
        jMenuItemFindDossier.setText("N dossier...");
        jMenuItemFindDossier.setMnemonic('n');
        jMenuItemFindDossier.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F11,0));
        jMenuItemFindDossier.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                jMenuItemFindDossier_actionPerformed(e);
            }
        });
        jMenuItemFind.setText("Demande...");
        jMenuItemFind.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F3,0));
        jMenuItemFind.setMnemonic('d');
        jMenuItemFind.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                jMenuItemFind_actionPerformed(e);
            }
        });
        jMenuBar1.add(jMenu1);
        jMenuBar1.add(jMenu2);
        jMenuBar1.add(jMenu3);
        jMenu2.add(jMenuItemRefresh);
        jMenu1.add(jMenu4);
        jMenu1.add(jMenuItemQuitter);
        jMenu3.add(jMenuItemHelp);
        jMenu3.add(jMenuItemAPropos);
        this.getContentPane().add(jPanel1, BorderLayout.CENTER);
        this.getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW)
            .put(KeyStroke.getKeyStroke(KeyEvent.VK_F2,0),ACTION_EDIT);
        this.getRootPane().getActionMap().put(ACTION_EDIT,new AbstractAction()
        {
            public void actionPerformed(ActionEvent e)
            {
                jButtonEditer_actionPerformed(e);
            }
        });
        jPanel1.add(jLabel1, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(6, 6, 0, 6), 0, 0));
        jPanel1.add(jScrollPane1, new GridBagConstraints(0, 1, 1, 1, 1.0, 1.0
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(6, 12, 0, 12), 0, 0));
        jPanel1.add(jPanel2, new GridBagConstraints(0, 2, 1, 1, 1.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(6, 6, 6, 6), 0, 0));
        jPanel2.add(jButtonNouveau, null);
        jPanel2.add(jButtonEditer, null);
        jPanel2.add(jButtonRechercher, null);
        jScrollPane1.getViewport().add(jListDemande, null);
        jMenu4.add(jMenuItemNewDem);
        jMenu4.add(jMenuItemNewEtab);
        jMenu4.add(jMenuItemNewContact);
        jMenu2.add(jMenu5);
        jMenu2.add(jMenuItemConfig);
        jMenu5.add(jMenuItemFind);
        jMenu5.add(jMenuItemFindEtab);
        jMenu5.add(jMenuItemFindContact);
        jMenu5.add(jMenuItemFindDossier);
        jListDemande.addMouseListener(new MouseAdapter()
        {
            public void mouseClicked(MouseEvent e)
            {
                jListDemande_mouseClicked(e);
            }
        });
        jListDemande.setCellRenderer(new MyCellRenderer());
        jListDemande.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        jListDemande.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER,0),ACTION_EDIT);
        jListDemande.getActionMap().put(ACTION_EDIT,new AbstractAction()
        {
            public void actionPerformed(ActionEvent e)
            {
                jButtonEditer_actionPerformed(e);
            }
        });
    }

    private void refreshDemandes()
    {
        jListDemande.setListData(m_ctlRechercher.getDemandes());

	//rend visible le dernier lment de la liste
	SwingUtilities.invokeLater(new Runnable() {
	    public void run() {
		jListDemande.ensureIndexIsVisible(jListDemande.getModel().getSize()-1);
	    }
	});
    }

    void jButtonNouveau_actionPerformed(ActionEvent e)
    {
        if(!m_ihmCreer1.isVisible())
            m_ihmCreer1.setVisible(true);
        else if(!m_ihmCreer2.isVisible())
            m_ihmCreer2.setVisible(true);
        else
            m_ihmCreer1.setVisible(true);
    }

    void jButtonEditer_actionPerformed(ActionEvent e)
    {
        if(!jListDemande.isSelectionEmpty())
        {
            m_ihmEditer.editer((DemandeDATA)jListDemande.getSelectedValue());
        }
        else
        {
            JOptionPane.showMessageDialog(this,"Vous devez slectionner une demande.",
                   "Erreur de saisie",JOptionPane.INFORMATION_MESSAGE);
        }
    }

    void jButtonRechercher_actionPerformed(ActionEvent e)
    {
        m_ihmRechDemande.setVisible(true);
    }

    void jMenuItemNewContact_actionPerformed(ActionEvent e)
    {
        m_ihmAddContact.setVisible(true);
    }

    void jMenuItemNewEtab_actionPerformed(ActionEvent e)
    {
        m_ihmAddEtab.setVisible(true);
    }

    void jMenuItemNewDem_actionPerformed(ActionEvent e)
    {
       jButtonNouveau_actionPerformed(null);
    }

    void jMenuItemConfig_actionPerformed(ActionEvent e)
    {
        new ConfigurationIHM();
    }

    void jMenuItemQuitter_actionPerformed(ActionEvent e)
    {
        this_windowClosed(null);
    }

    void jMenuItemRefresh_actionPerformed(ActionEvent e)
    {
        refreshDemandes();
    }

    void jMenuItemFind_actionPerformed(ActionEvent e)
    {
        jButtonRechercher_actionPerformed(null);
    }

    void jMenuItemFindContact_actionPerformed(ActionEvent e)
    {
	m_ihmRechContact.setVisible(true);
    }

    void jMenuItemFindEtab_actionPerformed(ActionEvent e)
    {
        m_ihmRechEtab.setVisible(true);
    }

    void jMenuItemFindDossier_actionPerformed(ActionEvent e)
    {
        int numDossier;

        String rep=JOptionPane.showInputDialog(this,"Numro du dossier :",
                "Recherche",JOptionPane.QUESTION_MESSAGE);

        if(rep!=null)   //si pas cancel
        {
            try
            {
                numDossier = Integer.parseInt(rep);
                if(numDossier<0) throw new NumberFormatException("Negative number not allowed");
            }
            catch(NumberFormatException ex)
            {
                if(cf.debug()) ex.printStackTrace();
                JOptionPane.showMessageDialog(null,"Vous devez saisir un entier positif !",
				"Erreur de saisie",
				JOptionPane.ERROR_MESSAGE);
                return;
            }

            //recherche dans les demandes en cours
            DemandeDATA dem = m_ctlRechercher.getDemande(numDossier);
            if(dem != null)
            {
                m_ihmEditer.editer(dem);
            }
            else  //si pas trouv : recherche dans les archives
            {
                m_ihmRechDemande.findByNumDossier(numDossier);
            }
        }
    }

    void jMenuItemHelp_actionPerformed(ActionEvent e)
    {
        //ouverture de l'aide
        new AideIHM();
    }


    void this_windowClosed(WindowEvent e)
    {
        //dverrouillage du lock
        m_ihmEditer.jButtonCancel_actionPerformed(null);

        //arret du timer
        timer.cancel();

        //sauvegarde des paramtres
        if(getExtendedState()!= Frame.MAXIMIZED_BOTH)
        {
            cf.setVisuSize(getSize());
            cf.setVisuLocation(getLocation());
        }
        if(m_ihmEditer.getExtendedState()!= Frame.MAXIMIZED_BOTH)
        {
            cf.setEditLocation(m_ihmEditer.getLocation());
            cf.setEditSize(m_ihmEditer.getSize());
        }
        if(m_ihmCreer1.getExtendedState()!= Frame.MAXIMIZED_BOTH)
        {
            cf.setCreerLocation(m_ihmCreer1.getLocation());
            cf.setCreerSize(m_ihmCreer1.getSize());
        }
        cf.setFulltextMode(m_ihmRechDemande.isFulltextMode());
        cf.save();
        if(cf.debug()) System.out.println("Program closed.");
        System.exit(0);
    }

    void jMenuItemAPropos_actionPerformed(ActionEvent e)
    {
        JOptionPane.showMessageDialog(this,
//GNU General Public Licence
"ECaMM version 2.1 ,  Copyleft (8) 2006  -  Nicolas AGIUS\n\n"+
"ECaMM comes with ABSOLUTELY NO WARRANTY. This is free\n"+
"software, and you are welcome to redistribute it under the terms\n"+
"of the GNU General Public License.\n"+
"   See the \"Help\" menu for details.",

            "A Propos",JOptionPane.INFORMATION_MESSAGE);
    }

    void jListDemande_mouseClicked(MouseEvent e)
    {
        if(e.getClickCount()==2)
            jButtonEditer_actionPerformed(null);
    }


    // Display an icon and a string for each object in the list.
    private class MyCellRenderer extends JLabel implements ListCellRenderer
    {
        private ImageIcon icon[] = new ImageIcon[3];

        public MyCellRenderer()
        {
            try
            {
                for(int i=0;i<icon.length;i++)
                {
                    icon[i] = new ImageIcon(getClass().getResource("rond"+((int)i+1)+".png"));
                }
            }
            catch(Exception e)
            {
                Tools.internalError(e,"Erreur lors du chargement des icones :\n",cf.debug(),false);
            }
        }

        // This is the only method defined by ListCellRenderer.
        // We just reconfigure the JLabel each time we're called.
        public Component getListCellRendererComponent(
                        JList list,
                        Object value,            // value to display
                        int index,               // cell index
                        boolean isSelected,      // is the cell selected
                        boolean cellHasFocus)    // the list and the cell have the focus
        {
            Color fgSelected,fgUnselected;
            DemandeDATA dem=(DemandeDATA)value;
            setText(value.toString());

	    //protection en cas d'erreur: etat=resolut dans demande (ou accs pendant update)
            if(dem.getEtat()==4)
                setIcon(null);
            else
                setIcon(icon[dem.getEtat()-1]);

            //Slection de la couleur en fonction de la priorit
            switch(dem.getPriorite())
            {
                case 1:     //priorit haute
                    fgSelected=new Color(255,120,120);  //rouge clair
                    fgUnselected=Color.red;
                    break;
                case 3:     //priorit basse
                    fgSelected=Color.green;
                    fgUnselected=new Color(0,189,0);    //vert fonc
                    break;
                default:     //priorit normale (cas 2)
                    fgSelected=list.getSelectionForeground();
                    fgUnselected=list.getForeground();
                    break;
            }

            if (isSelected) {
                setBackground(list.getSelectionBackground());
                setForeground(fgSelected);
            } else {
                setBackground(list.getBackground());
                setForeground(fgUnselected);
            }

            setEnabled(list.isEnabled());
            setFont(list.getFont());
            setOpaque(true);
            return this;
        }
    }
}
